<?php
/**
 * LeagueManager administration functions
 *
 */
 
/**
* Class to implement LeagueManager Administration panel
*
* @package LeagueManager
* @subpackage LeagueManagerAdmin
*/
final class LeagueManagerAdmin extends LeagueManager {
	/**
	 * current league ID
	 *
	 * @var int
	 */
	public $league_id = 0;
	
	
	/**
	 * Constructor
	 */
	public function __construct() {
		parent::__construct();
		
		require_once( ABSPATH . 'wp-admin/includes/template.php' );

		add_action('admin_enqueue_scripts', array(&$this, 'loadScripts') );
		add_action('admin_enqueue_scripts', array(&$this, 'loadStyles') );
		
		add_action( 'wp_dashboard_setup', array( $this, 'registerDashboardWidget') );
		add_action( 'admin_menu', array(&$this, 'menu') );

		// Add meta box to post screen
		add_action( 'publish_post', array(&$this, 'editMatchReport') );
		add_action( 'edit_post', array(&$this, 'editMatchReport') );
		add_action('add_meta_boxes', array(&$this, 'metaboxes'));
		
		add_action('wp_ajax_leaguemanager_get_season_dropdown', array(&$this, 'getSeasonDropdown'));
		add_action('wp_ajax_leaguemanager_get_match_dropdown', array(&$this, 'getMatchDropdown'));
		
		// add admin notices
		add_action('admin_notices', array(&$this, 'showAdminNotices'));
	}


	/**
	 * show admin notices about recommended plugins
	 */
	public function showAdminNotices() {
		$options = get_option('leaguemanager');
		$plugin_messages = array();
		
		// Save ignored notices
		if ( !isset($options['ignore_notices']) ) $options['ignore_notices'] = array();
		if ( isset($_GET['projectmanager_ignore_notice']) ) {
			$options['ignore_notices'][] = $_GET['projectmanager_ignore_notice'];
		}
		
		// Fancy Slideshows
		if( !is_plugin_active("sponsors-slideshow-widget/sponsors-slideshow-widget.php") ) {
			$plugin_messages['slideshows'] = sprintf(__( "You can extend LeagueManager with fancy slideshows by using the <a href='%s' target='_blank'>Fancy Slideshows Plugin</a>. (<a href='%s'>Ignore</a>)", 'leaguemanager' ), 'https://wordpress.org/plugins/sponsors-slideshow-widget/', admin_url()."index.php?projectmanager_ignore_notice=slideshows");
		}
		
		// ProjectManager
		if( !is_plugin_active("projectmanager/projectmanager.php") ) {
			$plugin_messages['projectmanager'] = sprintf(__( "You can extend LeagueManager with team rosters & profiles and match statistics by using the <a href='%s' target='_blank'>ProjectManager Plugin</a>. (<a href='%s'>Ignore</a>)", 'leaguemanager' ), 'https://wordpress.org/plugins/projectmanager/', admin_url()."index.php?projectmanager_ignore_notice=projectmanager");
		}
		
		// Display notifications
		if( count($plugin_messages) > 0 ) {
			foreach($plugin_messages as $key => $message) {
				if ( !in_array($key, $options['ignore_notices']) )
					echo '<div class="update-nag"><p><strong>'.$message.'</strong></p></div>';
			}
		}
		
		// update option with ignored notices
		update_option('leaguemanager', $options);
	}
	
	
	/**
	 * add menu to the admin interface
	 */
	public function menu() {
		$plugin = 'leaguemanager/leaguemanager.php';

		// keep capabilities here for next update
		$page = add_menu_page(__('LeagueManager','leaguemanager'), __('LeagueManager','leaguemanager'), 'league_manager', 'leaguemanager', array(&$this, 'display'), LEAGUEMANAGER_URL.'/admin/icons/cup_sw.png');
		add_action("admin_print_scripts-$page", array(&$this, 'loadScripts') );
		add_action("admin_print_scripts-$page", array(&$this, 'loadStyles') );

		$page = add_submenu_page('leaguemanager', __('LeagueManager', 'leaguemanager'), __('Overview','leaguemanager'),'league_manager', 'leaguemanager', array(&$this, 'display'));
		add_action("admin_print_scripts-$page", array(&$this, 'loadScripts') );
		add_action("admin_print_scripts-$page", array(&$this, 'loadStyles') );
		
		$page = add_submenu_page('leaguemanager', __('Settings', 'leaguemanager'), __('Settings','leaguemanager'),'leaguemanager_settings', 'leaguemanager-settings', array( $this, 'display' ));
		add_action("admin_print_scripts-$page", array(&$this, 'loadScripts') );
		add_action("admin_print_scripts-$page", array(&$this, 'loadStyles') );
					
		$page = add_submenu_page('leaguemanager', __('Import'), __('Import'),'import_leagues', 'leaguemanager-import', array( $this, 'display' ));
		add_action("admin_print_scripts-$page", array(&$this, 'loadScripts') );
		add_action("admin_print_scripts-$page", array(&$this, 'loadStyles') );
		
		$page = add_submenu_page('leaguemanager', __('Export'), __('Export'),'export_leagues', 'leaguemanager-export', array( $this, 'display' ));
		add_action("admin_print_scripts-$page", array(&$this, 'loadScripts') );
		add_action("admin_print_scripts-$page", array(&$this, 'loadStyles') );
		
		$page = add_submenu_page('leaguemanager', __('Documentation', 'leaguemanager'), __('Documentation','leaguemanager'), 'view_leagues', 'leaguemanager-doc', array( $this, 'display' ));
		add_action("admin_print_scripts-$page", array(&$this, 'loadScripts') );
		add_action("admin_print_scripts-$page", array(&$this, 'loadStyles') );

		add_filter( 'plugin_action_links_' . $plugin, array( &$this, 'pluginActions' ) );
	}
	

	/**
	 * adds the required Metaboxes
	 */
	public function metaboxes() {
		add_meta_box( 'leaguemanager', __('Match-Report','leaguemanager'), array(&$this, 'addMetaBox'), 'post' );
	}

	
	/**
	* Register LeagueManager Dashboard Widget
	*
	*/
	public static function registerDashboardWidget() {
		wp_add_dashboard_widget(
			'leaguemanager_dashboard',
			__('LeagueManager Latest Support News', 'leaguemanager'),
			array(
				'LeagueManagerAdmin',
				'latestSupportNews'
			)
		);
	}


	/**
	 * Get latest news from LeagueManager Support on WordPress.org
	 *
	 * @return string
	 */
	public static function latestSupportNews() {
		$options = get_option('leaguemanager');
			
		echo '<div class="rss-widget">';

		wp_widget_rss_output(array(
			'url'          => 'http://wordpress.org/support/rss/plugin/leaguemanager',
			'title'        => __('Latest LeagueManager support discussions...', 'leaguemanager'),
			'show_author' => $options['dashboard_widget']['show_author'],
			'show_date' => $options['dashboard_widget']['show_date'],
			'show_summary' => $options['dashboard_widget']['show_summary'],
			'items' => $options['dashboard_widget']['num_items']
		));

		echo '</div>';
	}
	
	
	/**
	 * build league menu
	 *
	 * @return array
	 */
	private function getMenu() {
		$league = get_league();

		$menu = array(
			'settings' => array( 'title' => __('Preferences', 'leaguemanager'), 'callback' => array(&$this, 'displaySettingsPage'), 'cap' => 'edit_league_settings' ),
			'seasons' => array( 'title' => __('Seasons', 'leaguemanager'), 'callback' => array(&$this, 'displaySeasonsPage'), 'cap' => 'edit_seasons' ),
			'team' => array( 'title' => __('Add Team', 'leaguemanager'), 'callback' => array(&$this, 'displayTeamPage'), 'cap' => 'edit_teams' ),
			'match' => array( 'title' => __('Add Matches', 'leaguemanager'), 'callback' => array(&$this, 'displayMatchPage'), 'cap' => 'edit_matches' )
		);
		
		/**
		 * Fires when league menu is constructed to add sports specific items
		 *
		 * @param array $menu
		 * @param int $league_id
		 * @param string $season
		 * @return array
		 * @category wp-filter
		 */
		$menu = apply_filters('league_menu_'.$league->sport, $menu, $league->id, $league->current_season['name']);
		/**
		 * Fires when league menu is constructed to add mode specific items
		 *
		 * @param array $menu
		 * @param int $league_id
		 * @param string $season
		 * @return array
		 * @category wp-filter
		 */
		$menu = apply_filters('league_menu_'.$league->mode, $menu, $league->id, $league->current_season['name']);

		return $menu;
	}


	/**
	 * showMenu() - show admin menu
	 *
	 */
	public function display() {
		global $league;

		$options = get_option('leaguemanager');

		// Update Plugin Version
		if ( $options['version'] != LEAGUEMANAGER_VERSION ) {
			$options['version'] = LEAGUEMANAGER_VERSION;
			update_option('leaguemanager', $options);
		}

		// Update database
		if( $options['dbversion'] != LEAGUEMANAGER_DBVERSION ) {
			include_once ( dirname (__FILE__) . '/upgrade.php' );
			leaguemanager_upgrade_page();
			return;
		}

		switch ($_GET['page']) {
			case 'leaguemanager-doc':
				include_once( dirname(__FILE__) . '/documentation.php' );
				break;
			case 'leaguemanager-settings':
				$this->displayGlobalOptionsPage();
				break;
			case 'leaguemanager-import':
				$this->displayImportPage();
				break;
			case 'leaguemanager-export':
				$this->displayExportPage();
				break;
			case 'leaguemanager':
			default:
				if ( isset($_GET['subpage']) ) {
					$this->league_id = intval($_GET['league_id']);
					$league = get_league($this->league_id);
					
					$menu = $this->getMenu();
					$page = htmlspecialchars($_GET['subpage']);
					if ( array_key_exists( $page, $menu ) ) {
						if ( isset($menu[$page]['callback']) )//&& is_callable(array($this, $menu[$page]['callback'])) )
							call_user_func_array($menu[$page]['callback'], array());
						else
							include_once( $menu[$page]['file'] );
					} else {
						$this->displayMainPage();
					}
				} else {
					$this->displayIndexPage();
				}

				break;
		}
	}


	/**
	 * show LeagueManager index page
	 *
	 */
	private function displayIndexPage() {
		global $league;
		
		if ( !current_user_can( 'view_leagues' ) ) {
			echo '<div class="error"><p style="text-align: center;">'.__("You do not have sufficient permissions to access this page.").'</p></div>';
		} else {
			if ( isset($_POST['addLeague']) || isset($_POST['doaction']) ) {
				if ( current_user_can('edit_leagues') ) {
					if ( isset($_POST['addLeague']) ) {
						check_admin_referer('leaguemanager_add-league');
						$this->addLeague( htmlspecialchars($_POST['league_title']) );
						//$this->printMessage();
					}
					if ( isset($_POST['doaction']) ) {
						if ( $_POST['action'] == 'delete' ) {
							if ( current_user_can('del_leagues') ) {
								check_admin_referer('leagues-bulk');
								foreach ( $_POST['league'] AS $league_id )
									$this->delLeague( intval($league_id) );
							} else {
								$this->setMessage(__("You don't have permission to perform this task", 'leaguemanager'), true);
							}
						}
					}
				} else {
					$this->setMessage(__("You don't have permission to perform this task", 'leaguemanager'), true);
				}
				$this->printMessage();
			}
			
			/*
			 * test number of queries
			 */
			/*
			global $wpdb;
			$num_queries = $wpdb->num_queries;
			$league = get_league(2);
			$m = $league->getMatches();
			$c = $league->getCrosstableField(52,53);
			$num_queries2 = $wpdb->num_queries;
			echo "<p>Number of queries: ".$num_queries."</p>";
			echo "<p>Number of queries: ".$num_queries2."</p>";
			echo "<p>Run queries: ".($num_queries2-$num_queries)."</p>";
			*/
			
			include_once( dirname(__FILE__) . '/index.php' );
		}
	}
	
	
	/**
	 * display league overview page
	 *
	 */
	private function displayMainPage() {
		global $league;
		
		if ( !current_user_can( 'view_leagues' ) ) {
			echo '<div class="error"><p style="text-align: center;">'.__("You do not have sufficient permissions to access this page.").'</p></div>';
		} else {
			$league = get_league();
			$options = get_option('leaguemanager');
			
			$tab = 0;
			if ( isset($_POST['updateLeague']) && !isset($_POST['doaction-teams']) && !isset($_POST['doaction-matches']) && !isset($_POST['doaction-match_day']) )  {
				if ( 'team' == $_POST['updateLeague'] ) {
					if ( current_user_can('edit_teams') ) {
						check_admin_referer('leaguemanager_manage-teams');
						
						$home = isset( $_POST['home'] ) ? 1 : 0;
						$custom = !isset($_POST['custom']) ? array() : $_POST['custom'];
						$roster =  isset($_POST['roster']) ? intval($_POST['roster']) : 0;
						$profile = isset($_POST['profile']) ? intval($_POST['profile']) : 0;
						$group = isset($_POST['group']) ? htmlspecialchars(strip_tags($_POST['group'])) : '';
								
						if ( $_POST['team_id'] == 0 ) {
							$this->addTeam( intval($_POST['league_id']), htmlspecialchars(strip_tags($_POST['season'])), htmlspecialchars(strip_tags($_POST['team'])), htmlspecialchars(strip_tags($_POST['website'])), htmlspecialchars($_POST['coach']), htmlspecialchars($_POST['stadium']), $home, $group, $roster, $profile, $custom, htmlspecialchars($_POST['logo_db']) );
						} else {			
							$del_logo = isset( $_POST['del_logo'] ) ? true : false;
							$overwrite_image = isset( $_POST['overwrite_image'] ) ? true: false;
							
							$this->editTeam( intval($_POST['team_id']), htmlspecialchars(strip_tags($_POST['team'])), htmlspecialchars(strip_tags($_POST['website'])), htmlspecialchars($_POST['coach']), htmlspecialchars($_POST['stadium']), $home, $group, $roster, $profile, $custom, htmlspecialchars($_POST['logo_db']), $del_logo, $overwrite_image );
						}
					} else {
						$this->setMessage(__("You don't have permission to perform this task", 'leaguemanager'), true);
					}
				} elseif ( 'match' == $_POST['updateLeague'] ) {
					check_admin_referer('leaguemanager_manage-matches');
					
					if ( current_user_can('edit_matches') ) {
						if ( 'add' == $_POST['mode'] ) {
							$num_matches = count($_POST['match']);
							foreach ( $_POST['match'] AS $i => $match_id ) {
								if ( isset($_POST['add_match'][$i]) || $_POST['away_team'][$i] != $_POST['home_team'][$i]  ) {
									$index = ( isset($_POST['mydatepicker'][$i]) ) ? $i : 0;
									$date = $_POST['mydatepicker'][$index].' '.$_POST['begin_hour'][$i].':'.$_POST['begin_minutes'][$i].':00';
									$match_day = ( isset($_POST['match_day'][$i]) ? $_POST['match_day'][$i] : (!empty($_POST['match_day']) ? intval($_POST['match_day']) : '' )) ;
									$group = htmlspecialchars(strip_tags($_POST['group'][$i]));
									$custom = isset($_POST['custom']) ? $_POST['custom'][$i] : array();

									$this->addMatch( $date, $_POST['home_team'][$i], $_POST['away_team'][$i], $match_day, htmlspecialchars(strip_tags($_POST['location'][$i])), intval($_POST['league_id']), htmlspecialchars(strip_tags($_POST['season'])), $group, htmlspecialchars(strip_tags($_POST['final'])), $custom );
								} else {
									$num_matches -= 1;
								}
							}
							$this->setMessage(sprintf(_n('%d Match added', '%d Matches added', $num_matches, 'leaguemanager'), $num_matches));
						} else {
							$num_matches = count($_POST['match']);
							$post_match = $this->htmlspecialchars_array($_POST['match']);
							foreach ( $post_match AS $i => $match_id ) {
								if( isset($_POST['mydatepicker'][$i]) ) {
									$index = ( isset($_POST['mydatepicker'][$i]) ) ? $i : 0;
									$date = htmlspecialchars(strip_tags($_POST['mydatepicker'][$index])).' '.$_POST['begin_hour'][$i].':'.$_POST['begin_minutes'][$i].':00';
								} else {
									$index = ( isset($_POST['year'][$i]) && isset($_POST['month'][$i]) && isset($_POST['day'][$i]) ) ? $i : 0;
									$date = intval($_POST['year'][$index]).'-'.intval($_POST['month'][$index]).'-'.intval($_POST['day'][$index]).' '.$_POST['begin_hour'][$i].':'.$_POST['begin_minutes'][$i].':00';
								}
								
								$match_day = (isset($_POST['match_day']) && is_array($_POST['match_day'])) ? intval($_POST['match_day'][$i]) : (isset($_POST['match_day']) && !empty($_POST['match_day']) ? intval($_POST['match_day']) : '' ) ;
								$custom = isset($_POST['custom']) ? $_POST['custom'][$i] : array();
								$home_team = isset($_POST['home_team'][$i]) ? htmlspecialchars(strip_tags($_POST['home_team'][$i])) : '';
								$away_team = isset($_POST['away_team'][$i]) ? htmlspecialchars(strip_tags($_POST['away_team'][$i])) : '';
								$group = htmlspecialchars(strip_tags($_POST['group'][$i]));
								
								$this->editMatch( $date, $home_team, $away_team, $match_day, htmlspecialchars($_POST['location'][$i]), htmlspecialchars($_POST['status'][$i]), intval($_POST['league_id']), $match_id, $group, htmlspecialchars(strip_tags($_POST['final'])), $custom );
							}
							$this->setMessage(sprintf(_n('%d Match updated', '%d Matches updated', $num_matches, 'leaguemanager'), $num_matches));
						}
					} else {
						$this->setMessage(__("You don't have permission to perform this task", 'leaguemanager'), true);
					}
				} elseif ( 'results' == $_POST['updateLeague'] ) {
					if ( current_user_can('update_results') ) {
						check_admin_referer('matches-bulk');
						$custom = isset($_POST['custom']) ? $_POST['custom'] : array();
						$this->updateResults( intval($_POST['league_id']), $_POST['matches'], $_POST['home_points'], $_POST['away_points'], $_POST['home_team'], $_POST['away_team'], $_POST['status'], $custom );
					} else {
						$this->setMessage(__("You don't have permission to perform this task", 'leaguemanager'), true);
					}
					$tab = 2;
					$league->match_day = intval($_POST['current_match_day']);
				} elseif ( 'teams_manual' == $_POST['updateLeague'] ) {
					if ( current_user_can('update_results') ) {
						check_admin_referer('teams-bulk');
						$league->saveStandingsManually( $_POST['team_id'], $_POST['points_plus'], $_POST['points_minus'], $_POST['num_done_matches'], $_POST['num_won_matches'], $_POST['num_draw_matches'], $_POST['num_lost_matches'], $_POST['add_points'], $_POST['custom'] );

						$this->setMessage(__('Standings Table updated','leaguemanager'));
					} else {
						$this->setMessage(__("You don't have permission to perform this task", 'leaguemanager'), true);
					}
				}
				
				$this->printMessage();
			}  elseif ( isset($_POST['doaction-teams']) || isset($_POST['doaction-matches']) ) {
				if ( isset($_POST['doaction-teams']) && $_POST['action-teams'] == "delete" ) {
					if ( current_user_can('del_teams') ) {
						check_admin_referer('teams-bulk');
						foreach ( $_POST['team'] AS $team_id )
							$this->delTeam( intval($team_id), true );
					} else {
						$this->setMessage(__("You don't have permission to perform this task", 'leaguemanager'), true);
					}
				} elseif ( isset($_POST['doaction-matches']) && $_POST['action-matches'] == "delete" ) {
					if ( current_user_can('del_matches') ) {
						check_admin_referer('matches-bulk');
						foreach ( $_POST['match'] AS $match_id )
							$this->delMatch( intval($match_id) );
					} else {
						$this->setMessage(__("You don't have permission to perform this task", 'leaguemanager'), true);
					}
					
					$tab = 2;
				}
				
				$this->printMessage();
			}

			// rank teams manually
			if (isset($_POST['updateRanking'])) {
				if ( current_user_can('update_results') ) {
					$js = ( $_POST['js-active'] == 1 ) ? true : false;
					
					$team_ranks = array();
					$team_ids = array_values($_POST['team_id']);
					foreach ($team_ids AS $key => $team_id) {
						if ( $js ) {
							$rank = $key + 1;
						} else {
							$rank = intval($_POST['rank'][$team_id]);
						}
						
						$team = $league->getTeam($team_id);
						$team_ranks[$rank-1] = $team;
					}
					ksort($team_ranks);
					$team_ranks = $league->getRanking($team_ranks);
					$league->updateRanking($team_ranks);
					$this->setMessage(__('Team ranking saved','leaguemanager'));
				} else {
					$this->setMessage(__("You don't have permission to perform this task", 'leaguemanager'), true);
				}
				$this->printMessage();
				
				$tab = 0;
			}

			// check if league is a cup championship
			$cup = ( $league->mode == 'championship' );
			$team_id = isset($_POST['team_id']) ? intval($_POST['team_id']) : false;

			$match_args = array("final" => "", "cache" => false);

			if ( $team_id )
				$match_args['team_id'] = $team_id;

			// set tab to matches
			if ( isset($_GET['match_paged']) ) 
				$tab = 2;
			
			if ( isset($_POST['doaction-match_day'])) {
				if ($_POST['match_day'] != -1) {
					$matchDay = intval($_POST['match_day']);
					$league->setMatchDay($matchDay);
					//$match_args["match_day"] = $matchDay;
				}
				$tab = 2;
			} else {
				if ( $league->match_display == 'current_match_day' )
					$league->setMatchDay('current');
				elseif ( $league->match_display == 'all' )
					$league->setMatchDay(-1);
			}
			
			if ( empty($league->seasons)  ) {
				$this->setMessage( __( 'You need to add at least one season', 'leaguemanager' ), true );
				$this->printMessage();
			}

			// get teams and matches
			if ( $league->mode != 'championship' ) {
				$teams = $league->getTeams();
				$matches = $league->getMatches( $match_args );
				//$league->setNumMatches();
			}

			// get specific standings table
			if ( isset($_GET['standingstable']) ) {
				$get = $_GET['standingstable'];
				$match_day = -1;
				$mode = 'all';
				if ( preg_match('/match_day-\d/', $get, $hits) ) {
					$res = explode("-", $hits[0]);
					$match_day = $res[1];
				} elseif ( in_array($get, array('home', 'away')) ) {
					$mode = htmlspecialchars($get);
				}
				$teams = $league->getStandings( $teams, $match_day, $mode );
			}

			if ( !wp_mkdir_p( $this->getFilePath() ) )
			  echo '<div class="error"><p>'.sprintf( __( 'Unable to create directory %s. Is its parent directory writeable by the server?' ), $this->getFilePath() ).'</p></div>';
			
			include_once( dirname(__FILE__) . '/show-league.php' );
		}
	}
	
	
	/**
	 * display league settings page
	 *
	 */
	private function displaySettingsPage() {
		global $projectmanager;
		
		if ( !current_user_can( 'edit_league_settings' ) ) {
			echo '<div class="error"><p style="text-align: center;">'.__("You do not have sufficient permissions to access this page.").'</p></div>';
		} else {
			$tab = 0;

			$league = get_league();
			if ( isset($_POST['updateSettings']) ) {
				check_admin_referer('leaguemanager_manage-league-options');

				$this->editLeague( htmlspecialchars($_POST['league_title']), (array)$_POST['settings'], intval($_POST['league_id']) );
				$this->printMessage();
				
				$options = get_option('leaguemanager');
				$league->reloadSettings();
				
				// Set active tab
				$tab = intval($_POST['active-tab']);
			}
			
			$forwin = $fordraw = $forloss = $forwin_overtime = $forloss_overtime = 0;
			// Manual point rule
			if ( is_array($league->point_rule) ) {
				$forwin = $league->point_rule['forwin'];
				$forwin_overtime = $league->point_rule['forwin_overtime'];
				$fordraw = $league->point_rule['fordraw'];
				$forloss = $league->point_rule['forloss'];
				$forloss_overtime = $league->point_rule['forloss_overtime'];
				$league->point_rule_data = $league->point_rule;
				$league->point_rule = 'user';
			}
			
			if ( isset($_GET['regenerate_thumbnails']) ) {
				$this->regenerateThumbnails();
				$tab = 1;
			}
			
			if ( isset($_GET['cleanUnusedFiles']) ) {
				$this->cleanUnusedMediaFiles();
				$tab = 1;
			}
			
			$menu_page_url = menu_page_url('leaguemanager', 0) . "&amp;subpage=settings&amp;league_id=".$league->id."&amp;season=".htmlspecialchars($_GET['season'])."&amp;group=".htmlspecialchars($_GET['group']);
			
			include_once( dirname(__FILE__) . '/settings.php' );
		}
	}
	
	
	/**
	 * display seasons administration page
	 *
	 */
	private function displaySeasonsPage() {
		if ( !current_user_can( 'edit_seasons' ) ) {
			echo '<div class="error"><p style="text-align: center;">'.__("You do not have sufficient permissions to access this page.").'</p></div>';
		} else {
			if ( isset($_POST['saveSeason']) ) {
				if ( !empty($_POST['season']) ) {
					if ( empty($_POST['season_id']) ) {
						$add_teams = isset($_POST['no_add_teams']) ? false : true;
						$this->saveSeason( htmlspecialchars($_POST['season']), intval($_POST['num_match_days']), $add_teams );
					} else {
						$this->saveSeason( htmlspecialchars($_POST['season']), intval($_POST['num_match_days']), false, $_POST['season_id'] );
					}
				} else {
					$this->setMessage( __( 'Season was empty', 'leaguemanager' ), true );
				}
				$this->printMessage();
			} elseif ( isset($_POST['doaction']) ) {
				if ( current_user_can('del_seasons') ) {
					check_admin_referer('seasons-bulk');
					if ( 'delete' == $_POST['action'] ) {
						$this->delSeasons( $_POST['del_season'], intval($_GET['league_id']) );
					}
				} else {
					$this->setMessage(__("You don't have permission to perform this task", 'leaguemanager'), true);
				}
				$this->printMessage();
			}

			$league = get_league();
			$season_id = false;
			$season_data = array('name' => '', 'num_match_days' => '');
			if ( isset($_GET['edit']) ) {
				$season_id = htmlspecialchars($_GET['edit']);
				$season_data = $league->seasons[$season_id];
			}
			
			include_once( dirname(__FILE__) . '/seasons.php' );
		}
	}
	
	
	/**
	 * display team editing page
	 *
	 */
	private function displayTeamPage() {
		global $projectmanager;
		
		if ( !current_user_can( 'edit_teams' ) ) {
			echo '<div class="error"><p style="text-align: center;">'.__("You do not have sufficient permissions to access this page.").'</p></div>';
		} else {
			$league = get_league();
			
			if ( isset( $_GET['edit'] ) ) {
				$form_title = __( 'Edit Team', 'leaguemanager' );
				$team = get_team(intval($_GET['edit']));
			} else {
				$form_title = __( 'Add Team', 'leaguemanager' );
				$team = new Team(); // new empty team
			}

			if ( !wp_mkdir_p( $this->getFilePath() ) )
				echo "<div class='error'><p>".sprintf( __( 'Unable to create directory %s. Is its parent directory writable by the server?' ), $this->getFilePath() )."</p></div>";
			
			include_once( dirname(__FILE__) . '/team.php' );
		}
	}
	
	
	/**
	 * display match editing page
	 *
	 */
	private function displayMatchPage() {
		global $wpdb;
		
		if ( !current_user_can( 'edit_matches' ) ) {
			echo '<div class="error"><p style="text-align: center;">'.__("You do not have sufficient permissions to access this page.").'</p></div>';
		} else {
			$league = get_league();
			
			//$non_group = (isset($league->non_group) ? $league->non_group : 0);
			$is_finals = $finalkey = false;
			$class = 'alternate';

			$matches = array();
			if ( isset( $_GET['edit'] ) ) {
				$mode = 'edit';
				$form_title  = $submit_title = __( 'Edit Match', 'leaguemanager' );

				$matches[0] = get_match(intval($_GET['edit']));
				$league->setMatchDay($matches[0]->match_day);

				$max_matches = 1;
			} elseif ( isset($_GET['match_day']) ) {
				$mode = 'edit';

				$league->setMatchDay(intval($_GET['match_day']));
				$league->setSeason(htmlspecialchars($_GET['season']));
			
				$form_title = sprintf(__( 'Edit Matches &#8211; %d. Match Day', 'leaguemanager' ), $league->match_day);
				$submit_title = __('Edit Matches', 'leaguemanager');

				$matches = $league->getMatches( array("limit" => false) );
				$max_matches = count($matches);
			} elseif ( isset($_GET['final']) ) {
				$is_finals = true;
				$mode = htmlspecialchars($_GET['mode']);
				$finalkey = $league->championship->getCurrentFinalKey();
				$max_matches = $league->championship->getNumMatches($finalkey);

				if ( 'add' == $mode ) {
					$form_title = $submit_title = sprintf(__( 'Add Matches &#8211; %s', 'leaguemanager' ), $league->championship->getFinalname($finalkey));
					for ( $i = 0; $i < $max_matches; $i++ ) {
						$matches[$i] = new Match();
						$matches[$i]->hour = $league->default_match_start_time['hour'];
						$matches[$i]->minutes = $league->default_match_start_time['minutes'];
					}
				} else {
					$form_title = $submit_title = sprintf(__( 'Edit Matches &#8211; %s', 'leaguemanager' ), $league->championship->getFinalname($finalkey));
					$matches = $league->getMatches(  array("final" => $finalkey, "limit" => false) );
				}
			} else {
				$mode = 'add';

				$form_title = $submit_title = __( 'Add Matches', 'leaguemanager' );
				$max_matches = ceil($league->num_teams_total/2); // set number of matches to add to half the number of teams

				// Simply limit the number of matches to add to 50
				if ($max_matches > 50) $max_matches = 50;

				for ( $i = 0; $i < $max_matches; $i++ ) {
					$matches[$i] = new Match();
					$matches[$i]->hour = $league->default_match_start_time['hour'];
					$matches[$i]->minutes = $league->default_match_start_time['minutes'];
				}
			}

			if ( $is_finals ) {
				/*if ( is_numeric($matches[0]->home_team) && is_numeric($matches[0]->away_team) ) {
					$team_args = array("league_id" => $league->id, "season" => $season['name'], "orderby" => array("title" => "ASC"));
					$teams = $leaguemanager->getTeams( $team_args );
				} else {
					$teams = $championship->getFinalTeams( $finalkey );
				}*/
				$teams = $league->championship->getFinalTeams( $finalkey );
				//$teamsHome = $teams;
			} else {
				// make sure to get all teams irrespective of group
				$teams = $league->getTeams( array("limit" => false, "group" => "", "orderby" => array("title" => "ASC")) );
			}
			
			include_once( dirname(__FILE__) . '/match.php' );
		}
	}
	
	
	/**
	 * display export page
	 *
	 */
	private function displayExportPage() {
		if ( !current_user_can( 'export_leagues' ) ) {
			echo '<div class="error"><p style="text-align: center;">'.__("You do not have sufficient permissions to access this page.").'</p></div>';
		} else {
			if ( isset($_POST['leaguemanager_export']) ) {
				$options = get_option('leaguemanager');
				if ($_POST['exportkey'] ==	$options['exportkey']) {
					ob_end_clean();
					$this->export(intval($_POST['league_id']), $_POST['mode']);
					unset($options['exportkey']);
					update_option('leaguemanager', $options);
				} else {
					ob_end_flush();
					$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
					$this->printMessage();
				}
			}

			$options = get_option('leaguemanager');
			$options['exportkey'] = uniqid(rand(), true);
			update_option('leaguemanager', $options);
			
			include_once( dirname(__FILE__) . '/export.php' );
		}
	}
	
	
	/**
	 * display import page
	 *
	 */
	private function displayImportPage() {
		if ( !current_user_can( 'import_leagues' ) ) {
			echo '<div class="error"><p style="text-align: center;">'.__("You do not have sufficient permissions to access this page.").'</p></div>';
		} else {
			if ( isset($_POST['import']) ) {
				check_admin_referer('leaguemanager_import-datasets');
				$this->import( intval($_POST['league_id']), $_FILES['leaguemanager_import'], htmlspecialchars($_POST['delimiter']), htmlspecialchars($_POST['mode']) );
				$this->printMessage();
			}
			
			include_once( dirname(__FILE__) . '/import.php' );
		}
	}
	
	
	/**
	 * display link to settings page in plugin table
	 *
	 * @param array $links array of action links
	 * @return array
	 */
	public function pluginActions( $links ) {
		$settings_link = '<a href="admin.php?page=leaguemanager-settings">' . __('Settings') . '</a>';
		array_unshift( $links, $settings_link );

		return $links;
	}


	/**
	 * load Javascript
	 *
	 */
	public function loadScripts() {
		wp_register_script( 'leaguemanager-functions', LEAGUEMANAGER_URL.'/admin/js/functions.js', array( 'thickbox', 'jquery', 'jquery-ui-core', 'jquery-ui-datepicker', 'jquery-ui-tabs', 'jquery-ui-accordion', 'jquery-ui-sortable', 'jquery-ui-tooltip', 'jquery-effects-core', 'jquery-effects-slide', 'jquery-effects-explode', 'scriptaculous-dragdrop', 'iris' ), LEAGUEMANAGER_VERSION );
		wp_enqueue_script('leaguemanager-functions');
		
		wp_register_script( 'leaguemanager-ajax', LEAGUEMANAGER_URL.'/admin/js/ajax.js', array('sack'), LEAGUEMANAGER_VERSION );
		wp_enqueue_script('leaguemanager-ajax');

		?>
		<script type='text/javascript'>
		<!--<![CDATA[-->
		LeagueManagerAjaxL10n = {
			requestUrl: "<?php bloginfo( 'wpurl' ); ?>/wp-admin/admin-ajax.php", manualPointRuleDescription: "<?php _e( 'Order: win, win overtime, tie, loss, loss overtime', 'leaguemanager' ) ?>", pluginUrl: "<?php echo LEAGUEMANAGER_URL; ?>", Edit: "<?php _e("Edit"); ?>", Post: "<?php _e("Post"); ?>", Save: "<?php _e("Save"); ?>", Cancel: "<?php _e("Cancel"); ?>", pleaseWait: "<?php _e("Please wait..."); ?>", Delete: "<?php _e('Delete', 'leaguemanager') ?>", Yellow: "<?php _e( 'Yellow', 'leaguemanager') ?>", Red: "<?php _e( 'Red', 'leaguemanager') ?>", Yellow_Red: "<?php _e('Yellow/Red', 'leaguemanager') ?>", Insert: "<?php _e( 'Insert', 'leaguemanager' ) ?>", InsertPlayer: "<?php _e( 'Insert Player', 'leaguemanager' ) ?>", AddPlayerFromRoster: "<?php _e( 'Add Player from Team Roster', 'leaguemanager' ) ?>"
		}
		<!--]]>-->
		</script>
		<?php
	}


	/**
	 * load CSS styles
	 *
	 */
	public function loadStyles() {
		wp_register_style('leaguemanager', LEAGUEMANAGER_URL . "/admin/style.css", false, '1.0', 'screen');
		wp_enqueue_style('leaguemanager');
		
		wp_register_style('jquery-ui', LEAGUEMANAGER_URL . "/css/jquery/jquery-ui.min.css", false, '1.11.4', 'all');
		wp_register_style('jquery-ui-structure', LEAGUEMANAGER_URL . "/css/jquery/jquery-ui.structure.min.css", array('jquery-ui'), '1.11.4', 'all');
		wp_register_style('jquery-ui-theme', LEAGUEMANAGER_URL . "/css/jquery/jquery-ui.theme.min.css", array('jquery-ui', 'jquery-ui-structure'), '1.11.4', 'all');
		
		wp_enqueue_style('jquery-ui-structure');
		wp_enqueue_style('jquery-ui-theme');
		
		//wp_register_style('jquery_ui_css', 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.9.2/themes/smoothness/jquery-ui.css', false, '1.0', 'screen');
		//wp_enqueue_style('jquery_ui_css');
		wp_enqueue_style('thickbox');
	}


	/**
	 * get available league modes
	 *
	 * @return array
	 */
	public function getModes() {
		$modes = array( 'default' => __('Default', 'leaguemanager') );
		/**
		 * Fired when league modes are built
		 *
		 * @param array $modes
		 * @return array
		 * @category wp-filter
		 */
		$modes = apply_filters( 'leaguemanager_modes', $modes);
		return $modes;
	}


	/**
	 * get array of supported point rules
	 *
	 * @return array
	 */
	public function getPointRules() {
		$rules = array( 'manual' => __( 'Update Standings Manually', 'leaguemanager' ), 'one' => __( 'One-Point-Rule', 'leaguemanager' ), 'two' => __('Two-Point-Rule','leaguemanager'), 'three' => __('Three-Point-Rule', 'leaguemanager'), 'score' => __( 'Score', 'leaguemanager'), 'user' => __('User defined', 'leaguemanager') );

		/**
		 * Fired when league point rules are built
		 *
		 * @param array $rules
		 * @return array
		 * @category wp-filter
		 */
		$rules = apply_filters( 'leaguemanager_point_rules_list', $rules );
		asort($rules);

		return $rules;
	}


	/**
	 * get available point formats
	 *
	 * @return array
	 */
	public function getPointFormats() {
		$point_formats = array( '%d:%d' => '%d:%d', '%d - %d' => '%d - %d', '%d' => '%d', '%.1f:%.1f' => '%f:%f', '%.1f - %.1f' => '%f - %f', '%.1f' => '%f' );
		/**
		 * Fired when league point formats are built
		 *
		 * @param array $point_formats
		 * @return array
		 * @category wp-filter
		 */
		$point_formats = apply_filters( 'leaguemanager_point_formats', $point_formats );
		return $point_formats;
	}


	/**
	 * update match results
	 *
	 * @param int $league_id
	 * @param array $matches
	 * @param array $home_points
	 * @param array $away_points
	 * @param array $home_team
	 * @param array $away_team
	 * @param array $status
	 * @param array $custom
	 * @param boolean $final
	 * @param boolean $message
	 * @return boolean
	 */
	private function updateResults( $league_id, $matches, $home_points, $away_points, $home_team, $away_team, $status, $custom, $final = false, $message = true ) {
		if ( !current_user_can('update_results') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$league = get_league();
		$league->setFinals($final);
		$league->_updateResults($matches, $home_points, $away_points, $home_team, $away_team, $status, $custom);
		
		if ( $message )
			$this->setMessage( sprintf(__('Updated Results of %d matches','leaguemanager'), count($matches)) );
		
		
		return true;
	}

	
/************
*
*   LEAGUE SECTION
*
*
*/

	/**
	 * add new League
	 *
	 * @param string $title
	 * @return boolean
	 */
	private function addLeague( $title ) {
		global $wpdb;
		
		if ( !current_user_can('edit_leagues') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}

		$settings = array( 'standings' => array('pld' => 1, 'won' => 1, 'tie' => 1, 'lost' => 1, 'winPercent' => 1, 'last5' => 1) );
		$wpdb->query( $wpdb->prepare ( "INSERT INTO {$wpdb->leaguemanager} (title, settings, seasons) VALUES ('%s', '%s', '%s')", $title, maybe_serialize($settings), '') );
		$league_id = $wpdb->insert_id;
		
		$league = get_league( $league_id );
		// create upload directory
		wp_mkdir_p( $this->getFilePath('League-'.$league_id) );
		
		$this->setMessage( __('League added', 'leaguemanager') );
		
		/**
		 * Fired when a new league is added
		 *
		 * @param int $league_id
		 * @category wp-action
		 */
		do_action('add_league', $league_id);
		
		return true;
	}


	/**
	 * edit League settings
	 *
	 * @param string $title
	 * @param array $settings
	 * @param int $league_id
	 * @return boolean
	 */
	private function editLeague( $title, $settings, $league_id ) {
		global $wpdb;
		
		if ( !current_user_can('edit_league_settings') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		if ( $settings['point_rule'] == 'user' && isset($_POST['forwin']) && is_numeric($_POST['forwin']) )
			$settings['point_rule'] = array( 'forwin' => intval($_POST['forwin']), 'fordraw' => intval($_POST['fordraw']), 'forloss' => intval($_POST['forloss']), 'forwin_overtime' => intval($_POST['forwin_overtime']), 'forloss_overtime' => intval($_POST['forloss_overtime']) );

		$settings['num_matches_per_page'] = intval($settings['num_matches_per_page']);
		$settings['tiny_size'] = array( 'width' => intval($settings['tiny_size']['width']), 'height' => intval($settings['tiny_size']['height']) );
		$settings['thumb_size'] = array( 'width' => intval($settings['thumb_size']['width']), 'height' => intval($settings['thumb_size']['height']) );
		$settings['large_size'] = array( 'width' => intval($settings['large_size']['width']), 'height' => intval($settings['large_size']['height']) );
		$settings['crop_image']['tiny'] = isset($settings['crop_image']['tiny']) ? 1 : 0;
		$settings['crop_image']['thumb'] = isset($settings['crop_image']['thumb']) ? 1 : 0;
		$settings['crop_image']['large'] = isset($settings['crop_image']['large']) ? 1 : 0;
		
		foreach ( $this->getStandingsDisplayOptions() AS $key => $label )
			$settings['standings'][$key] = isset($settings['standings'][$key]) ? 1 : 0;
		
		/*$settings['standings']['logos'] = isset($settings['standings']['logos']) ? 1 : 0;
		$settings['standings']['team_link'] = isset($settings['standings']['team_link']) ? 1 : 0;
		$settings['standings']['pld'] = isset($settings['standings']['pld']) ? 1 : 0;
		$settings['standings']['won'] = isset($settings['standings']['won']) ? 1 : 0;
		$settings['standings']['tie'] = isset($settings['standings']['tie']) ? 1 : 0;
		$settings['standings']['lost'] = isset($settings['standings']['lost']) ? 1 : 0;
		$settings['standings']['winPercent'] = isset($settings['standings']['winPercent']) ? 1 : 0;
		$settings['standings']['last5'] = isset($settings['standings']['last5']) ? 1 : 0;*/
		
		$settings['num_ascend'] = intval($settings['num_ascend']);
		$settings['num_descend'] = intval($settings['num_descend']);
		$settings['num_relegation'] = intval($settings['num_relegation']);
		
		$settings['slideshow']['show_logos'] = ( isset($settings['slideshow']['show_logos']) && $settings['slideshow']['show_logos'] == 1 ) ? 1 : 0;
		$settings['slideshow']['num_matches'] = intval($settings['slideshow']['num_matches']);
		
		// teamroster and profile
		if ( isset($settings['teamroster']) )
			$settings['teamroster'] = intval($settings['teamroster']);
		if ( isset($settings['teamprofiles']) )
			$settings['teamprofiles'] = intval($settings['teamprofiles']);
		if ( isset($settings['matchstats']) )
			$settings['matchstats'] = intval($settings['matchstats']);
		
		// Set sport-specific textdomain
		$options = get_option('leaguemanager');
		$options['textdomain'] = (string)$settings['sport'];
		update_option('leaguemanager', $options);
		
		$wpdb->query( $wpdb->prepare ( "UPDATE {$wpdb->leaguemanager} SET `title` = '%s', `settings` = '%s' WHERE `id` = '%d'", $title, maybe_serialize($settings), $league_id ) );
		$this->setMessage( __('Settings saved', 'leaguemanager') );
		
		/**
		 * Fired when league settings are saved
		 *
		 * @param int $league_id
		 * @category wp-action
		 */
		do_action('edit_league_settings', $league_id);
		
		return true;
	}


	/**
	 * delete League
	 *
	 * @param int $league_id
	 * @return boolean
	 */
	private function delLeague( $league_id ) {
		global $wpdb, $league;

		if ( !current_user_can('del_leagues') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$league = get_league($league_id);
		// Delete Teams and with it Matches
		foreach ( $league->getTeams() AS $team ) {
			$this->delTeam( $team->id );
		}

		// remove remaining matches
		$wpdb->query( $wpdb->prepare("DELETE FROM {$wpdb->leaguemanager_matches} WHERE `league_id` = '%d'", $league_id) );
		
		// remove statistics
		$wpdb->query( $wpdb->prepare("DELETE FROM {$wpdb->leaguemanager_stats} WHERE `league_id` = '%d'", $league_id) );
		$wpdb->query( $wpdb->prepare("DELETE FROM {$wpdb->leaguemanager} WHERE `id` = '%d'", $league_id) );
		
		// Delete league subfolder
		$this->removeDir($league->getImagePath());
		
		/**
		 * Fired when league is deleted
		 *
		 * @param int $league_id
		 * @category wp-action
		 */
		do_action('del_league', $league_id);
		
		return true;
	}


	/**
	 * add new season to league
	 *
	 * @param string $season
	 * @param int $num_match_days
	 * @param boolean $add_teams
	 * @param string $key
	 * @return boolean
	 */
	private function saveSeason( $season, $num_match_days, $add_teams = false, $key = false ) {
		global $wpdb;

		if ( !current_user_can('edit_seasons') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$league = get_league();
		if ( $add_teams && !empty($league->seasons) && !$key ) {
			$last_season = end($league->seasons);
			if ( !empty($last_season) ) {
				if ( $teams = $league->getTeams(array("season" => $last_season['name'], "cache" => false)) ) {
					foreach ( $teams AS $team ) {
						$this->addTeamFromDB( $league->id, $season, $team->id, false );
					}
				}
			}
		}

		if ( $key ) {
			$key = htmlspecialchars($key);
			if ( $teams = $league->getTeams( array("season" => $key) ) ) {
				foreach ( $teams AS $team ) {
					$wpdb->query( $wpdb->prepare("UPDATE {$wpdb->leaguemanager_teams} SET `season` = '%s' WHERE `id` = '%d'", $season, $team->id) );
				}
			}
			if ( $matches = $league->getMatches( array("season" => $key, "limit" => false, "cache" => false, "reset_query_args" => true) ) ) {
				foreach ( $matches AS $match ) {
					$wpdb->query( $wpdb->prepare("UPDATE {$wpdb->leaguemanager_matches} SET `season` = '%s' WHERE `id` = '%d'", $season, $match->id) );
				}
			}
		}

		// unset broken season, due to delete bug
		if ( $key && $key != $season )
			unset($league->seasons[$key]);

		$league->seasons[$season] = array( 'name' => $season, 'num_match_days' => $num_match_days );
		ksort($league->seasons);
		$wpdb->query( $wpdb->prepare("UPDATE {$wpdb->leaguemanager} SET `seasons` = '%s' WHERE `id` = '%d'", maybe_serialize($league->seasons), $league->id) );

		if ( $key )
			$this->setMessage( sprintf(__('Season <strong>%s</strong> modified','leaguemanager'), $season ) );
		else
			$this->setMessage( sprintf(__('Season <strong>%s</strong> added','leaguemanager'), $season ) );
		
		/**
		 * Fired when season is saved
		 *
		 * @param string|false $key
		 * @category wp-action
		 */
		do_action('save_season', $key);
		
		return true;
	}


	/**
	 * delete season of league
	 *
	 * @param array $seasons
	 * @param int $league_id
	 * @return boolean
	 */
	private function delSeasons( $seasons, $league_id ) {
		global $wpdb;
		
		$league = get_league();

		if ( !current_user_can('del_seasons') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		if ( !empty($seasons) ) {
			foreach ( $seasons AS $key ) {
				$season = $league->seasons[$key];

				// Delete teams and matches if there are any
				if ( $teams = $league->getTeams(array("season" => $season['name'], "cache" => false)) ) {
					foreach ( $teams AS $team )
						$this->delTeam($team->id);
				}

				unset($league->seasons[$key]);
			}
			$wpdb->query( $wpdb->prepare("UPDATE {$wpdb->leaguemanager} SET `seasons` = '%s' WHERE `id` = '%d'", maybe_serialize($league->seasons), $league->id) );
		}
		
		return true;
	}


/************
*
*   TEAM SECTION
*
*
*/

	/**
	 * add new team
	 *
	 * @param int $league_id
	 * @param mixed $season
	 * @param string $title
	 * @param string $website
	 * @param string $coach
	 * @param string $stadium
	 * @param int $home 1 | 0
	 * @param mixed $group
	 * @param int|array $roster
	 * @param int $profile
	 * @param array $custom
	 * @param string $logo (optional)
	 * @param boolean $message (optional)
	 * @return int|false
	 */
	private function addTeam( $league_id, $season, $title, $website, $coach, $stadium, $home, $group, $roster, $profile, $custom, $logo = '', $message = true ) {
		global $wpdb;

		if ( !current_user_can('edit_teams') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$sql = "INSERT INTO {$wpdb->leaguemanager_teams} (`title`, `website`, `coach`, `stadium`, `home`, `group`, `roster`, `profile`, `season`, `custom`, `logo`, `league_id`) VALUES ('%s', '%s', '%s', '%s', '%d', '%s', '%d', '%d', '%s', '%s', '%s', '%d')";
		$wpdb->query( $wpdb->prepare ( $sql, $title, $website, $coach, $stadium, $home, $group, $roster, $profile, $season, maybe_serialize($custom), basename($logo), $league_id ) );
		$team_id = $wpdb->insert_id;

		if ( isset($_FILES['logo']) && $_FILES['logo']['name'] != '' )
			$this->uploadLogo($team_id, $_FILES['logo']);

		if ( $message )
			$this->setMessage( __('Team added','leaguemanager') );

		/**
		 * Fired when new team is added
		 *
		 * @param int $team_id
		 * @category wp-action
		 */
		do_action('add_team', $team_id);
		
		return $team_id;
	}


	/**
	 * add new team with data from existing team
	 *
	 * @param int $league_id
	 * @param string $season
	 * @param int $team_id
	 * @param boolean $message (optional)
	 */
	private function addTeamFromDB( $league_id, $season, $team_id, $message = false ) {
		global $wpdb;
		
		$team = get_team($team_id);
		$new_team_id = $this->addTeam($league_id, $season, $team->title, $team->website, $team->coach, $team->stadium, $team->home, $team->group, $team->roster, $team->profile, maybe_unserialize($team->custom), $team->logo, $message);
	}


	/**
	 * edit team
	 *
	 * @param int $team_id
	 * @param string $title
	 * @param string $website
	 * @param string $coach
	 * @param string $stadium
	 * @param int $home 1 or 0
	 * @param mixed $group
	 * @param int|array $roster
	 * @param int $profile
	 * @param array $custom
	 * @param string $logo
	 * @param boolean $del_logo
	 * @param boolean $overwrite_image
	 * @return boolean
	 */
	private function editTeam( $team_id, $title, $website, $coach, $stadium, $home, $group, $roster, $profile, $custom, $logo, $del_logo = false, $overwrite_image = false ) {
		global $wpdb;

		if ( !current_user_can('edit_teams') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$wpdb->query( $wpdb->prepare ( "UPDATE {$wpdb->leaguemanager_teams} SET `title` = '%s', `website` = '%s', `coach` = '%s', `stadium` = '%s', `logo` = '%s', `home` = '%d', `group` = '%s', `roster`= '%d', `profile` = '%d', `custom` = '%s' WHERE `id` = %d", $title, $website, $coach, $stadium, basename($logo), $home, $group, $roster, $profile, maybe_serialize($custom), $team_id ) );

		// Delete Image if options is checked
		if ($del_logo || $overwrite_image) {
			$this->delLogo( $logo, $team_id );
		}

		if ( isset($_FILES['logo']) && $_FILES['logo']['name'] != '' )
			$this->uploadLogo($team_id, $_FILES['logo'], $overwrite_image);

		$this->setMessage( __('Team updated','leaguemanager') );
		
		/**
		 * Fired when team is modified
		 *
		 * @param int $team_id
		 * @category wp-action
		 */
		do_action('edit_team', $team_id);
		
		return true;
	}


	/**
	 * delete Team
	 *
	 * @param int $team_id
	 * @return boolean
	 */
	private function delTeam( $team_id ) {
		global $wpdb;

		if ( !current_user_can('del_teams') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$team = get_team($team_id);
		$wpdb->query( $wpdb->prepare("DELETE FROM {$wpdb->leaguemanager_matches} WHERE `home_team` = '%d' OR `away_team` = '%d'", $team_id, $team_id) );
		$wpdb->query( $wpdb->prepare("DELETE FROM {$wpdb->leaguemanager_teams} WHERE `id` = '%d'", $team_id) );
		
		$this->delLogo( $team->logo );
		
		/**
		 * Fired when team is deleted
		 *
		 * @param int $team_id
		 * @category wp-action
		 */
		do_action('del_team', $team_id);
		
		return true;
	}


	/**
	 * display dropdon menu of teams (cleaned from double entries)
	 *
	 */
	private function teamsDropdownCleaned() {
		global $wpdb;
		$all_teams = $wpdb->get_results( "SELECT `title`, `id` FROM {$wpdb->leaguemanager_teams} ORDER BY `title` ASC" );
		$teams = array();
		foreach ( $all_teams AS $team ) {
			if ( !in_array($team->title, $teams) ) {
				$name = htmlspecialchars(stripslashes($team->title), ENT_QUOTES);
				$teams[$team->id] = $name;
				echo "<option value='".$team->id."'>".$name."</option>";
			}
		}	
	}


	/**
	 * crop image
	 *
	 * @param string $imagepath
	 * @param array $dest_size
	 * @param string $size
	 * @param boolean $crop
	 * @param boolean $force_resize
	 * @return the image url
	 */
	private function resizeImage ( $imagepath, $dest_size, $size, $crop = false, $force_resize = false ) {	
		$league = get_league();
		$options = get_option('leaguemanager');
		
		// load image editor
		$image = wp_get_image_editor( $imagepath );
		$imageurl = $league->getImageURL( $imagepath );

		// editor will return an error if the path is invalid - save original image url
		if ( is_wp_error( $image ) ) {
			return $imageurl;
		} else {
			// create destination file name
			$destination_file = $league->getImagePath($imagepath, $size);
			$this->destination_file = $destination_file;

			// resize only if the image does not exists
			if ( !file_exists($destination_file) || $force_resize ) {			
				// resize image, optionally with cropping enabled
				$image->resize( $dest_size['width'], $dest_size['height'], $crop );
				// save image
				$saved = $image->save( $destination_file );
				// return original url if an error occured
				 if ( is_wp_error( $saved ) ) {
					return $imageurl;
				}
			}
			
			$new_img_url = dirname($imageurl) . '/' . basename($destination_file);
			
			// record resized images with key using md5 hash of path to original image
			if ( isset($saved) && !in_array(basename($new_img_url), $options['resized_images'][$league->id][md5($imagepath)]) )
				$options['resized_images'][$league->id][md5($imagepath)][] = basename($new_img_url);
			
			update_option('leaguemanager', $options);
			
			return esc_url($new_img_url);
		}
	}
	
	
	/**
	 * Create different thumbnail sizes
	 *
	 * @param string $filename
	 * @param boolean $force_resize
	 */
	private function createThumbnails($filename, $force_resize = false) {
		$league = get_league();
		
		$options = get_option('leaguemanager');
		/*
		 * create resized image records
		 */
		if ( !isset($options['resized_images']) )
			$options['resized_images'] = array();
	
		if ( !isset($options['resized_images'][$league->id]) ) {
			$options['resized_images'][$league->id] = array();
		}
		
		if ( !isset($options['resized_images'][$league->id][md5($filename)]) ) {
			$options['resized_images'][$league->id][md5($filename)] = array();
		}
		update_option('leaguemanager', $options);
				
		// create different thumbnails
		$sizes = array( 'tiny' => $league->tiny_size, 'thumb' => $league->thumb_size, 'large' => $league->large_size );
		foreach ( $sizes AS $size => $dest_size ) {
			$crop = ( $league->crop_image[$size] == 1 ) ? true : false;
			$imageurl = $this->resizeImage( $filename, $dest_size, $size, $crop, $force_resize );
		}
	}
	
	
	/**
	 * get supported file types
	 *
	 * @return array
	 */
	private function getSupportedImageTypes() {
		return array( "jpg", "jpeg", "png", "gif" );
	}
	

	/**
	 * check if image type is supported
	 *
	 * @param string $image
	 * @return boolean
	 */
	private function isSupportedImage( $image ) {
		if ( in_array($this->getImageType($image), $this->getSupportedImageTypes()) )
			return true;
		
		return false;
	}
	
	
	/**
	 * get image type of supplied image
	 *
	 * @param string $image
	 * @return string
	 */
	private function getImageType( $image ) {
		$league = get_league();
		$file = $league->getImagePath($image);
		$file_info = pathinfo($file);
		return strtolower($file_info['extension']);
	}
	
	
	/**
	 * regenerate all thumbnails of current project
	 *
	 */
	private function regenerateThumbnails() {
		global $wpdb;
		
		$league = get_league();
		
		/*
		 * regenerate dataset image thumbnails
		 */
		$teams = $league->getTeams(array("limit"=>false, "season" => "any", "group" => "", "cache" => false));
		foreach ( $teams AS $team ) {
			if ( $team->logo != "" ) {
				$this->createThumbnails($league->getImagePath($team->logo), true);
			}
		}
	}
	
	
	/**
	 * list or remove unused media files
	 *
	 */
	private function cleanUnusedMediaFiles() {
		global $wpdb;
		
		$league = get_league();
		
		$dir = $league->getImagePath();
		$files = array_diff(scandir($dir), array('.','..'));
		$img_sizes = array( 'tiny', 'thumb', 'large' );
		// get all thumbnail images
		$thumbs = array();
		foreach ( $img_sizes AS $size ) {
			$thumbs = array_merge($thumbs, preg_grep("/".$size."\_/", $files));
		}
		// remove thumbnail images from filelist
		$files = array_diff($files, $thumbs);
		
		// check if file is used
		foreach ( $files AS $key => $file ) {
			$num = $wpdb->get_var( $wpdb->prepare("SELECT COUNT(ID) FROM {$wpdb->leaguemanager_teams} WHERE `logo` = '%s'", basename($file)) );
			if ( $num > 0 ) {
				$file_info = pathinfo( $league->getImagePath($file) );
				// remove file from list
				unset($files[$key]);
				// remove fancy slideshow widget thumbnails
				$files = array_diff($files, preg_grep("/".str_replace(".{$file_info['extension']}", "", basename($file))."-\d+x\d+.+/", $files));
			}
			
			$num = $wpdb->get_var( $query = $wpdb->prepare( "SELECT COUNT(ID) FROM {$wpdb->leaguemanager_teams} WHERE `logo` = '%s'", $league->getImageURL(basename($file)) ) );
			if ( $num > 0 ) {
				$file_info = pathinfo( $league->getImagePath($file) );
				// remove file from list
				unset($files[$key]);
				// remove fancy slideshow widget thumbnails
				$files = array_diff($files, preg_grep("/".str_replace(".{$file_info['extension']}", "", basename($file))."-\d+x\d+.+/", $files));
			}
		}
		
		if ( isset($_POST['delete_unused_files']) ) {
			check_admin_referer('leaguemanager_delete-unused-media-files');
			
			foreach ( $files AS $file ) {
				$this->delLogo($file);
			}
			echo "<div class='box success updated fade'>";
			echo "<p>".__( 'Unused media files deleted', 'leaguemanager' )."</p>";
			echo "</div>";
		} else {
			echo "<div class='box fade'>";
			if ( count($files) == 0 ) {
				echo "<p>".__( 'Congratulations! This league has no orphaned media files.', 'leaguemanager' )."</p>";
			} else {
				echo "<p>".__( 'The following files do not do not seem to be used. If this is true you can subsequently delete them.', 'leaguemanager' )."</p>";
				echo "<ul>";
				foreach ( $files AS $file ) {
					echo "<li style='margin-left: 2em;'>".$league->getImagePath($file)."</li>";
				}
				echo "</ul>";
				echo "<form action='' method='post'>";
				echo "<input type='submit' class='button-primary' value='".__('Delete unused media files', 'leaguemanager')."' />";
				wp_nonce_field( 'leaguemanager_delete-unused-media-files' ); 
				echo "<input type='hidden' name='delete_unused_files' value='yes' />";
				echo "</form>";
			}
			echo "</div>";
		}
	}
	
	
	/**
	 * set image path in database and upload image to server
	 *
	 * @param int $team_id
	 * @param string $file
	 * @param boolean $overwrite
	 * @return boolean
	 */
	private function uploadLogo( $team_id, $file, $overwrite = false ) {
		global $wpdb;

		$league = get_league();
		$new_file = $league->getImagePath( basename($file['name']) );
		$info = pathinfo( $new_file );
		// make sure that file extension is lowercase
		$new_file = str_replace($info['extension'], strtolower($info['extension']), $new_file);
		
		if ( $this->isSupportedImage($new_file) ) {
			if ( $file['size'] > 0 ) {
				if ( file_exists($new_file) && !$overwrite ) {
					$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->leaguemanager_teams} SET `logo` = '%s' WHERE id = '%d'", basename($file['name']), $team_id ) );
					$this->setMessage( __('Logo exists and is not uploaded. Set the overwrite option if you want to replace it.','leaguemanager'), true );
				} else {
					if ( move_uploaded_file($file['tmp_name'], $new_file) ) {
						$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->leaguemanager_teams} SET `logo` = '%s' WHERE id = '%d'", basename($file['name']), $team_id ) );
						
						$team = get_team( $team_id );
						$this->delLogo($team->logo);

						$this->createThumbnails($new_file, $overwrite);
						
						return true;
					} else {
						$this->setMessage( sprintf( __('The uploaded file could not be moved to %s.' ), $league->getImagePath() ), true );
					}
				}
			}
		} else {
			$this->setMessage( __('The file type is not supported.','leaguemanager'), true );
		}
		
		return false;
	}


	/**
	 * delete logo from server
	 *
	 * @param string $image
	 * @param int $team_id
	 */
	private function delLogo( $image, $team_id = false ) {
		global $wpdb;
		
		// Unset logo in database if team is provided - for editTeam()
		if ( $team_id ) $wpdb->query( $wpdb->prepare("UPDATE {$wpdb->leaguemanager_teams} SET `logo` = '' WHERE `id` = '%d'", $team_id) );
		
		$league = get_league();
		$num = $wpdb->get_var( $wpdb->prepare("SELECT COUNT(ID) FROM {$wpdb->leaguemanager_teams} WHERE `logo` = '%s'", basename($image)) );
		if ( $num == 0 ) {
			$sizes = array( 'tiny', 'thumb', 'large', 'full' );
			foreach ($sizes AS $size) {
				@unlink( $league->getImagePath($image, $size) );
			}
		}
	}

/************
*
*   MATCH SECTION
*
*
*/

	/**
	 * add Match
	 *
	 * @param string $date
	 * @param int $home_team
	 * @param int $away_team
	 * @param int $match_day
	 * @param string $location
	 * @param int $league_id
	 * @param mixed $season
	 * @param mixed $group
	 * @param string $final
	 * @param array $custom
	 * @return int|false
	 */
	private function addMatch( $date, $home_team, $away_team, $match_day, $location, $league_id, $season, $group, $final, $custom ) {
	 	global $wpdb;
		
		if ( !current_user_can('edit_matches') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$sql = "INSERT INTO {$wpdb->leaguemanager_matches} (date, home_team, away_team, match_day, location, league_id, season, final, custom, `group`, `status`) VALUES ('%s', '%s', '%s', '%d', '%s', '%d', '%s', '%s', '%s', '%s', 'scheduled')";
		$wpdb->query ( $wpdb->prepare ( $sql, $date, $home_team, $away_team, $match_day, $location, $league_id, $season, $final, maybe_serialize($custom), $group ) );
		$match_id = $wpdb->insert_id;
		
		/**
		 * Fired when new match is added
		 *
		 * @param int $team_id
		 * @category wp-action
		 */
		do_action('add_match', $match_id);
		
		return $match_id;
	}


	/**
	 * edit Match
	 *
	 * @param string $date
	 * @param int $home_team
	 * @param int $away_team
	 * @param int $match_day
	 * @param string $location
	 * @param string $status
	 * @param int $league_id
	 * @param int $match_id
	 * @param mixed $group
	 * @param string $final
	 * @param array $custom
	 * @return boolean
	 */
	private function editMatch( $date, $home_team, $away_team, $match_day, $location, $status, $league_id, $match_id, $group, $final, $custom ) {
	 	global $wpdb;
		
		if ( !current_user_can('edit_matches') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$this->league_id = $league_id;
		$home_points = (!isset($home_points)) ? '' : $home_points;
		$away_points = (!isset($away_points)) ? '' : $away_points;

		$match = $wpdb->get_results( $wpdb->prepare("SELECT `custom` FROM {$wpdb->leaguemanager_matches} WHERE `id` = '%d'", $match_id) );
		$custom = (!empty($match) ? array_merge( (array)maybe_unserialize($match[0]->custom), $custom ) : '' );
		$wpdb->query( $wpdb->prepare ( "UPDATE {$wpdb->leaguemanager_matches} SET `date` = '%s', `home_team` = '%s', `away_team` = '%s', `match_day` = '%d', `location` = '%s', `status` = '%s',`league_id` = '%d', `group` = '%s', `final` = '%s', `custom` = '%s' WHERE `id` = %d", $date, $home_team, $away_team, $match_day, $location, $status, $league_id, $group, $final, maybe_serialize($custom), $match_id ) );
		
		/**
		 * Fired when match is modified
		 *
		 * @param int $team_id
		 * @category wp-action
		 */
		do_action('edit_match', $match_id);
		
		return true;
	}


	/**
	 * delete Match
	 *
	 * @param int $match_id
	 * @return boolean
	 */
	private function delMatch( $match_id ) {
	  	global $wpdb;
		
		if ( !current_user_can('del_matches') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$wpdb->query( $wpdb->prepare("DELETE FROM {$wpdb->leaguemanager_matches} WHERE `id` = '%d'", $match_id) );
		
		/**
		 * Fired when team is deleted
		 *
		 * @param int $team_id
		 * @category wp-action
		 */
		do_action('del_match', $match_id);
		
		return true;
	}


	/**
	 * get date selection.
	 *
	 * @param int $day
	 * @param int $month
	 * @param int $year
	 * @param int $index default 0
	 * @return string
	 */
	public function getDateSelection( $day, $month, $year, $index = 0 ) {
		$out = '<select size="1" name="day['.$index.']" class="date">';
		$out .= "<option value='00'>".__('Day','leaguemanager')."</option>";
		for ( $d = 1; $d <= 31; $d++ ) {
			$selected = ( $d == $day ) ? ' selected="selected"' : '';
			$out .= '<option value="'.str_pad($d, 2, 0, STR_PAD_LEFT).'"'.$selected.'>'.$d.'</option>';
		}
		$out .= '</select>';
		$out .= '<select size="1" name="month['.$index.']" class="date">';
		$out .= "<option value='00'>".__('Month','leaguemanager')."</option>";
		foreach ( $this->getMonths() AS $key => $m ) {
			$selected = ( $key == $month ) ? ' selected="selected"' : '';
			$out .= '<option value="'.str_pad($key, 2, 0, STR_PAD_LEFT).'"'.$selected.'>'.$m.'</option>';
		}
		$out .= '</select>';
		$out .= '<select size="1" name="year['.$index.']" class="date">';
		$out .= "<option value='0000'>".__('Year','leaguemanager')."</option>";
		for ( $y = date("Y")-20; $y <= date("Y")+10; $y++ ) {
			$selected =  ( $y == $year ) ? ' selected="selected"' : '';
			$out .= '<option value="'.$y.'"'.$selected.'>'.$y.'</option>';
		}
		$out .= '</select>';
		return $out;
	}


	/**
	 * display global settings page (e.g. color scheme options)
	 *
	 */
	public function displayGlobalOptionsPage() {
		if ( !current_user_can( 'manage_leaguemanager' ) ) {
			echo '<p style="text-align: center;">'.__("You do not have sufficient permissions to access this page.").'</p>';
		} else {
			$options = get_option('leaguemanager');

			$tab = 0;
			if ( isset($_POST['updateLeagueManager']) ) {
				check_admin_referer('leaguemanager_manage-global-league-options');
				$options['colors']['headers'] = htmlspecialchars($_POST['color_headers']);
				$options['colors']['rows'] = array( 'alternate' => htmlspecialchars($_POST['color_rows_alt']), 'main' => htmlspecialchars($_POST['color_rows']), 'ascend' => htmlspecialchars($_POST['color_rows_ascend']), 'descend' => htmlspecialchars($_POST['color_rows_descend']), 'relegation' => htmlspecialchars($_POST['color_rows_relegation']) );
				$options['colors']['boxheader'] = array(htmlspecialchars($_POST['color_boxheader1']), htmlspecialchars($_POST['color_boxheader2']));
				$options['dashboard_widget']['num_items'] = intval($_POST['dashboard']['num_items']);
				$options['dashboard_widget']['show_author'] = isset($_POST['dashboard']['show_author']) ? 1 : 0;
				$options['dashboard_widget']['show_date'] = isset($_POST['dashboard']['show_date']) ? 1 : 0;
				$options['dashboard_widget']['show_summary'] = isset($_POST['dashboard']['show_summary']) ? 1 : 0;
				
				update_option( 'leaguemanager', $options );
				$this->setMessage(__( 'Settings saved', 'leaguemanager' ));
				$this->printMessage();
				
				// Set active tab
				$tab = intval($_POST['active-tab']);
			}

			require_once (dirname (__FILE__) . '/settings-global.php');
		}
	}


	/**
	 * add meta box to post screen
	 *
	 * @param object $post
	 */
	public function addMetaBox( $post ) {
		global $wpdb, $post_ID;

		if ( $leagues = $this->getLeagues() ) {
			$league_id = $match_id = 0;
			$match = false;
			if ( $post->ID != 0 ) {
				$match = $wpdb->get_row( $wpdb->prepare("SELECT `id`, `league_id`, `season` FROM {$wpdb->leaguemanager_matches} WHERE `post_id` = '%d'", $post->ID) );

				if ( $match ) {
					$match_id = $match->id;
					$league_id = $match->league_id;
					$curr_league = get_league($league_id);
					$curr_league->setSeason($match->season);
				}
			}

			echo "<input type='hidden' name='curr_match_id' value='".$match_id."' />";
			echo "<select name='league_id' class='alignleft' id='league_id' onChange='Leaguemanager.getSeasonDropdown(\"".get_bloginfo( 'wpurl' )."/wp-admin/admin-ajax.php\", this.value)'>";
			echo "<option value='0'>".__('Choose League','leaguemanager')."</option>";
			foreach ( $leagues AS $league ) {
				echo "<option value='".$league->id."' ".selected($league_id, $league->id, false).">".$league->title."</option>";
			}
			echo "</select>";

			echo "<div id='seasons'>";
			if ( $match )
				echo $curr_league->getSeasonDropdown($curr_league->getSeason());
			echo '</div>';
			echo "<div id='matches'>";
			if ( $match )
				echo $curr_league->getMatchesDropdown($match->id);
			echo '</div>';

			echo '<br style="clear: both;" />';
		}
	}


	/**
	 * update post id for match report
	 *
	 */
	public function editMatchReport() {
		global $wpdb;

		if (isset($_POST['post_ID'])) {
			$post_ID = (int) $_POST['post_ID'];
			$match_ID = isset($_POST['match_id']) ? (int) $_POST['match_id'] : false;
			$curr_match_ID = isset($_POST['curr_match_id']) ? (int) $_POST['curr_match_id'] : false;
			
			if ( $match_ID && $curr_match_ID != $match_ID ) {
				$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->leaguemanager_matches} SET `post_id` = '%d' WHERE `id` = '%d'", $post_ID, $match_ID ) );
				if ( $curr_match_ID != 0 )
					$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->leaguemanager_matches} SET `post_id` = 0 WHERE `id` = '%d'", $curr_match_ID ) );
			}
		}
	}


	/**
	 * Upload file to webserver
	 * 
	 * @param array $file
	 * @param boolean $overwrite
	 * @return string|false
	 */
	private function uploadFile( $file, $overwrite = false ) {
		$league = get_league($this->league_id);
		
		$new_file = $league->getFilePath($file['name']);
		$info = pathinfo( $new_file );
		// make sure that file extension is lowercase
		$new_file = str_replace($info['extension'], strtolower($info['extension']), $new_file);
		
		if ( file_exists($new_file) && !$overwrite ) {
			$this->setMessage( __('File exists and is not uploaded. Set the overwrite option if you want to replace it.','leaguemanager'), true );
		} else {
			if ( !move_uploaded_file($file['tmp_name'], $new_file) ) {
				$this->setMessage( sprintf( __('The uploaded file could not be moved to %s.' ), $league->getFilePath() ), true );
			} else {
				return $new_file;
			}
		}
		
		return false;
	}
	
	
	/**
	 * import data from CSV file
	 *
	 * @param int $league_id
	 * @param array $file CSV file
	 * @param string $delimiter
	 * @param array $what 'teams' | 'matches'
	 * @return void|false
	 */
	private function import( $league_id, $file, $delimiter, $what ) {
		if ( !current_user_can('import_leagues') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		if ( $file['size'] > 0 ) {
			$this->league_id = intval($league_id);
			
			/*
			* Upload file to image directory, temporarily
			*/
			$new_file =  $this->getFilePath($file['name']);
			if ( move_uploaded_file($file['tmp_name'], $new_file) ) {
				if ( 'teams' == $what )
					$this->importTeams($new_file, $delimiter);
				elseif ( 'matches' == $what )
					$this->importMatches($new_file, $delimiter);
				elseif ( 'media' == $what )
					$this->importMedia($new_file);
			} else {
				$this->setMessage(sprintf( __('The uploaded file could not be moved to %s.' ), $this->getFilePath()) );
			}
			@unlink($new_file); // remove file from server after import is done
		} else {
			$this->setMessage( __('The uploaded file seems to be empty', 'leaguemanager'), true );
		}
	}


	/**
	 * import media files
	 *
	 * @param string $file
	 */
	private function importMedia( $file ) {
		$league = get_league($this->league_id);
		if (file_exists($file)) {
			if ($this->unzipFiles($file, $league->getFilePath())) {
				$this->setMessage(__('Media file have been successfully imported', 'leaguemanager'));
				// remove zip file
				@unlink($file);
			} else {
				$this->setMessage(__('Media zip file could not be unpacked','leaguemanager'), true);
			}
		}
	}
	
	
	/**
	 * import teams from CSV file
	 *
	 * @param string $file
	 * @param string $delimiter
	 * @return void|false
	 */
	private function importTeams( $file, $delimiter ) {
		if ( !current_user_can('import_leagues') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$handle = @fopen($file, "r");
		if ($handle && current_user_can('manage_leaguemanager')) {
			$league = get_league( $this->league_id );
			if ( "TAB" == $delimiter ) $delimiter = "\t"; // correct tabular delimiter

			$teams = $points_plus = $points_minus = $points2_plus = $points2_minus = $pld = $won = $draw = $lost = $custom = array();

			$i = $x = 0;
			while (!feof($handle)) {
				$buffer = fgets($handle, 4096);
				$line = explode($delimiter, $buffer);

				// ignore header and empty lines
				if ( $i > 0 && count($line) > 1 ) {
					$season = $line[0];
					$team = utf8_encode(trim($line[1]));
					$website = isset($line[2]) ? $line[2] : '';
					$coach = isset($line[3]) ? utf8_encode(trim($line[3])) : '';
					$stadium = isset($line[4]) ? utf8_encode(trim($line[4])) : '';
					$home = isset($line[5]) ? $line[5] : '';
					$group = isset($line[6]) ? $line[6] : '';
					$logo = isset($line[7]) ? basename(trim($line[7])) : '';
					$roster = ( isset($line[13]) ) ? $line[13] : '';
					$profile = 0;

					/**
					 * Fired when teams are imported
					 *
					 * @param array $custom
					 * @param array $line
					 * @param int $col the starting column index
					 * @return array
					 * @category wp-filter
					 */
					$custom = apply_filters( 'leaguemanager_import_teams_'.$league->sport, $custom, $line, 14 );
					$team_id = $this->addTeam( $this->league_id, $season, $team, $website, $coach, $stadium, $home, $group, $roster, $profile, $custom, $logo, false );

					if ( $team_id ) {
						$teams[$team_id] = $team_id;
						$pld[$team_id] = isset($line[8]) ? $line[8] : 0;
						$won[$team_id] = isset($line[9]) ? $line[9] : 0;
						$draw[$team_id] = isset($line[10]) ? $line[10] : 0;
						$lost[$team_id] = isset($line[11]) ? $line[11] : 0;
						
						//$points2 = ( isset($line[12]) ) ? explode(":", $line[12]) : array(0,0);
						$points = ( isset($line[12]) ) ? explode(":", $line[12]) : array(0,0);
						
						$points_plus[$team_id] = $points[0];
						$points_minus[$team_id] = $points[1];
						//$custom[$team_id]['points2'] = array( 'plus' => $points2[0], 'minus' => $points2[1] );
						$custom_data[$team_id] = $custom;
					}
					
					$x++;
				}
				$i++;
			}

			$league->saveStandingsManually($teams, $points_plus, $points_minus, $pld, $won, $draw, $lost, 0, $custom_data);

			fclose($handle);

			$this->setMessage(sprintf(__( '%d Teams imported', 'leaguemanager' ), $x));
		}
	}


	/**
	 * import matches from CSV file
	 *
	 * @param string $file
	 * @param string $delimiter
	 * @return void|false
	 */
	private function importMatches( $file, $delimiter ) {
		if ( !current_user_can('import_leagues') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$handle = @fopen($file, "r");
		if ($handle && current_user_can('manage_leaguemanager')) {
			if ( "TAB" == $delimiter ) $delimiter = "\t"; // correct tabular delimiter

			$league = get_league( $this->league_id );

			$matches = $home_points = $away_points = $home_teams = $away_teams = $custom = array();

			$i = $x = 0;
			while (!feof($handle)) {
				$buffer = fgets($handle, 4096);
				$line = explode($delimiter, $buffer);

				// ignore header and empty lines
				if ( $i > 0 && count($line) > 1 ) {
					$date = ( !empty($line[6]) ) ? $line[0]." ".$line[6] : $line[0]. " 00:00";
					$season = $this->season = isset($line[1]) ? $line[1] : '';
					$match_day = isset($line[2]) ? $line[2] : '';
					$date = trim($date);
					$home_team = $this->getTeamID(utf8_encode(trim($line[3])));
					$away_team = $this->getTeamID(utf8_encode(trim($line[4])));
					$location = isset($line[5]) ? utf8_encode(trim($line[5])) : '';
					$group = isset($line[7]) ? trim($line[7]) : '';

					$match_id = $this->addMatch($date, $home_team, $away_team, $match_day, $location, $this->league_id, $season, $group,'', array());

					$matches[$match_id] = $match_id;
					$home_teams[$match_id] = $home_team;
					$away_teams[$match_id] = $away_team;
					if ( isset($line[8]) && !empty($line[8]) ) {
						$score = explode("-", $line[8]);
						$home_points[$match_id] = $score[0];
						$away_points[$match_id] = $score[1];
					} else {
						$home_points[$match_id] = $away_points[$match_id] = '';
					}
					
					$status[$match_id] = isset($line[9]) ? $line[9] : '';
					
					/**
					 * Fired when matches are imported
					 *
					 * @param array $custom
					 * @param array $line
					 * @param int $match_id
					 * @param int $col the starting column index
					 * @return array
					 * @category wp-filter
					 */
					$custom = apply_filters( 'leaguemanager_import_matches_'.$league->sport, $custom, $line, $match_id, 10 );

					$x++;
				}

				$i++;
			}
			$this->updateResults( $league->id, $matches, $home_points, $away_points, $home_teams, $away_teams, $status, $custom, false );

			fclose($handle);

			$this->setMessage(sprintf(__( '%d Matches imported', 'leaguemanager' ), $x));
		}
	}


	/**
	 * get Team ID for given string
	 *
	 * @param string $title
	 * @return int
	 */
	private function getTeamID( $title ) {
		global $wpdb;

		$team = $wpdb->get_results( $wpdb->prepare("SELECT `id` FROM {$wpdb->leaguemanager_teams} WHERE `title` = '%s' AND `league_id` = '%d' AND `season` = '%s'", $title, $this->league_id, $this->season) );
		return $team[0]->id;
	}


	/**
	 * Checks if a particular user has a role.
	 * Returns true if a match was found.
	 *
	 * @param string $role Role name.
	 * @param int $user_id (Optional) The ID of a user. Defaults to the current user.
	 * @return bool
	 *
	 * put together by AppThemes (http://docs.appthemes.com/tutorials/wordpress-check-user-role-function/)
	 */
	function checkUserRole( $role, $user_id = null ) {

		if ( is_numeric( $user_id ) )
			$user = get_userdata( $user_id );
		else
			$user = wp_get_current_user();

		if ( empty( $user ) )
			return false;

		return in_array( $role, (array) $user->roles );
	}


	/**
	 * export league data
	 *
	 * @param int $league_id
	 * @param string $what
	 * @return file|false
	 */
	private function export( $league_id, $what ) {
		global $league;

		if ( !current_user_can('export_leagues') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$league = get_league(intval($league_id));
		$filename = sanitize_title($league->title)."-".$what."_".date("Y-m-d").".xls";

		if ( 'teams' == $what )
			$contents = $this->exportTeams();
		elseif ( 'matches' ==  $what )
			$contents = $this->exportMatches();
		elseif ( 'media' == $what )
			$this->exportMedia();

		header('Content-Type: text/csv');
		header('Content-Disposition: inline; filename="'.$filename.'"');
		echo $contents;
		exit();
	}
	
	
	/**
	 * retrieve path to leagues backups
	 *
	 * @param string|false $file
	 * @return string
	 */
	public function getBackupPath( $file = false ) {
		$base = WP_CONTENT_DIR.'/uploads/leagues/backups';
		if ( $file )
			return $base .'/'. basename($file);
		else
			return $base;
	}


	/**
	 * export media files, e.g. team logos
	 *
	 * @return file
	 */
	private function exportMedia() {
		wp_mkdir_p( $this->getBackupPath() );
		
		$league = get_league();
		$teams = $league->getTeams( array("cache" => false, "season" => false, "group" => "") );
		
		$files = array();
		$filename = sanitize_title($league->title)."_Media_".date("Y-m-d").".zip";
		foreach ( $teams AS $team ) {
			if ( $team->logo != "" ) {
				$files[] = $team->getImagePath($team->logo);
				$files[] = $team->getImagePath($team->logo, 'large');
				$files[] = $team->getImagePath($team->logo, 'medium');
				$files[] = $team->getImagePath($team->logo, 'thumb');
				$files[] = $team->getImagePath($team->logo, 'tiny');
			}
		}
		
		// create zip Archive of media files
		$ret = $this->createZip($files, $this->getBackupPath($filename));
		if ($ret) {
			header("Content-Description: File Transfer");
			header("Content-type: application/octet-stream");
			header("Content-Disposition: attachment; filename=\"".$filename."\"");
			header("Content-Transfer-Encoding: binary");
			header("Content-Length: ".filesize($this->getBackupPath($filename)));
			//ob_end_flush();
			@readfile($this->getBackupPath($filename));
			exit();
		} else {
			$this->setMessage(__('No media files found to export', 'leaguemanager'), true);
			$this->printMessage();
		}
	}
	
	
	/**
	 * export teams
	 *
	 * @return string|false
	 */
	private function exportTeams() {
		if ( !current_user_can('export_leagues') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$league = get_league();
		$teams = $league->getTeams( array("cache" => false, "season" => false, "group" => "") );

		if ( $teams ) {
			$contents = __('Season','leaguemanager')."\t".
			__('Team','leaguemanager')."\t".
			__('Website','leaguemanager')."\t".
			__('Coach','leaguemanager')."\t".
			__('Stadium','leaguemanager')."\t".
			__('Home Team','leaguemanager') ."\t".
			__('Group','leaguemanager')."\t".
			__('Logo','leaguemanager')."\t".
			__('Played','leaguemanager')."\t".
			__('Won','leaguemanager')."\t".
			__('Tie','leaguemanager')."\t".
			__('Lost','leaguemanager')."\t".
			//__('Points2', 'leaguemanager')."\t".
			//__('Diff','leaguemanager')."\t".
			__('Pts','leaguemanager')."\t".
			__('Roster', 'leaguemanager');

			/**
			 * Fired when teams header is exported
			 *
			 * @param string $contents
			 * @return string
			 * @category wp-filter
			 */
			$contents = apply_filters( 'leaguemanager_export_teams_header_'.$league->sport, $contents );

			foreach ( $teams AS $team ) {
				$contents .= "\n".$team->season."\t"
					.utf8_decode($team->title)."\t"
					.utf8_decode($team->website)."\t"
					.utf8_decode($team->coach)."\t"
					.utf8_decode($team->stadium)."\t"
					.$team->home."\t"
					.utf8_decode($team->group)."\t"
					.basename($team->logo)."\t"
					.$team->done_matches."\t"
					.$team->won_matches."\t"
					.$team->draw_matches."\t"
					.$team->lost_matches."\t"
					//.sprintf("%d-%d",$team->points2_plus, $team->points2_minus)."\t"
					//.$team->diff."\t"
					.sprintf("%d-%d", $team->points_plus, $team->points_minus)."\t"
					.$team->roster;

				/**
				 * Fired when teams data are exported
				 *
				 * @param string $content
				 * @param Team $team
				 * @return string
				 * @category wp-filter
				 */
				$contents = apply_filters( 'leaguemanager_export_teams_data_'.$league->sport, $contents, $team );
			}
			return $contents;
		}
		
		return false;
	}


	/**
	 * export matches
	 *
	 * @return string|false
	 */
	private function exportMatches() {
		if ( !current_user_can('export_leagues') ) {
			$this->setMessage( __("You don't have permission to perform this task", 'leaguemanager'), true );
			return false;
		}
		
		$league = get_league();
		$matches = $league->getMatches( array("limit" => false, "match_day" => -1, "group" => "", "cache" => false) );
			
		if ( $matches ) {
			// Build header
			$contents =
			__('Date','leaguemanager')."\t".
			__('Season','leaguemanager')."\t".
			__('Match Day','leaguemanager')."\t".
			__('Home','leaguemanager')."\t".
			__('Guest','leaguemanager')."\t".
			__('Location','leaguemanager')."\t".
			__('Begin','leaguemanager')."\t".
			__('Group','leaguemanager')."\t".
			__('Score','leaguemanager')."\t".
			__('Status', 'leaguemanager');

			/**
			 * Fired when matches header is exported
			 *
			 * @param string $contents
			 * @return string
			 * @category wp-filter
			 */
			$contents = apply_filters( 'leaguemanager_export_matches_header_'.$league->sport, $contents );

			foreach ( $matches AS $match ) {
				$contents .= "\n".mysql2date('Y-m-d', $match->date)."\t".
					$match->season."\t".
					$match->match_day."\t".
					utf8_decode(preg_replace("/\<strong\>(.*)\<\/strong\>/", "$1", $match->teams['home']->title))."\t".
					utf8_decode(preg_replace("/\<strong\>(.*)\<\/strong\>/", "$1", $match->teams['away']->title))."\t".
					utf8_decode($match->location)."\t".
					mysql2date("H:i", $match->date)."\t".
					$match->group."\t";
				$contents .= !empty($match->home_points) ? sprintf("%d-%d",$match->home_points, $match->away_points) : '';
				$contents .= "\t".utf8_decode($match->status);
				/**
				 * Fired when matches data are exported
				 *
				 * @param string $contents
				 * @param Match $match
				 * @return string
				 * @category wp-filter
				 */
				$contents = apply_filters( 'leaguemanager_export_matches_data_'.$league->sport, $contents, $match );
			}

			return $contents;
		}
		
		return false;
	}
	

	/**
	 * recursively apply htmlspecialchars to an array
	 *
	 * @param array $arr
	 */
	public function htmlspecialchars_array($arr = array()) {
		$rs =  array();
		while(list($key,$val) = each($arr)) {
			if(is_array($val)) {
				$rs[$key] = $this->htmlspecialchars_array($val);
			} else {
				$rs[$key] = htmlspecialchars($val, ENT_QUOTES);
			}   
		}
		
		return $rs;
	}
	
	
	/**
	 * show database columns of LeagueManager
	 */
	private function showDatabaseColumns() {
		global  $wpdb;
		
		$tables = array($wpdb->leaguemanager, $wpdb->leaguemanager_teams, $wpdb->leaguemanager_matches, $wpdb->leaguemanager_stats);
		
		foreach( $tables AS $table ) {
			$results = $wpdb->get_results("SHOW COLUMNS FROM {$table}");
			$columns = array();
			foreach ( $results AS $result ) {
				$columns[] = "<li>".$result->Field." ".$result->Type.", NULL: ".$result->Null.", Default: ".$result->Default.", Extra: ".$result->Extra."</li>";
			}
			echo "<p>Table ".$table."<ul>";
			echo implode("", $columns);
			echo "</ul></p>";
		}
	}
}
?>