<?php
/**
 * AJAX response methods
 *
 */

/**
 * Implement AJAX responses
 *
 * @package	LeagueManager
 * @subpackage LeagueManagerAJAX
*/
class LeagueManagerAJAX extends LeagueManager {
	/**
	 * register ajax actions
	 */
	public function __construct() {
		// admin/team.php
		add_action( 'wp_ajax_leaguemanager_insert_logo_from_library', array(&$this, 'insertLogoFromLibrary') );
		add_action( 'wp_ajax_leaguemanager_add_team_from_db', array(&$this, 'addTeamFromDB') );
		
		// admin/admin.php
		add_action( 'wp_ajax_leaguemanager_get_season_dropdown', array(&$this, 'setSeasonDropdown') );
		add_action( 'wp_ajax_leaguemanager_get_match_dropdown', array(&$this, 'setMatchesDropdown') );
		
		// admin/standings.php
		add_action( 'wp_ajax_leaguemanager_save_add_points', array(&$this, 'saveAddPoints') );
	
		// admin/match.php
		add_action( 'wp_ajax_leaguemanager_set_match_day_popup', array(&$this, 'setMatchDayPopUp') );
		add_action( 'wp_ajax_leaguemanager_set_match_date', array(&$this, 'setMatchDate') );
		add_action( 'wp_ajax_leaguemanager_insert_home_stadium', array(&$this, 'insertHomeStadium') );
		
		// admin/settings.php
		add_action( 'wp_ajax_leaguemanager_toggle_point_rule', array(&$this, 'setPointRuleInput') );
		
		// lib/widget.php
		add_action( 'wp_ajax_leaguemanager_get_match_box', array(&$this, 'getMatchBox') );
		add_action( 'wp_ajax_nopriv_leaguemanager_get_match_box', array(&$this, 'getMatchBox') );
	}


	/**
	 * AJAX Response to set match index in widget
	 *
	 * @see lib/widget.php
	 */
	public function getMatchBox() {
		$widget = new LeagueManagerWidget(true);

		$current = intval($_POST['current']);
		$element = ($_POST['element'] == 'next') ? 'next' : 'prev';
		$league_id = intval($_POST['league_id']);
		$match_limit = intval($_POST['match_limit']);
		$widget_number = intval($_POST['widget_number']);
		$season = htmlspecialchars($_POST['season']);
		$group = ( isset($_POST['group']) ? htmlspecialchars($_POST['group']) : '' );
		$home_only = htmlspecialchars($_POST['home_only']);
		$date_format = htmlspecialchars($_POST['date_format']);

		$index = ( $_POST['operation'] == 'prev' ) ? $current - 1 : $current + 1;

		$widget->setMatchIndex( $index, $element );

		$instance = array( 'league' => $league_id, 'match_limit' => $match_limit, 'group' => $group, 'season' => $season, 'home_only' => $home_only, 'date_format' => $date_format );
			
		if ( $element == 'next' ) {
			$parent_id = 'next_matches_'.$widget_number;
			$match_box = $widget->showNextMatchBox($widget_number, $instance, false);
		} elseif ( $element == 'prev' ) {
			$parent_id = 'prev_matches_'.$widget_number;
			$match_box = $widget->showPrevMatchBox($widget_number, $instance, false, true);
		}

		die( "jQuery('div#".$parent_id."').fadeOut('fast', function() {
			jQuery('div#".$parent_id."').html('".addslashes_gpc($match_box)."').fadeIn('fast');
		});");
	}


	/**
	 * AJAX response to manually set additional points
	 *
	 * @see admin/standings.php
	 */
	public function saveAddPoints() {
		global $wpdb;
		
		$league = get_league(intval($_POST['league_id']));
		$team_id = intval($_POST['team_id']);
		$points = intval($_POST['points']);
		
		$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->leaguemanager_teams} SET `add_points` = '%s' WHERE `id` = '%d'", $points, $team_id ) );
		$league->_rankTeams(true);
		
		die("jQuery('#loading_".$team_id."').fadeOut('fast'); window.location.reload(true);");
	}


	/**
	 * AJAX response to get team data from database and insert into team edit form
	 *
	 * @see admin/team.php
	 */
	public function addTeamFromDB() {
		$team = get_team( intval($_POST['team_id']) );
		$index = intval($_POST['index']);
		
		$roster = '';
		/*
		if ( $this->hasProjectManager() ) {
			global $projectmanager;
			$html = '<select size="1" name="roster_'.$index.'" id="roster_'.$index.'" onChange="Leaguemanager.toggleTeamRosterGroups(this.value);return false;"><option value="">'.__('None','leaguemanager').'</option>';
			foreach ( $projectmanager->getProjects() AS $project ) {
				$selected = ( $project->id == $team->roster['id'] ) ? ' selected="selected"' : '';
				$html .= '<option value="'.$project->id.'"'.$selected.'>'.$project->title.'</option>';
			}
			$html .= '</select>';
			$roster = "jQuery('span#rosterbox_".$index."').fadeOut('fast', function() {
					jQuery('span#rosterbox_".$index."').html('".addslashes_gpc($html)."').fadeIn('fast')
				   });";

			if ( isset($team->roster['cat_id']) ) {
				$project = get_project($team->roster['id']);

				$html = "<select size='1' name='roster_group_".$index."'>";
				$html .= "<option value=''>".__('Select Group (Optional)', 'leaguemanager')."</option>";			
				foreach ($project->getCategories() AS $category) {
					$html .= "<option value='".$category->id."' ".selected($category->id, $project->getCatID(), false).">".$category->title."</option>";
				}
				$html .= "</select>";
				
				$roster .= "jQuery('span#team_roster_groups_".$index."').fadeOut('fast', function () {
						jQuery('span#team_roster_groups_".$index."').html('".addslashes_gpc($html)."').fadeIn('fast');
					   });";
			} else {
				$roster .= "jQuery('span#team_roster_groups_".$index."').fadeOut('fast');";
			}
		}
	*/
	
		$home = ( $team->isHome() ) ? "document.getElementById('home_".$index."').checked = true;" : "document.getElementById('home_".$index."').checked = false;";
		$logo = ( $team->hasLogo() ) ? "<img src='".$team->getImageUrl($team->logo)."' />" : "";
		die("
			document.getElementById('team_".$index."').value = '".$team->title."';
			document.getElementById('website_".$index."').value = '".$team->website."';
			document.getElementById('coach_".$index."').value = '".$team->coach."';
			document.getElementById('stadium_".$index."').value = '".$team->stadium."';
			document.getElementById('logo_db_".$index."').value = '".$team->logo."';
			jQuery('#logo_db_box_".$index."').html('".addslashes_gpc($logo)."').fadeIn('fast');
			".$home."
			".$roster."
		");
	}


	/**
	 * insert Logo from Library in team edit
	 *
	 * @see admin/team.php
	 */
	public function insertLogoFromLibrary() {
		$logo = htmlspecialchars($_POST['logo']);
		$logo = 'http://' . $logo;
		$index = intval($_POST['index']);
		$html = "<img id='logo_image' src='".$logo."' />";

		if ( $_SERVER['HTTP_HOST'] != substr($logo, 7, strlen($_SERVER['HTTP_HOST'])) ) {
			die("alert('".__('The image cannot be on a remote server', 'leaguemanager')."')");
		} else {
			die("jQuery('div#logo_db_box_".$index."').fadeOut('fast', function() {
				document.getElementById('logo_db_".$index."').value = '".$logo."';
				jQuery('div#logo_db_box_".$index."').html('".addslashes_gpc($html)."').fadeIn('fast');
			});");
		}
	}


	/**
	 * insert home team stadium if available
	 *
	 * @see admin/match.php
	 */
	public function insertHomeStadium() {
		$i = (int)$_POST['i'];

		$team = get_team( intval($_POST['team_id']) );
		die("if (jQuery('#location[".$i."]).val() == '') {
			jQuery('#location[".$i."]').val('".$team->stadium."');
		}");
	}
	

	/**
	 * change all Match Day Pop-ups to match first one set
	 *
	 * @see admin/match.php
	 */
	public function setMatchDayPopUp() {
		$match_day = (int)$_POST['match_day'];
		$i = (int)$_POST['i'];
		$max_matches = (int)$_POST['max_matches'];
		$mode = htmlspecialchars($_POST['mode']);

        if ( $i == 0 && $mode == 'add') {
            $myAjax = "";
            for ( $xx = 1; $xx < $max_matches; $xx++ ) {
    		    $myAjax .= "document.getElementById('match_day_".$xx."').value = '".$match_day."'; ";
            }
    		die("".$myAjax."");
        }
    }

	/**
	 * change all Match Date fields to match first one set
	 *
	 * @see admin/match.php
	 */
	public function setMatchDate() {
		$match_date = htmlspecialchars($_POST['match_date']);
		$i = (int)$_POST['i'];
		$max_matches = (int)$_POST['max_matches'];
		$mode = htmlspecialchars($_POST['mode']);

        if ( $i == 0 && $mode == 'add' ) {
            $myAjax = "";
            for ( $xx = 1; $xx < $max_matches; $xx++ ) {
    		    $myAjax .= "document.getElementById('mydatepicker[".$xx."]').value = '".$match_date."'; ";
            }
    		die("".$myAjax."");
        }
    }
	
	
	/**
	 * set season dropdown for post metabox for match report
	 *
	 * @see admin/admin.php
	 */
	public function setSeasonDropdown() {
		$league = get_league(intval($_POST['league_id']));
		
		$html = $league->getSeasonDropdown(true);
		die( "jQuery('div#matches').fadeOut('fast', function() {
			jQuery('div#seasons').fadeOut('fast');
			jQuery('div#seasons').html('".addslashes_gpc($html)."').fadeIn('fast');
		});");
	}
	
	
	/**
	 * set matches dropdown for post metabox for match report
	 *
	 * @see admin/admin.php
	 */
	public function setMatchesDropdown() {
		$league = get_league(intval($_POST['league_id']));
		$league->setSeason(htmlspecialchars($_POST['season']));
		$html = $league->getMatchesDropdown();
		
		die( "jQuery('div#matches').fadeOut('fast', function() {
			jQuery('div#matches').html('".addslashes_gpc($html)."').fadeIn('fast');
		});");
	}
	
	
	/**
	 * set point rule input
	 *
	 * @see admin/settings.php
	 */
	public function setPointRuleInput() {
		$league = get_league();
		$rule = htmlspecialchars($_POST['rule']);
	
		$html = "";
		if ( $rule == 'user' ) {
			$html = '<div id="point_rule_manual_content">';
			$html .= "<input type='text' name='forwin' id='forwin' value='".$league->point_rule_data['forwin']."' size='2' />";
			$html .= "<input type='text' name='forwin_overtime' id='forwin_overtime' value='".$league->point_rule_data['forwin_overtime']."' size='2' />";
			$html .= "<input type='text' name='fordraw' id='fordraw' value='".$league->point_rule_data['fordraw']."' size='2' />";
			$html .= "<input type='text' name='forloss' id='forloss' value='".$league->point_rule_data['forloss']."' size='2' />";
			$html .= "<input type='text' name='forloss_overtime' id='forloss_overtime' value='".$league->point_rule_data['forloss_overtime']."' size='2' />";
			$html .= "&#160;<span class='setting-description'>".__( 'Order: win, win overtime, tie, loss, loss overtime', 'leaguemanager' )."</span>";
			$html .= "</div>";
		}
		
		die( "jQuery('div#point_rule_manual').fadeOut('fast', function() {
			jQuery('div#point_rule_manual').html('".addslashes_gpc($html)."').fadeIn('fast');
		});");
	}
}
?>