<?php
/**
 * League API: League class
 *
 * @package LeagueManager
 * @subpackage League
 */
 
/**
 * Class to implement the League object
 *
 */
class League {
	/**
	 * League ID
	 *
	 * @var int
	 */
	public $id;
	
	/**
	 * League title
	 *
	 * @var string
	 */
	public $title;
	
	/**
	 * seasons data
	 *
	 * @var array
	 */
	public $seasons = array();
	
	/**
	 * number of seasons
	 *
	 * @var int
	 */
	public $num_seasons = 0;
	
	/**
	 * sport type
	 *
	 * @var string
	 */
	public $sport = "default";
	
	/**
	 * point rule
	 *
	 * @var string
	 */
	public $point_rule = "three";
	
	/**
	 * primary points format
	 *
	 * @var string
	 */
	public $point_format = "%d:%d";
	
	/**
	 * secondary points format
	 *
	 * @var string
	 */
	public $point_format2 = "%d:%d";

	/**
	 * team ranking mode
	 *
	 * @var string
	 */
	public $team_ranking = "auto";
	
	/**
	 * league mode
	 *
	 * @var string
	 */
	public $mode = "default";
	
	/**
	 * default match starting time
	 *
	 * @var array
	 */
	public $default_match_start_time = array("hour" => 0, "minutes" => 0);
	
	/**
	 * standings table layout settings
	 *
	 * @var array
	 */
	public $standings = array( 'status' => 1, 'logos' => 1, 'team_link' => 1, 'pld' => 1, 'won' => 1, 'tie' => 1, 'lost' => 1, 'winPercent' => 1, 'last5' => 1 );
	
	/**
	 * number of teams ascending
	 *
	 * @var int
	 */
	public $num_ascend = 0;
	
	/**
	 * number of teams descending
	 *
	 * @var int
	 */
	public $num_descend = 0;
	
	/**
	 * number of teams for relegationnum_relegation
	 *
	 * @var int
	 */
	public $num_relegation = 0;
	
	/**
	 * number of teams per page in list
	 *
	 * @var int
	 */
	public $num_teams_per_page = 10;
	
	/**
	 * number of pages for teams
	 *
	 * @var int
	 */
	public $num_pages_teams = 0;
	
	/**
	 * current page for teams
	 *
	 * @var int
	 */
	public $current_page_teams = 1;
	
	/**
	 * teams pagination
	 *
	 * @var string
	 */
	public $pagination_teams = '';
	
	/**
	 * number of matches per page
	 *
	 * @var int
	 */
	public $num_matches_per_page = 10;
	
	/**
	 * default display filter for matches
	 *
	 * @var string
	 */
	public $match_display = 'current_match_day';
	
	/**
	 * number of pages for matches
	 *
	 * @var int
	 */
	public $num_pages_matches = 0;
	
	/**
	 * current page for matches
	 *
	 * @var int
	 */
	public $current_page_matches = 1;
	
	/**
	 * matches pagination
	 *
	 * @var string
	 */
	public $pagination_matches = '';
	
	/**
	 * slideshow options
	 *
	 * @var array
	 */
	public $slideshow = array( 'season' => 'latest', 'num_matches' => 0, 'show_logos' => 1 );
	
	/**
	 * logo tiny size
	 *
	 * @var array
	 */
	public $tiny_size = array("width" => 40, "height" => 40);
	
	/**
	 * logo thumbnail size
	 *
	 * @var array
	 */
	public $thumb_size = array("width" => 100, "height" => 100);
	
	/**
	 * logo large size
	 *
	 * @var array
	 */
	public $large_size = array("width" => 1024, "height" => 1024);
	
	/**
	 * crop images
	 *
	 * @var array
	 */
	public $crop_image = array('tiny' => 1, 'thumb' => 1, 'medium' => 0, 'large' => 0);
	
	/**
	 * team groups
	 *
	 * @var array
	 */
	public $groups = array();
	
	/**
	 * curent team group
	 *
	 * @var string
	 */
	public $current_group = '';
	
	/**
	 * teams
	 *
	 * @var array
	 */
	public $teams = array();

	/**
	 * total number of teams
	 *
	 * @var int
	 */
	public $num_teams_total = 0;
	
	/**
	 * number of teams for current query
	 *
	 * @var int
	 */
	public $num_teams = 0;
	
	/**
	 * number of teams by group
	 *
	 * @var array
	 */
	public $num_teams_by_group = array();
	
	/**
	 * matches
	 *
	 * @var array
	 */
	public $matches = array();
	
	/**
	 * total number of matches
	 *
	 * @var int
	 */
	public $num_matches_total = 0;
	
	/**
	 * number of matches
	 *
	 * @var int
	 */
	public $num_matches = 0;
	
	/**
	 * current season
	 *
	 * @var array
	 */
	public $current_season = array();
	
	/**
	 * number of match days
	 *
	 * @var int
	 */
	public $num_match_days = 0;
	
	/**
	 * current match day
	 *
	 * @var int
	 */
	public $match_day = -1;
	
	/**
	 * query arguments
	 *
	 * @var array
	 */
	private $query_args = array();

	/**
	 * team database query args
	 *
	 * @var array
	 */
	private $team_query_args = array( 'limit' => false, 'group' => '', 'season' => '', 'rank' => 0, 'orderby' => array("rank" => "ASC"), "home" => false, 'ids' => array(), 'cache' => true );
	
	/**
	 * team query argument types
	 *
	 * @var array
	 */
	private $team_query_args_types = array( 'limit' => 'numeric', 'group' => 'string', 'season' => 'string', 'rank' => 'numeric', 'orderby' => 'array', 'home' => 'boolean', 'ids' => 'array_numeric', 'cache' => 'boolean' );
	
	/**
	 * match query arguments
	 *
	 * @var array
	 */
	private $match_query_args = array( 'limit' => true, 'group' => '', 'season' => '', 'final' => '', 'match_day' => -1, 'time' => '', 'home_only' => false, 'count' => false, 'orderby' => array("date" => "ASC"), 'standingstable' => false, 'cache' => true, 'team_id' => 0, 'home_team' => '', 'away_team' => '', 'team_pair' => array(), 'winner_id' => false, 'loser_id' => false, 'home_points' => false, 'away_points' => false, 'mode' => '', 'reset_limit' => true, 'reset_query_args' => false, 'update_results' => false );
	
	/**
	 * match query argument types
	 *
	 * @var array
	 */
	private $match_query_args_types = array( 'limit' => 'numeric', 'group' => 'string', 'season' => 'string', 'final' => 'string', 'match_day' => 'numeric', 'time' => 'string', 'home_only' => 'boolean', 'count' => 'boolean', 'orderby' => 'array', 'standingstable' => 'boolean', 'cache' => 'boolean', 'team_id' => 'numeric', 'home_team' => 'string', 'away_team' => 'string', 'team_pair' => 'array', 'winner_id' => 'numeric', 'loser_id' => 'numeric', 'home_points' => 'string', 'away_points' => 'string', 'mode' => 'string', 'reset_limit' => 'boolean', 'reset_query_args' => 'boolean', 'update_results' => 'boolean' );
	
	/**
	 * settings keys
	 *
	 * @var array
	 */
	private $settings_keys = array();
	
	/**
	 * team offsets indexed by ID
	 *
	 * @var array
	 */
	public $team_index = array();
	
	/**
	 * team loop
	 *
	 * @var boolean
	 */
	public $in_the_team_loop = false;
	
	/**
	 * current team
	 *
	 * @var int
	 */
	public $current_team = -1;
	
	/**
	 * team is selected?
	 *
	 * @var boolean
	 */
	public $is_selected_team = false;
	
	/**
	 * match loop
	 *
	 * @var boolean
	 */
	public $in_the_match_loop = false;
	
	/**
	 * current match
	 *
	 * @var int
	 */
	public $current_match = -1;
	
	/**
	 * match is selected
	 *
	 * @var boolean
	 */
	public $is_selected_match = false;
	
	/**
	 * toggle match day selection menu display
	 *
	 * @var boolean
	 */
	public $show_match_day_selection = true;
	
	/**
	 * toggle team selection menu display
	 *
	 * @var boolean
	 */
	public $show_team_selection = true;
	
	/**
	 * toggle matches selection menu display, depends on $show_match_day_selection and $show_team_selection
	 *
	 * @var boolean
	 */
	public $show_matches_selection = true;
	
	/**
	 * is this an archive
	 *
	 * @var boolean
	 */
	public $is_archive = false;
	
	/**
	 * set archive tab
	 *
	 * @var int
	 */
	public $archive_tab = 0;
	
	/**
	 * save templates for whole league or archive display
	 *
	 * @var array
	 */
	public $templates = array();
	
	/**
	 * Project ID for team profiles
	 *
	 * @var int
	 */
	public $teamprofiles = 0;
	
	/**
	 * Project ID for team roster
	 *
	 * @var int
	 */
	public $teamroster = 0;
		
	/**
	 * Project ID for match statistics
	 *
	 * @var int
	 */
	public $matchstats = 0;
	
	/**
	 * championship flag
	 *
	 * @var boolean
	 */
	public $is_championship = false;
	
	/**
	 * League_Championship object
	 *
	 * @var League_Championship
	 */
	public $championship = null;
	
	/**
	 * define active tiebreak rules
	 *
	 * @var array
	 */
	public $tiebreak = array( 'headToHeadTwoTeams' => 1 );
	
	/**
	 * is tiebreak flag
	 *
	 * @var boolean
	 */
	private $is_tiebreak = false;
	
	/**
	 * tied teams flag
	 *
	 * @var boolean
	 */
	private $is_tie = false;
	
	/**
	 * custom team input field keys and translated labels
	 *
	 * @var array
	 */
	protected $fields_team = array();
	
	/**
	 * custom match input field keys and translated labels
	 *
	 * @var array
	 */
	protected $fields_match = array();
	
	
	/**
	 * retrieve league instance
	 *
	 * @param int $league_id
	 */
	public static function get_instance($league_id) {
		global $wpdb;

		$league_id = (int) $league_id;
		if ( ! $league_id )
			return false;

		$league = wp_cache_get( $league_id, 'leagues' );
		
		if ( ! $league ) {
			$league = $wpdb->get_row( $wpdb->prepare( "SELECT `title`, `id`, `seasons`, `settings` FROM {$wpdb->leaguemanager} WHERE `id` = '%d' LIMIT 1", $league_id ) );
			$league->settings = (array)maybe_unserialize($league->settings);
			$league = (object)array_merge((array)$league, $league->settings);
			
			if ( !$league ) return false;
		
			// check if specific sports class exists
			if ( !isset($league->sport) ) $league->sport = '';
			$instance = "League_". ucfirst($league->sport);
			if (class_exists($instance)) {
				$league = new $instance( $league );
			} else {
				$league = new League( $league );
			}
			
			wp_cache_add( $league->id, $league, 'leagues' );
		}
		
		return $league;
	}
	
	
	/**
	 * Constructor
	 *
	 * @param object $league League object.
	 */
	public function __construct( $league ) {
		if (isset($league->settings)) {
			$league->settings = (array)maybe_unserialize($league->settings);
			$league->settings_keys = array_keys((array)maybe_unserialize($league->settings));
			$league = (object)array_merge((array)$league, $league->settings);
			unset($league->settings);
		}
		
		foreach ( get_object_vars( $league ) as $key => $value ) {
			if ( $key == "standings")
				$this->$key = array_merge($this->$key, $value);
			else
				$this->$key = $value;
		}
		
		$this->title = stripslashes($this->title);
		
		// make sure that groups are array
		if ( is_string($this->groups) )
			$this->groups = explode(";", $this->groups);
		
		// set seasons
		if ( $this->seasons == '' ) $this->seasons = array();
		$this->seasons = (array)maybe_unserialize($this->seasons);
		$this->num_seasons = count($this->seasons);
		
		// set season to latest
		$this->setSeason();
		
		// set current group
		$this->setGroup();
		
		$this->setMatchQueryArgs();
		$this->setNumMatches(true); // get total number of matches
		$this->setNumMatches();
		$this->setNumTeams(true); // get total number of teams
		
		// set default standings display options for additional team fields
		if ( count($this->fields_team) > 0 ) {
			foreach ( $this->fields_team AS $key => $data ) {
				if ( !isset($this->standings[$key]) )
					$this->standings[$key] = 1;
			}
		}
		
		// set selected team marker
		if (isset($_GET['team_'.$this->id])) {
			$this->current_team = intval($_GET['team_'.$this->id]);
			$this->is_selected_team = true;
		}
		/*if (isset($_GET['team_id_'.$this->id])) {
			$this->current_team = intval($_GET['team_id_'.$this->id]);
			$this->is_selected_team = true;
		}*/
		
		// set selected match marker
		if (isset($_GET['match_'.$this->id])) {
			$this->current_match = intval($_GET['match_'.$this->id]);
			$this->is_selected_match = true;
		}
		
		// Championship
		if ( $this->mode == "championship" ) {
			$this->is_championship = true;
			$this->championship = new League_Championship( $this->id, $this->getSettings() );
			//$this->num_teams_in_league = $this->championship->num_groups * $this->championship->teams_per_group;
		}
		
		// add actions & filter
		add_filter( 'league_standings_options', array(&$this, 'standingsTableDisplayOptions') );
		add_action( 'team_edit_form', array(&$this, 'editTeamForm') );
		
		add_filter( 'leaguemanager_export_matches_header_'.$this->sport, array(&$this, 'exportMatchesHeader') );
		add_filter( 'leaguemanager_export_matches_data_'.$this->sport, array(&$this, 'exportMatchesData'), 10, 2 );
		add_filter( 'leaguemanager_import_matches_'.$this->sport, array(&$this, 'importMatches'), 10, 4 );
		
		add_filter( 'leaguemanager_export_teams_header_'.$this->sport, array(&$this, 'exportTeamsHeader') );
		add_filter( 'leaguemanager_export_teams_data_'.$this->sport, array(&$this, 'exportTeamsData'), 10, 2 );
		add_filter( 'leaguemanager_import_teams_'.$this->sport, array(&$this, 'importTeams'), 10, 3 );
	}
	
	
	/**
	 * get settings
	 *
	 * @param string $key settings key
	 * @return array
	 */
	public function getSettings($key=false) {
		$settings = array();
		foreach ($this->settings_keys AS $k)
			$settings[$k] = $this->$k;
		
		if ( $key )
			return (isset($settings[$key])) ? $settings[$key] : false;
		
		return $settings;
	}
	
	
	/**
	 * reload settings from database
	 */
	public function reloadSettings() {
		global $wpdb;
		
		$result = $wpdb->get_row( $wpdb->prepare("SELECT `settings` FROM {$wpdb->leaguemanager} WHERE `id` = '%d'", intval($this->id)) );
		foreach ( maybe_unserialize($result->settings) as $key => $value )
			$this->$key = $value;
	}
	
	
	/**
	 * set query args
	 *
	 * @param string $key
	 * @param $value
	 */
	private function setQueryArg($key, $value) {
		$this->query_args[$key] = $value;
	}
	
	
	/**
	 * set detault dataset query arguments
	 */
	private function setMatchQueryArgs() {
		// set to latest match day by default
		$this->setMatchQueryArg('match_day', 'current');
		
		// set number of matches per page
		$this->setMatchQueryArg('limit', $this->num_matches_per_page);
	}
	
	
	/**
	 * set match query argument
	 *
	 * @param string $key
	 * @param mixed $value
	 * @param boolean $replace - used for arrays to add arguments or replace with values
	 */
	public function setMatchQueryArg( $key, $value, $replace = true ) {
		if ($key == 'limit' && ($value === true || $value == "true")) $value = $this->num_matches_per_page;
		
		/*
		 * sanitize query arg types
		 */
		$v = $value;
		if ($this->match_query_args_types[$key] == 'numeric')
			$v = intval($value);
		if ($this->match_query_args_types[$key] == 'boolean')
			$v = intval($value) == 1;
		
		if (is_array($this->match_query_args[$key]) && !$replace) {
			if (!is_array($v)) $v = array($v);
			$this->match_query_args[$key] = array_merge($this->match_query_args[$key], $v);
		} else {
			$this->match_query_args[$key] = $v;
		}
		
		if ($key == "match_day")
			$this->setMatchDay($value);
	}
	
	
	/**
	 * set team query argument
	 *
	 * @param string $key
	 * @param mixed $value
	 * @param boolean $replace - used for arrays to add arguments or replace with values
	 */
	public function setTeamQueryArg( $key, $value, $replace = true ) {
		if ($key == 'limit' && ($value === true || $value == "true")) $value = $this->num_teams_per_page;
		
		/*
		 * sanitize query arg types
		 */
		if ($this->team_query_args_types[$key] == 'numeric')
			$value = intval($value);
		if ($this->team_query_args_types[$key] == 'boolean')
			$value = intval($value) == 1;
		
		if (is_array($this->team_query_args[$key]) && !$replace) {
			if (!is_array($value)) $value = array($value);
			$this->team_query_args[$key] = array_merge($this->team_query_args[$key], $value);
		} else {
			$this->team_query_args[$key] = $value;
		}
	}
	
	
	/**
	 * get tiebreak methods
	 *
	 * @return array
	 */
	public function getTiebreakRules() {
		$rules = array(
			'headToHeadTwoTeams' => __( 'Head to head matches', 'leaguemanager' )
		);
		
		return $rules;
	}
	
	
	/**
	 * set current season
	 *
	 * @param mixed $season
	 * @param boolean $force_overwrite
	 */
	public function setSeason( $season = false, $force_overwrite = false ) {
		if ( !empty($season) && $force_overwrite === true ) {
			$data = $this->seasons[$season];
		} elseif ( isset($_GET['season']) && !empty($_GET['season']) ) {
			$key = htmlspecialchars(strip_tags($_GET['season']));
			if (!isset($this->seasons[$key]))
				$data = false;
			else
				$data = $this->seasons[$key];
		} elseif ( isset($_GET['season_'.$this->id]) && !empty($_GET['season_'.$this->id]) ) {
			$key = htmlspecialchars(strip_tags($_GET['season_'.$this->id]));
			if (!isset($this->seasons[$key]))
				$data = false;
			else
				$data = $this->seasons[$key];
		} elseif ( !empty($season) ) {
			$data = $this->seasons[$season];
		} else {
			$data = end($this->seasons);
		}
		
		if (empty($data)) $data = end($this->seasons);
		
		$this->current_season = $data;
		$this->num_match_days = $data['num_match_days'];
		
		$this->setTeamQueryArg('season', $this->current_season['name']);
		$this->setMatchQueryArg('season', $this->current_season['name']);
	}

	
	/**
	 * get current season name
	 *
	 * @return string
	 */
	public function getSeason() {
		return stripslashes($this->current_season['name']);
	}
	
	
	/**
	 * set group
	 *
	 * @param string $group
	 * @param boolean $force_overwrite
	 */
	public function setGroup( $group = '', $force_overwrite = false ) {
		if ( $group != '' && $force_overwrite === true ) {
			$group = $group;
		} elseif ( isset($_GET['group']) ) {
			$group = $_GET['group'];
		} else if ( is_admin() && isset($_POST['group']) ) {
			$group = $_POST['group'];
		} else {
			// set to first group in league by default
			$groups = $this->getGroups();
			if ( isset($groups[0]) ) $group = $groups[0];
		}
		
		if (is_array($group)) $group = $group[0];
		
		$group = htmlspecialchars(strip_tags($group));
		if ($this->groupExists($group)) {
			$this->setTeamQueryArg('group', $group);
			$this->setMatchQueryArg('group', $group);
			
			$this->current_group = $group;
		}
	}
	
	
	/**
	 * get current group
	 *
	 */
	public function getGroup() {
		return $this->current_group;
	}
	
	
	/**
	 * get groups
	 *
	 * @return array
	 */
	public function getGroups() {
		if ( is_string($this->groups) )
			$groups = explode(";", $this->groups);
		else
			$groups = $this->groups;
		
		if ( !is_array($groups) )
			return false;
		
		return $groups;
	}
	
	
	/**
	 * retrieve match day
	 *
	 * @param mixed $_match_day
	 */
	public function setMatchDay( $_match_day = false ) {
		global $wpdb;
		
		if (is_numeric($_match_day) && $_match_day != 0) {
			$match_day = intval($_match_day);
		} elseif ( isset($_GET['match_day']) ) {
			$match_day = intval($_GET['match_day']);
		} elseif ( isset($_GET['match_day_'.$this->id])) {
			$match_day = intval($_GET['match_day_'.$this->id]);
		} elseif (isset($_POST['match_day'])) {
			$match_day = intval($_POST['match_day']);
		} elseif ( $_match_day == "last" ) {
			$match_day = wp_cache_get( 'last_'.$this->id, 'leagues_match_days' );
			if (!$match_day) {
				$sql = "SELECT `match_day`, DATEDIFF(NOW(), `date`) AS datediff FROM {$wpdb->leaguemanager_matches} WHERE `league_id` = '%d' AND `season` = '%s' AND DATEDIFF(NOW(), `date`) > 0 ORDER BY datediff ASC LIMIT 1";
				$match = $wpdb->get_row( $wpdb->prepare($sql, $this->id, $this->current_season['name']) );
				if ($match)  {
					$match_day = $match->match_day;
					wp_cache_add( 'last_'.$this->id, $match_day, 'leagues_match_days' );
				} else {
					$match_day = $_match_day;
				}
			}
		} elseif ( $_match_day == "next" ) {
			$match_day = wp_cache_get( 'next_'.$this->id, 'leagues_match_days' );
			if (!$match_day) {
				$sql = "SELECT `match_day`, DATEDIFF(NOW(), `date`) AS datediff FROM {$wpdb->leaguemanager_matches} WHERE `league_id` = '%d' AND `season` = '%s' AND DATEDIFF(NOW(), `date`) < 0 ORDER BY datediff DESC LIMIT 1";
				$match = $wpdb->get_row( $wpdb->prepare($sql, $this->id, $this->current_season['name']) );
				if ($match)  {
					$match_day = $match->match_day;
					wp_cache_add( 'next_'.$this->id, $match_day, 'leagues_match_days' );
				} else {
					$match_day = $_match_day;
				}
			}
		} elseif ( $_match_day == "current" || $_match_day == "latest") {
			$match_day = wp_cache_get( 'current_'.$this->id, 'leagues_match_days' );
			if (!$match_day) {
				$sql = "SELECT `id`, `match_day`, DATEDIFF(NOW(), `date`) AS datediff FROM {$wpdb->leaguemanager_matches} WHERE `league_id` = '%d' AND `season` = '%s' ORDER BY datediff ASC LIMIT 1";
				$match = $wpdb->get_row( $wpdb->prepare($sql, $this->id, $this->current_season['name']) );
				if ($match) {
					$match_day = $match->match_day;
					wp_cache_add( 'current_'.$this->id, $match_day, 'leagues_match_days' );
				} else {
					$match_day = $_match_day;
				}
			}
		} else {
			$match_day = 1;
		}
		
		if (empty($match_day) || !is_numeric($match_day)) $match_day = 1;
		
		$this->match_day = intval($match_day);
		$this->match_query_args['match_day'] = $match_day;
	}
	
	
	/** 
	 * get maximum match day
	 *
	 * @return int
	 */
	public function getMaxMatchDay() {
		global $wpdb;
		
		$args = array( $this->id, $this->getSeason() );
		$sql = "SELECT MAX(match_day) FROM {$wpdb->leaguemanager_matches} WHERE  `league_id` = '%d' AND `season` = '%s'";
		if ( $this->getGroup() != "" && $this->groupExists(htmlspecialchars($this->getGroup())) ) {
			$sql .= " AND `group` = '%s'";
			$args[] = htmlspecialchars($this->getGroup());
		}
		
		$match_day = $wpdb->get_var( $wpdb->prepare($sql, $args) );
		return $match_day;
	}
	
	
	/**
	 * get pagination
	 *
	 * @param string $which
	 * @return string
	 */
	public function getPageLinks($which = 'matches') {
		$this->getCurrentPage($which);
		
		if ( $which == 'matches' ) {
			$base = is_admin() ? 'match_paged' : 'match_paged_'.$this->id;
			$num_pages = $this->num_pages_matches;
			$per_page = $this->num_matches_per_page;
			$current_page = $this->current_page_matches;
			$num_items = $this->num_matches;
		} elseif ( $which == 'teams' ) {
			$base = is_admin() ? 'team_paged' : 'team_paged_'.$this->id;
			$num_pages = $this->num_pages_teams;
			$per_page = $this->num_teams_per_page;
			$current_page = $this->current_page_teams;
			$num_items = $this->num_matches;
		} else {
			return '';
		}
		
	
		$query_args = is_admin() ? array('league_id' => $this->id) : $this->query_args;
		
		if ($which == 'matches' && isset($_POST['match_day']) && is_string($_POST['match_day'])) {
			$query_args['match_day'] = htmlspecialchars(strip_tags($_POST['match_day']));
		}

		$page_links = paginate_links( array(
			'base' => add_query_arg( $base, '%#%' ),
			'format' => '',
			'prev_text' => '&#9668;',
			'next_text' => '&#9658;',
			'total' => $num_pages,
			'current' => $current_page,
			'add_args' => $query_args
		));
		
		if ( $page_links && is_admin() ) {
			/*$page_links = sprintf( '<span class="displaying-num">' . __( 'Displaying %s&#8211;%s of %s', 'leaguemanager' ) . '</span>',
				number_format_i18n( ( $current_page - 1 ) * $per_page + 1 ),
				number_format_i18n( min( $current_page * $per_page,  $num_items ) ),
				number_format_i18n(  $num_items ),
				$page_links
			);*/
			$page_links = sprintf( '<span class="displaying-num">' . __( '%s Matches', 'leaguemanager' ) . '</span>%s',
				number_format_i18n(  $num_items ),
				$page_links
			);
		}
		
		return $page_links;
	}
	
	
	/**
	 * set number of pages for matches
	 *
	 * @param string $which
	 */
	public function setNumPages($which='matches') {
		if ( $which == 'matches' ) {
			$this->num_pages_matches = ( $this->num_matches_per_page == 0 ) ? 1 : ceil( $this->num_matches/$this->num_matches_per_page );
			if ( $this->num_pages_matches == 0 ) $this->num_pages_matches = 1;
		}
		
		if ( $which == 'teams' ) {
			$this->num_pages_teams = ( $this->num_teams_per_page == 0 ) ? 1 : ceil( $this->num_teams/$this->num_teams_per_page );
			if ( $this->num_pages_teams == 0 ) $this->num_pages_teams = 1;
		}
	}
	
	
	/**
	 * retrieve current page
	 *
	 * @param string $which
	 */
	public function getCurrentPage($which='matches') {
		global $wp;
		
		$this->setNumPages($which);
		
		if ( $which == 'matches' )
			$key = 'match_paged';
		elseif ( $which == 'teams' )
			$key = 'team_paged';
		
		if (isset($_GET[$key]))
			$current_page = intval($_GET[$key]);
		elseif (isset($wp->query_vars[$key]))
			$current_page = max(1, intval($wp->query_vars[$key]));
		elseif (isset($_GET[$key."_".$this->id]))
			$current_page = intval($_GET[$key."_".$this->id]);
		elseif (isset($wp->query_vars[$key."_".$this->id]))
			$current_page = max(1, intval($wp->query_vars[$key."_".$this->id]));
		else
			$current_page = 1;
		
		if ( $which == 'matches' && $current_page > $this->num_pages_matches )
			$current_page = $this->num_pages_matches;	
		if ( $which == 'teams' && $current_page > $this->num_pages_teams )
			$current_page = $this->num_pages_teams;
		
		// Prevent negative offsets
		if ( $current_page == 0 )
			$current_page = 1;
		
		if ( $which == 'matches' )
			$this->current_page_matches = $current_page;
		if ( $which == 'teams' )
			$this->current_page_teams = $current_page;
	}
	
	
	/**
	 * get teams from database
	 *
	 * @param array $query_args
	 * @return array
	 */
	public function getTeams( $query_args = array() ) {
		global $wpdb;
		
		$old_query_args = $this->team_query_args;
		
		// set query args
		foreach ($query_args AS $key => $value)
			$this->setTeamQueryArg($key, $value);
			
		extract($this->team_query_args, EXTR_SKIP);

		$args = array($this->id);
		$sql = "SELECT `title`, `website`, `coach`, `stadium`, `logo`, `home`, `group`, `roster`, `profile`, `points_plus`, `points_minus`, `points2_plus`, `points2_minus`, `add_points`, `done_matches`, `won_matches`, `draw_matches`, `lost_matches`, `diff`, `league_id`, `id`, `season`, `rank`, `status`, `custom` FROM {$wpdb->leaguemanager_teams} WHERE `league_id` = '%d'";
		
		
		if ( $season == "" ) {
			$sql .= " AND `season` = '%s'";
			$args[] = $this->current_season['name'];
		} else {
			if ($season == "any") {
				$sql .= " AND `season` != ''";
			} elseif ($this->seasonExists(htmlspecialchars($season))) {
				$sql .= " AND `season` = '%s'";
				$args[] = htmlspecialchars($season);
			}
		}
		
		// specific group
		if ($group != "" && $this->groupExists(htmlspecialchars($group))) {
			$sql .= " AND `group` = '%s'";
			$args[] = htmlspecialchars($group);
		}
		
		// given rank
		if ( intval($rank) > 0 ) {
			$sql .= " AND `rank` = '%d'";
			$args[] = intval($rank);
		}
		
		// home teams
		if ( $home === true ) {
			$sql .= " AND `home` = 1";
		}
		
		// specific team IDs
		if ( count($ids) > 0 ) {
			$ids = array_map('intval',$ids);
			$sql .= " AND `id` IN(".implode(",", $ids).")";
		}
		
		// create ordering string
		$orderby_string = ""; $i = 0;
		foreach ($orderby AS $order => $direction) {
			if (!in_array($direction, array("DESC", "ASC", "desc", "asc"))) $direction = "ASC";
			if ($this->databaseColumnExists("teams", $order)) {
				$orderby_string .= "`".$order."` ".$direction;
				if ($i < (count($orderby)-1)) $orderby_string .= ",";
			}
			$i++;
		}
		$order = $orderby_string;
		
		$offset = intval($limit > 0) ? ( $this->current_page_teams - 1 ) * $limit : 0;
		$sql .= " ORDER BY $order";
		if ( intval($limit > 0) ) $sql .= " LIMIT ".intval($offset).",".intval($limit)."";

		$sql = $wpdb->prepare($sql, $args);
		
		$teams = wp_cache_get( md5($sql), 'teams' );
		if (!$teams || $cache === false) {
			$teams = $wpdb->get_results( $sql );
			wp_cache_add( md5($sql), $teams, 'teams' );
		}
		
		$class = ''; $team_index = array();
		foreach ( $teams AS $i => $team ) {
			$team = get_team($team);
			
			$class = ( 'alternate' == $class ) ? '' : 'alternate';
			
			// Add class for home team
			if ( 1 == $team->home ) $team->cssClass[] = 'homeTeam';
			if ( $team->rank > 0 ) {
				// Add classes for ascend or descend
				if ( $team->rank <= $this->num_ascend ) $team->cssClass[] = 'ascend';
				elseif ( count($teams)-$team->rank < $this->num_descend ) $team->cssClass[] =  'descend';

				// Add class for relegation
				if ( $team->rank > count($teams)-$this->num_descend-$this->num_relegation && $team->rank <= count($teams)-$this->num_descend ) $team->cssClass[] = 'relegation-down';
				if ( $team->rank > $this->num_ascend && $team->rank <= $this->num_ascend + $this->num_relegation ) $team->cssClass[] = 'relegation-up';
			}
			
			$team->cssClass[] = $class;
			$team->class = implode(' ', $team->cssClass);
			
			if ( 1 == $team->home ) $team->title = '<strong>'.$team->title.'</strong>';

			$team->pointsFormatted = array( 'primary' => sprintf($this->point_format, $team->points_plus, $team->points_minus), 'secondary' => sprintf($this->point_format2, $team->points2_plus, $team->points2_minus) );
			
			$team_index[$team->id] = $i;
			$teams[$i] = $team;
		}
		
		$this->teams = $teams;
		$this->team_index = $team_index;
		
		// set number of teams
		$this->setNumTeams();
		
		// reset query args
		//$this->team_query_args = $old_query_args;
	
		return $teams;
	}
	
	
	/**
	 * get single team from League cache
	 *
	 * @param int $team_id
	 * @return Team object
	 */
	public function getTeam( $team_id ) {
		if (isset($this->team_index[$team_id])) {
			return $this->teams[$this->team_index[$team_id]];
		} else {
			return get_team($team_id);
		}
	}
	
	
	/**
	 * get matches from database
	 * 
	 * @param array $query_args
	 * @return array
	 */
	public function getMatches( $query_args = array() ) {
	 	global $wpdb;
		
		$old_query_args = $this->match_query_args;

		// set query args
		foreach ($query_args AS $key => $value)
			$this->setMatchQueryArg($key, $value, true);

		extract($this->match_query_args, EXTR_SKIP);
		
		// get all teams to reduce the number of MySQL queries here for getting home/away team
		if ( empty($this->teams) ) {
			$this->getTeams();
		}
		
		
		$args = array($this->id);
		if ( $count === true )
			$sql = "SELECT COUNT(ID) FROM {$wpdb->leaguemanager_matches} WHERE `league_id` = '%d'";
		else
			$sql = "SELECT `final` AS final_round, `group`, `home_team`, `away_team`, DATE_FORMAT(`date`, '%%Y-%%m-%%d %%H:%%i') AS date, DATE_FORMAT(`date`, '%%e') AS day, DATE_FORMAT(`date`, '%%c') AS month, DATE_FORMAT(`date`, '%%Y') AS year, DATE_FORMAT(`date`, '%%H') AS `hour`, DATE_FORMAT(`date`, '%%i') AS `minutes`, `match_day`, `location`, `status`, `league_id`, `home_points`, `away_points`, `winner_id`, `loser_id`, `stat_id`, `post_id`, `season`, `id`, `custom` FROM {$wpdb->leaguemanager_matches} WHERE `league_id` = '%d'";
		
		
		// disable limit for championship 
		if ( $this->mode == "championship" ) $limit = 0;
		
		if ( $season == "" ) {
			$sql .= " AND `season` = '%s'";
			$args[] = $this->current_season['name'];
		} else {
			if ($season == "any") {
				$sql .= " AND `season` != ''";
			} elseif ($this->seasonExists(htmlspecialchars($season))) {
				$sql .= " AND `season` = '%s'";
				$args[] = htmlspecialchars($season);
			}
		}
		
		// specific final
		if ($final != false && $this->finalExists(htmlspecialchars(strip_tags($final)))) {
			$sql .= " AND `final` = '%s'";
			$args[] = htmlspecialchars(strip_tags($final));
			// remove match_day query for final matches
			$match_day = -1;
			// no limit
			$limit = 0;
			// no group
			$group = '';
		} else {
			$sql .= " AND `final` = ''";
		}
		
		// specific group
		if ( $group != "" && $this->groupExists(htmlspecialchars($group))) {
			$sql .= " AND `group` = '%s'";
			$args[] = htmlspecialchars($group);
		} elseif ( $group == 'empty' ) {
			$sql .= " AND `group` = ''";
		}
		
		// specific team ID or home/away team
		if (intval($team_id) > 0) {
			$sql .= " AND (`home_team` = '%d' OR `away_team` = '%d')";
			$args[] = $team_id;
			$args[] = $team_id;
		} elseif (count($team_pair) == 2) {
			$sql .= " AND ( (`home_team` = '%d' AND `away_team` = '%d' ) OR (`home_team` = '%d' AND `away_team` = '%d' ) )";
			$args[] = intval($team_pair[0]);
			$args[] = intval($team_pair[1]);
			$args[] = intval($team_pair[1]);
			$args[] = intval($team_pair[0]);
		} else {
			if (!empty($home_team)) {
				$sql .= " AND `home_team` = '%s'";
				$args[] = $home_team;
			}
			if (!empty($away_team)) {
				$sql .= " AND `away_team` = '%s'";
				$args[] = $away_team;
			}
		}
		
		// specific match day or up-to match day
		if ( intval($match_day) > 0 ) {
			if ( $standingstable === true ) {
				$sql .= " AND `match_day` <= '%d'";
				$args[] = $match_day;
			} else {
				$sql .= " AND `match_day` = '%d'";
				$args[] = $match_day;
			}
		}
		
		// get only finished matches with score for time 'latest'
		if ( $time == 'latest' ) {
			$home_points = $away_points = false;
			$sql .= " AND (`home_points` != '' OR `away_points` != '')";
		}
		
		if ($home_points != "") {
			if ($home_points == "null")
				$sql .= " AND `home_points` IS NULL";
			elseif ($home_points == "not_null")
				$sql .= " AND `home_points` IS NOT NULL";
			elseif ($home_points == "not_empty")
				$sql .= " AND `home_points` != ''";
		}
		if ($away_points != "") {
			if ($away_points == "null")
				$sql .= " AND `away_points` IS NULL";
			elseif ($away_points == "not_null")
				$sql .= " AND `away_points` IS NOT NULL";
			elseif ($away_points == "not_empty")
				$sql .= " AND `away_points` != ''";
		}
		
		if ($winner_id) {
			$sql .= " AND `winner_id` = '%d'";
			$args[] = $winner_id;
		}
		if ($loser_id) {
			$sql .= " AND `loser_id` = '%d'";
			$args[] = $loser_id;
		}
		
		if ( $time == 'next' ) {
			$sql .= " AND TIMESTAMPDIFF(MINUTE, NOW(), `date`) >= 0";
		} elseif ( $time == 'prev' || $time == 'latest' ) {
			$sql .= " AND TIMESTAMPDIFF(MINUTE, NOW(), `date`) < 0";
		} elseif ( $time == 'prev1' ) {
			$sql .= " AND TIMESTAMPDIFF(MINUTE, NOW(), `date`) < 0) AND (`winner_id` != 0) ";
		} elseif ( $time == 'today' ) {
			$sql .= " AND DATEDIFF(NOW(), `date`) = 0";
		} elseif ( $time == 'day' ) {
			$sql .= " AND DATEDIFF('". htmlspecialchars(strip_tags($match_date))."', `date`) = 0";
		}
		
		if ($home_only === true) {
			$sql .= $this->buildHomeOnlyQuery();
		}
			
		// Force ordering by date ascending if next matches are queried
		if ( $time == 'next' ) {
			$orderby['date'] = 'ASC';
		}
		// Force ordering by date descending if previous/latest matches are queried
		if ( $time == 'prev' || $time == 'latest' ) {
			$orderby['date'] = 'DESC';
		}
		
		// get number of matches
		if ( $count === true ) {
			$this->setMatchQueryArg('count', false);
			$sql = $wpdb->prepare($sql, $args);
			
			// Use Wordpress cache for counting matches
			$matches = wp_cache_get( md5($sql), 'num_matches' );
			if (!$matches || $cache === false) {
				$matches = intval($wpdb->get_var($sql));
				wp_cache_add( md5($sql), $matches, 'num_matches' );
			}
		} else {
			$orderby_string = ""; $i = 0;
			foreach ($orderby AS $order => $direction) {
				if (!in_array($direction, array("DESC", "ASC", "desc", "asc"))) $direction = "ASC";
				if ($this->databaseColumnExists("matches", $order)) {
					$orderby_string .= "`".$order."` ".$direction;
					if ($i < (count($orderby)-1)) $orderby_string .= ",";
				}
				$i++;
			}
			$order = $orderby_string;
			
			$offset = intval($limit > 0) ? ( $this->current_page_matches - 1 ) * $limit : 0;
			$sql .= " ORDER BY $order";
			if ( intval($limit > 0) ) $sql .= " LIMIT ".intval($offset).",".intval($limit)."";
			
			$sql = $wpdb->prepare($sql, $args);
			
			$matches = wp_cache_get( md5($sql), 'matches' );
			if (!$matches || $cache === false) {
				$matches = $wpdb->get_results( $sql );
				wp_cache_add( md5($sql), $matches, 'matches' );
			}
			
			$class = '';
			foreach ( $matches AS $i => $match ) {
				$match = get_match($match);
				
				$class = ( 'alternate' == $class ) ? '' : 'alternate';
				$match->class = $class;

				$matches[$i] = $match;
			}
			
			if ($limit == 1 && $matches) $matches = $matches[0];
		}
		
		// reset match limit
		if ($reset_limit === true) {
			$this->setMatchQueryArg('limit', $old_query_args['limit']);
			$this->setMatchQueryArg('reset_limit', false);
		}
		
		// reset match query args
		if ($reset_query_args === true) {
			//$this->match_query_args = $old_query_args;
			foreach ($old_query_args AS $key => $query_arg)
				$this->setMatchQueryArg($key, $query_arg, true);
				
			$this->setMatchQueryArg('reset_query_args', false);
		}
		
		if ( $count !== true) {
			$this->matches = $matches;
		} elseif ($count === true) {
			$this->num_matches = $matches;
		}
		
		return $matches;
	}
	
	
	/**
	 * build home only query
	 *
	 * @return string MySQL search query or false
	 */
	function buildHomeOnlyQuery() {
		global $wpdb;
			
		$sql = $wpdb->prepare("SELECT `id` FROM {$wpdb->leaguemanager_teams} WHERE `league_id` = '%d' AND `home` = 1", intval($this->id));
		$teams = wp_cache_get( md5($sql), 'teams' );
		if (!$teams) {
			$teams = $wpdb->get_results( $sql );
			wp_cache_add( md5($sql), $teams, 'teams' );
		}
		
		if ( $teams ) {
			$queries = array();
			foreach ( $teams AS $team )
				$queries[] = $wpdb->prepare("`home_team` = '%d' OR `away_team` = '%d'", $team->id, $team->id);
		
			$query = " AND (".implode(" OR ", $queries).")";
			
			return $query;
		}
		
		return false;
	}
	
	
	/**
	 * get standings table
	 *
	 * @param array $teams
	 * @param int $match_day
	 * @param string $mode
	 * @param string $group
	 * @return array the ranked teams
	 */
	public function getStandings( $teams = false, $match_day = -1, $mode = 'all', $group = '' ) {
		if ( !$teams ) $teams = $this->teams;
		
		$rule = $this->getPointRule();
		extract( (array)$rule );
		
		// hide status as it's meaningless
		$this->standings['status'] = 0;

		// set basic match query args
		$this->setMatchQueryArg('standingstable', true);
		$this->setMatchQueryArg('match_day', $match_day);
		$this->setMatchQueryArg('final', '');
		$this->setMatchQueryArg('limit', false);
		$this->setMatchQueryArg('group', $group);
		$this->setMatchQueryArg('mode', $mode);
		
		foreach ( $teams AS $i => $team ) {
			$match_args = array();

			// get only home matches
			if ( $mode == "home" )
				$match_args['home_team'] = $team->id;
			// get only away matches
			if ( $mode == "away" )
				$match_args['away_team'] = $team->id;
			// get all matches for given team
			if ( $mode == "all" )
				$match_args['team_id'] = $team->id;

			// initialize team standings data
			$team->done_matches = 0;
			$team->won_matches = 0;
			$team->draw_matches = 0;
			$team->lost_matches = 0;
			$team->points_plus = 0;
			$team->points_minus = 0;
			$team->points2_plus = 0;
			$team->points2_minus = 0;
			$points = array( 'plus' => 0, 'minus' => 0 );
			$points2 = array( 'plus' => 0, 'minus' => 0 );
			$team_points = 0;
			
			// get matches
			$matches = $this->getMatches( $match_args );
			foreach ( $matches AS $match ) {
				if ( $match->home_points != "" && $match->away_points != "" )
					$team->done_matches += 1;
				
				if ( $match->winner_id == $team->id )
					$team->won_matches += 1;
				
				if ( $match->loser_id == $team->id )
					$team->lost_matches += 1;
				
				if ( $match->winner_id == -1 && $match->loser_id == -1 )
					$team->draw_matches += 1;
			}
			
			$team->points = $this->calculatePoints( $team, $matches );
			$team->points_plus = $team->points['plus'];
			$team->points_minus = $team->points['minus'];
			
			$team->points2 = $team->custom['points2'] = $this->calculateSecondaryPoints( $team, $matches );
			$team->points2_plus = $team->points2['plus'];
			$team->points2_minus = $team->points2['minus'];
			
			$team->diff = $team->custom['diff'] = $team->points2_plus - $team->points2_minus;
			$team->winPercent();
			
			$custom = $this->getStandingsData( $team, $team->custom, $matches );
			foreach ( $custom AS $key => $value )
				$team->{$key} = $value;
			
			$teams[$i] = $team;
		}
		
		// rank teams
		$teams = $this->rankTeams($teams);
		$teams = $this->getRanking($teams);
		
		return $teams;
	}
	

	/**
	 * get standings selection
	 *
	 * @return string
	 */
	public function getStandingsSelection() {
		$selected = isset($_GET['standingstable']) ? htmlspecialchars($_GET['standingstable']) : '';
		
		$options = array( 'all' => __( 'Current Table', 'leaguemanager' ), 'home' => __( 'Hometable', 'leaguemanager' ), 'away' => __( 'Awaytable', 'leaguemanager' ) );
		$action = is_admin() ? menu_page_url('leaguemanager', 0)."&amp;subpage=show-league&amp;league_id=".$this->id : get_permalink();
		$out = "<select size='1' name='standingstable'>";
		foreach ( $options AS $value => $label ) {
			$out .= "<option value='".$value."'".selected($value, $selected, false).">".$label."</option>";
		}
		for ( $day = 1; $day <= $this->current_season['num_match_days']; $day++ ) {
			$out .= "<option value='match_day-".$day."'".selected("match_day-".$day, $selected, false).">".sprintf(__("%d. Match Day", 'leaguemanager'), $day)."</option>";
		}
		$out .= "</select>";
		
		return $out;
	}
	
	
	/**
	 * set finals flag for championship mode
	 *
	 * @param boolean $is_final
	 */
	public function setFinals($is_final = true) {
		$this->is_final = $is_final;
	}
	
	
	/**
	 * check if finals flag is set
	 *
	 * @return boolean
	 */
	public function isFinals() {
		if (!isset($this->is_final)) return false;
		
		return intval($this->is_final) == 1;
	}
	
	
	/**
	 * get point rule depending on selection.
	 * For details on point rules see http://de.wikipedia.org/wiki/Drei-Punkte-Regel (German)
	 *
	 * @param mixed $rule
	 * @return array
	 */
	public function getPointRule( $rule = false ) {
		if (!$rule) $rule = $this->point_rule;
		$rule = maybe_unserialize($rule);

		// Manual point rule
		if ( is_array($rule) ) {
			return $rule;
		} else {
			$point_rules = array();
			// One point rule
			$point_rules['one'] = array( 'forwin' => 1, 'fordraw' => 0, 'forloss' => 0 );
			// Two point rule
			$point_rules['two'] = array( 'forwin' => 2, 'fordraw' => 1, 'forloss' => 0 );
			// Three-point rule
			$point_rules['three'] = array( 'forwin' => 3, 'fordraw' => 1, 'forloss' => 0 );
			// Score. One point for each scored goal
			$point_rules['score'] = 'score';

			/**
			 * Fired when point rules are retrieved
			 *
			 * @param array $point_rules
			 * @return array
			 * @category wp-filter
			 */
			$point_rules = apply_filters( 'leaguemanager_point_rules', $point_rules );

			return $point_rules[$rule];
		}
	}
	
	
	/**
	 * get number of teams for specific league
	 *
	 * @param boolean $total
	 */
	public function setNumTeams($total = false) {
		global $wpdb;
		
		if ($total === true) {
			// get total number of teams
			$this->num_teams_total = $wpdb->get_var( $wpdb->prepare("SELECT COUNT(ID) FROM {$wpdb->leaguemanager_teams} WHERE `league_id` = '%d' AND `season`= '%s'", $this->id, $this->current_season['name']) );
		} else {
			// get number of teams for each group
			if ($this->groups != "") {
				$groups = $this->getGroups();
				foreach ($groups AS $group) {
					$this->num_teams_by_group[$group] = $wpdb->get_var( $wpdb->prepare("SELECT COUNT(ID) FROM {$wpdb->leaguemanager_teams} WHERE `league_id` = '%d' AND `season` = '%s' AND `group` = '%s'", $this->id, $this->current_season['name'], $group) );
				}
			}
			
			$group = $this->team_query_args['group'];
			if ( $group != "" && isset($this->num_teams_by_group[$group]))
				$this->num_teams = $this->num_teams_by_group[$group];
			else
				$this->num_teams = $this->num_teams_total;//$wpdb->get_var( $wpdb->prepare("SELECT COUNT(ID) FROM {$wpdb->leaguemanager_teams} WHERE `league_id` = '%d' AND `season` = '%s'", $this->id, $this->current_season['name']) );
			
			$this->pagination_teams = $this->getPageLinks('teams');
		}
	}
	
	
	/**
	 * gets number of matches
	 *
	 * @param boolean $total
	 */
	public function setNumMatches($total=false) {
		global $wpdb;
		
		if ($total === true) {
			$this->num_matches_total = $wpdb->get_var( $wpdb->prepare("SELECT COUNT(ID) FROM {$wpdb->leaguemanager_matches} WHERE `league_id` = '%d'", $this->id) );
		} else {
			$this->getMatches(array("limit" => 0, "count" => true));
			$this->pagination_matches = $this->getPageLinks('matches');
		}
	}
	
	
	/**
	 * data loader
	 *
	 * @param string $param
	 */
	public function loadData($param) {
		if ($param == "teams" && empty($this->teams))
			$this->getTeams();
		
		if ($param == "matches" && empty($this->matches))
			$this->getMatches();
	}
	
	
	/**
	 * data getter
	 *
	 * @param string $param which parameter "formfields" or "categories"
	 * @param string $column
	 * @param string $search
	 * @param int $limit
	 * @param boolean $load
	 * @return array
	 */
	public function getData($param, $column="", $search="", $limit=0, $load = true) {
		// make sure that corresponding data is loaded from database
		if ( $load ) $this->loadData($param);
		
		if ($param == "matches")
			$data = $this->matches;
		elseif ($param == "teams")
			$data = $this->teams;
		else
			return false;
		
		$data2 = $data;
		if ($column != "") {
			$data2 = array_column($data, $column);
		}
		
		if ( $search === 0 || $search != "" ) {
			$d = array();
			foreach (array_keys($data2, $search) AS $key) {
				$d[] = $data[$key];
			}
			$data = $d;
		}

		foreach ($data AS $i => $d) {
			$data[$i] = (object)$d;
		}
		
		if (intval($limit) > 0) $data = array_slice($data, 0, $limit);
		if (intval($limit) == 1) $data = $data[0];
		
		return $data;
	}
	
	
	/**
	 * get crosstable match
	 *
	 * @param int $curr_team_id
	 * @param int $opponent_id
	 * @return Match
	 */
	private function getCrosstableMatch( $curr_team_id, $opponent_id ) {
		
	}
	
	
	/**
	 * get specific field for crosstable
	 *
	 * @param int $curr_team_id
	 * @param int $opponent_id
	 * @param int $home
	 * @return string
	 */
	public function getCrosstableField($curr_team_id, $opponent_id, $home = 0) {
		if ( $curr_team_id == $opponent_id )
			return '-';
		
		//$match = $this->getMatches( array("home_team" => $curr_team_id, "away_team" => $opponent_id, "match_day" => -1, "limit" => false, "reset_query_args" => true) );
		$team_pair = array($curr_team_id, $opponent_id);
		sort($team_pair);
		$match = $this->getMatches( array("team_pair" => $team_pair, "match_day" => -1, "limit" => false, "reset_query_args" => true) );
		if ($match) $match = $match[0];
		
		$score = $this->getScore($curr_team_id, $match, $home);

 		/*if ( $match ) {
			$score = $this->getScore($curr_team_id, $match, $home);
		} else {
			$match = $this->getMatches( array("home_team" => $opponent_id, "away_team" => $curr_team_id, "match_day" => -1, "limit" => false, "reset_query_args" => true) );
			if ($match) $match = $match[0];
			$score = $this->getScore($curr_team_id, $match, $home);
		}*/
		
		return $score;
	}
	
	
	/**
	 * get score for specific field of crosstable
	 *
	 * @param int $team_id
	 * @param Match $match
	 * @param int $home
	 * @return string
	 */
	public function getScore($team_id, $match, $home = 0) {
		if ( !$match ) return '-:-';
		
		if ( $team_id == $match->home_team )
			$out = sprintf("%s:%s", $match->homeScore, $match->awayScore);
		else
			$out = sprintf("%s:%s", $match->awayScore, $match->homeScore);
		
		if ( $home == 1 ) $out = "<strong>".$out."</strong>";
		
		return $out;
	}
	
	
	/**
	 * get image directory
	 *
	 * @param string|false $file
	 * @param string $size
	 * @return string
	 */
	public function getImagePath( $file = false, $size = 'full' ) {
		$base = WP_CONTENT_DIR.'/uploads/leagues/League-'.$this->id;

		if ( $file ) {
			if ( $size == 'full' || $size == '' )
				$file = basename($file);
			else
				$file = $size . "_" . basename($file);
			
			return $base .'/'. $file;
		} else {
			return $base;
		}
	}
	
	
	/**
	 * get url of image url
	 *
	 * @param string|false $file image file
	 * @param string $size
	 * @return string
	 */
	public function getImageURL( $file = false, $size = 'full' ) {
		$base = WP_CONTENT_URL.'/uploads/leagues/League-'.$this->id;

		if ( $file ) {
			if ( $size == 'full' || $size == '' )
				$file = basename($file);
			else
				$file = $size . "_" . basename($file);
			
			if (file_exists($this->getImagePath($file, '')))
				return esc_url($base .'/'. $file);
			else
				return false;
		} else {
			return esc_url($base);
		}
	}
	
	
	/**
	 * set match day selection in shortcode
	 *
	 * @param string $match_day_selection
	 * @param int $match_day
	 */
	public function setMatchDaySelection($match_day_selection, $match_day = -1) {
		if (intval($match_day) > 0) {
			$this->show_match_day_selection = false;
		} else {
			if ($match_day_selection == 'true')
				$this->show_match_day_selection = true;
			else if ($match_day_selection == 'false')
				$this->show_match_day_selection = false;
		}
	}
	
	
	/**
	 * set team selection in shortcode
	 *
	 * @param string $team_selection
	 * @param int $team_id
	 */
	public function setTeamSelection($team_selection, $team_id = 0) {
		if (intval($team_id) > 0) {
			$this->show_team_selection = false;
		} else {
			if ($team_selection == 'true')
				$this->show_team_selection = true;
			
			if ($team_selection == 'false')
				$this->show_team_selection = false;
		}
	}
	
	
	/**
	 * set matches selection in shortcode.
	 *
	 * @param string $show_match_day_selection
	 * @param int $match_day
	 * @param string $show_team_selection
	 * @param int $team_id
	 */
	public function setMatchesSelection($show_match_day_selection, $match_day, $show_team_selection, $team_id) {
		$this->setMatchDaySelection($show_match_day_selection, $match_day);
		$this->setTeamSelection($show_team_selection, $team_id);
		
		if (($this->show_match_day_selection || $this->show_team_selection) && $this->mode != 'championship')
			$this->show_matches_selection = true;
		else
			$this->show_matches_selection = false;
	}
	
	
	/**
	 * set tab in league/archive shortcodes
	 *
	 * @param boolean $is_archive
	 */
	public function setTab($is_archive = false) {
		if ( isset($_GET['match_day_'.$this->id]) || isset($_GET['team_id_'.$this->id]) || isset($_GET['match_paged_'.$this->id]) ) 
			$this->archive_tab = 2;
		if ( isset($_GET['team_'.$this->id]) )
			$this->archive_tab = 3;
		if ( isset($_GET['match_'.$this->id]) )
			$this->archive_tab = 2;
		if ( isset($_GET['team_paged_'.$this->id]) || isset($_GET['show_'.$this->teamroster]) || isset($_GET['paged_'.$this->teamroster]) )
			$this->archive_tab = 3;
		
		$this->is_archive = $is_archive;
	}
	
	
	/**
	 * set template in league/archive shortcodes
	 *
	 * @param string $key
	 * @param string $template
	 */
	public function setTemplate($key, $template) {
		$this->templates[$key] = $template;
	}
	
	
	/**
	 * set all templates in league/archive shortcodes
	 *
	 * @param array $templates An associative array of templatey key => template associations
	 */
	public function setTemplates($templates) {
		foreach ($templates AS $key => $template)
			$this->setTemplate($key, $template);
	}
	
	
	/**
	 * check if season exists
	 *
	 * @param string $season
	 * @return boolean
	 */
	private function seasonExists($season) {
		if (is_array($this->seasons) && in_array($season, array_keys($this->seasons)))
			return true;
		else
			return false;
	}
	
	/**
	 * check if group exists
	 *
	 * @param string $group
	 * @return boolean
	 */
	private function groupExists($group) {
		if (isset($this->groups)) {
			$groups = $this->getGroups();
			if (in_array($group, $groups))
				return true;
		}
		return false;
	}
	
	/**
	 * check if final exists
	 *
	 * @param string $final
	 * @return boolean
	 */
	private function finalExists($final) {
		if (! $this->championship instanceof League_Championship)
			return false;
		
		$finals = $this->championship->getFinals();
		if (in_array($final, array_keys($finals)))
			return true;
		else
			return false;
	}
	
	/**
	 * check if database column exists
	 *
	 * @param string $table
	 * @param string $column
	 * @return boolean
	 */
	private function databaseColumnExists($table, $column) {
		global $wpdb;
		
		if ($table == "teams")
			$table = $wpdb->leaguemanager_teams;
		elseif ($table == "matches")
			$table = $wpdb->leaguemanager_matches;
		else
			return false;
		
		$sql = $wpdb->prepare("SHOW COLUMNS FROM {$table} LIKE %s", $column);
		
		$res = wp_cache_get( md5($sql), 'leaguemanager' );
		
		if ( !$res ) {
			$num = $wpdb->query( $sql );
			$res = ( $num == 1 ) ? true : false;
			
			wp_cache_add( md5($sql), $res, 'leaguemanager' );
		}
		
		return $res;
	}
	
	
	/**
	 * rank teams
	 *
	 * @param boolean $update
	 * @return array $teams ranked teams
	 */
	public function _rankTeams( $update=true ) {
		// rank Teams in groups
		$groups = $this->getGroups();
		if (empty($groups)) $groups = array('');
		
		foreach ( $groups AS $group ) {
			$team_args = array("cache" => false);
			if ( $group != "" ) $team_args["group"] = $group;

			$teams = $this->getTeams( $team_args );
			
			if ( !empty($teams) && $this->team_ranking == 'auto' ) {				
				$teams = $this->rankTeams($teams);
				$teams = $this->getRanking($teams);
				if ( $update ) $this->updateRanking($teams);
			}
		}
	}
	
	
	/**
	 * Update Team Rank and status
	 * Modified from original code by LaMonte Forthun
	 *
	 * @param array $teams
	 */
	public function getRanking( $teams ) {
		$rank = 1;
		$incr = 1;
		$new_teams = array();
		foreach ( $teams AS $key => $team ) {
			$team->oldRank = $team->rank;
			
			if ($key > 0) {
				// check if current team is tied with previous team
				if ($this->isTie($team, $teams[$key-1])) {
					$incr += 1;
				} else {
					$rank += $incr;
					$incr = 1;
				}
			}

			$team->rank = $rank;
			$team->status = $this->getTeamStatus($team, $rank);
			
			$new_teams[$key] = $team;
		}
		
		// run tie breaks
		$new_teams = $this->tiebreak($new_teams);
		
		return $new_teams;
	}
	
	
	/**
	 * get team status depending on previous rank
	 *
	 * @param Team $team
	 * @param int $rank
	 * @return string
	 */
	private function getTeamStatus( $team, $rank ) {
		if ( $team->oldRank != 0 && $team->done_matches > 1 ) {
			if ( $rank == $team->oldRank ){
				$status = '&#8226;';
			} elseif ( $rank < $team->oldRank ){
				$status = '&#8593;';
			} else{
				$status = '&#8595;';
			}
		} else {
			$status = '&#8226;';
		}
		
		return $status;
	}
	
	
	/**
	 * update team ranks in database
	 *
	 * @param array $teams
	 */
	public function updateRanking( $teams ) {
		global $wpdb;
		
		foreach ( $teams AS $team ) {
			$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->leaguemanager_teams} SET `rank` = '%d', `status` = '%s' WHERE `id` = '%d'", $team->rank, $team->status, $team->id ) );
		}
	}
	
	
	/**
	 * get season dropdown menu
	 *
	 * @param string $season selected season
	 * @return string
	 */
	public function getSeasonDropdown($season = '') {
		$out = '<select size="1" class="alignleft" id="season" name="season" onChange="Leaguemanager.getMatchesDropdown(\''.get_bloginfo( 'wpurl' ).'/wp-admin/admin-ajax.php\', '.$this->id.', this.value);">';
		$out .= '<option value="">'.__('Choose Season', 'leaguemanager').'</option>';
		foreach ( $this->seasons AS $s ) {
			$out .= '<option value="'.$s['name'].'" '.selected($season, $s['name'], false).'>'.$s['name'].'</option>';
		}
		$out .= '</select>';
		
		return $out;
	}
	
	
	/**
	 * get matches dropdown menu
	 *
	 * @param int $match_id selected match ID
	 * @return string
	 */
	public function getMatchesDropdown($match_id = 0) {
		$matches = $this->getMatches( array("season" => $this->current_season['name'], "limit" => false, "match_day" => -1, "reset_query_args" => true) );
		
		$out = '<select size="1" name="match_id" id="match_id" class="alignleft">';
		$out .= '<option value="0">'.__('Choose Match', 'leaguemanager').'</option>';
		foreach ( $matches AS $match ) {
			$out .= '<option value="'.$match->id.'" '.selected($match_id, $match->id, false).'>'.$match->getTitle(false).'</option>';
		}
		$out .= '</select>';
		
		return $out;
	}
	
	
	/**
	 * check if team roster project is set
	 *
	 * @return boolean
	 */
	public function hasTeamRoster() {
		if ( isset($this->teamroster) && intval($this->teamroster) > 0 )
			return true;
		
		return false;
	}
	
	
	/**
	 * check if team profile project is set
	 *
	 * @return boolean
	 */
	public function hasTeamProfiles() {
		if ( isset($this->teamprofiles) && intval($this->teamprofiles) > 0 )
			return true;
		
		return false;
	}
	
	
	/**
	 * check if match statistics project is set
	 *
	 * @return boolean
	 */
	public function hasMatchStatistics() {
		if ( isset($this->matchstats) && intval($this->matchstats) > 0 )
			return true;
		
		return false;
	}
	
	
	/**
	 * =======================
	 * Administation section
	 * =======================
	 */
	 
	/**
	 * update standings manually
	 *
	 * @param array $teams
	 * @param array $points_plus
	 * @param array $points_minus
	 * @param array $num_done_matches
	 * @param array $num_won_matches
	 * @param array $num_draw_matches
	 * @param array $num_lost_matches
	 * @param array $add_points
	 * @param array $custom
	 */
	public function saveStandingsManually( $teams, $points_plus, $points_minus, $num_done_matches, $num_won_matches, $num_draw_matches, $num_lost_matches, $add_points, $custom ) {
		global $wpdb;
		
		if (is_admin() && current_user_can('update_results')) {
			while ( list($id) = each($teams) ) {
				$points2_plus = isset($custom[$id]['points2']) ? intval($custom[$id]['points2']['plus']) : 0;
				$points2_minus = isset($custom[$id]['points2']) ? intval($custom[$id]['points2']['minus']) : 0;
				$diff = $points2_plus - $points2_minus;

				$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->leaguemanager_teams} SET `points_plus` = '%d', `points_minus` = '%d', `points2_plus` = '%d', `points2_minus` = '%d', `done_matches` = '%d', `won_matches` = '%d', `draw_matches` = '%d', `lost_matches` = '%d', `diff` = '%d', `add_points` = '%d', `custom` = '%s' WHERE `id` = '%d'", $points_plus[$id], $points_minus[$id], $points2_plus, $points2_minus, $num_done_matches[$id], $num_won_matches[$id], $num_draw_matches[$id], $num_lost_matches[$id], $diff[$id], $add_points[$id], maybe_serialize($custom[$id]), $id ) );
			}

			// Update Teams Rank and Status if not set to manual ranking
			if ($this->team_ranking != 'manual')
				$this->_rankTeams();
		}
	}
	
	
	/**
	 * update match results
	 *
	 * @param array $matches
	 * @param array $home_points
	 * @param array $away_points
	 * @param array $home_team
	 * @param array $away_team
	 * @param array $status
	 * @param array $custom
	 * @return boolean
	 */
	public function _updateResults($matches, $home_points, $away_points, $home_team, $away_team, $status, $custom) {
		global $wpdb;
		
		if ( is_admin() && current_user_can('update_results') ) {
			// update match results
			if ( !empty($matches) ) {
				while ( list($match_id) = each($matches) ) {
					$match = get_match( $match_id );

					// update match results, also updating match object for subsequent custom updates
					$c = isset($custom[$match_id]) ? $custom[$match_id] : array();
					$match->updateResults( $home_points[$match_id], $away_points[$match_id], $status[$match_id], $c );
					// custom results update
					$match = $this->updateResults( $match );

					$wpdb->query( $wpdb->prepare("UPDATE {$wpdb->leaguemanager_matches} SET `home_points` = '%s', `away_points` = '%s', `winner_id` = '%d', `loser_id` = '%d', `status` = '%s', `custom` = '%s' WHERE `id` = '%d'", $match->home_points, $match->away_points, $match->winner_id, $match->loser_id, $match->status, maybe_serialize($match->custom), $match_id) );
				}
			}

			// save team standings data
			$teams = $this->getTeams( array("cache" => false) );
			foreach ( $teams AS $i => $team ) {
				$teams[$i] = $this->saveStandings($team);
			}

			// Update Teams Rank and Status
			$this->_rankTeams();
			
			$this->setMatchQueryArgs();
			$this->setNumMatches(true); // get total number of matches
			$this->setNumMatches();
		
			return true;
		}
		
		return false;
	}
	
	
	/**
	 * update points for given team
	 *
	 * @param Team $team
	 * @return Team
	 */
	private function saveStandings( $team ) {
		global $wpdb;

		if ( $this->point_rule != 'manual' ) {
			$team->getNumDoneMatches();
			$team->getNumLostMatches();
			$team->getNumDrawMatches();
			$team->getNumWonMatches();

			$team->points = $this->calculatePoints( $team );
			$team->points2 = $team->custom['points2'] = $this->calculateSecondaryPoints( $team );
			$team->diff = $team->custom['diff'] = $team->points2['plus'] - $team->points2['minus'];
			
			// get custom team standings data
			$team->custom = $this->getStandingsData( $team, $team->custom );
			
			$wpdb->query ( $wpdb->prepare( "UPDATE {$wpdb->leaguemanager_teams} SET `points_plus` = '%d', `points_minus` = '%d', `points2_plus` = '%d', `points2_minus` = '%d', `done_matches` = '%d', `won_matches` = '%d', `draw_matches` = '%d', `lost_matches` = '%d', `diff` = '%d', `custom` = '%s' WHERE `id` = '%d'", $team->points['plus'], $team->points['minus'], $team->points2['plus'], $team->points2['minus'], $team->done_matches, $team->won_matches, $team->draw_matches, $team->lost_matches, $team->diff,maybe_serialize($team->custom), $team->id ) );
		}
		
		return $team;
	}
	
	
	/**
	 * calculate points for given team depending on point rule
	 *
	 * @param Team $team
	 * @param array|false $matches
	 * @return array|int
	 */
	private function calculatePoints( $team, $matches = false ) {
		$rule = $this->getPointRule();
		$points = array( 'plus' => 0, 'minus' => 0 );
		$team_points = 0;

		if ( !$matches ) $matches = $this->getMatches( array("team_id" => $team->id, "match_day" => -1, "limit" => 0, "cache" => false, 'reset_query_args' => true) );
		
		if ( 'score' == $rule ) {
			foreach ( $matches AS $match ) {
				if ($team->id == $match->home_team) {
					$points['plus'] += $match->home_points;
					$points['minus'] += $match->away_points;
				} else {
					$points['plus'] += $match->away_points;
					$points['minus'] += $match->home_points;
				}
			}
		} else {
			extract( (array)$rule );
			foreach ( $matches AS $match ) {
				if ($team->id == $match->home_team) {
					$team_points += $match->home_points;
				} else {
					$team_points += $match->away_points;
				}
			}

			$points['plus'] = $team->won_matches * $forwin + $team->draw_matches * $fordraw + $team->lost_matches * $forloss + ($team_points * (isset($forscoring) ? $forscoring : 0));
			$points['minus'] = $team->lost_matches * $forwin + $team->draw_matches * $fordraw + $team->won_matches * $forloss;
		}

		/**
		 * Fired when primary points are calculated
		 *
		 * @param array $points
		 * @param int $team_id
		 * @param mixed $rule
		 * @param array $matches
		 * @return array
		 * @category wp-filter
		 */
		$points = apply_filters( 'team_points_'.$this->sport, $points, $team->id, $rule, $matches );
		
		return $points;
	}
	
	
	/**
	 * =======================
	 * Tiebreaker rules - Not used yet
	 * =======================
	 */
	
	/**
	 * break ties
	 *
	 * @param array $teams
	 * @return array
	 */
	private function tiebreak( $teams ) {
		$i = 0;
		$n = count($teams);
		while ( $i < $n-1 ) {
			// current team is tied with next team
			if ( $teams[$i]->rank == $teams[$i+1]->rank && $this->isTie($teams[$i], $teams[$i+1]) ) {
				// head to head matches
				if ( $this->tiebreak['headToHeadTwoTeams'] == 1 ) {
					$teams = $this->headToHeadTwoTeams( $teams, $i, $i+1 );
				}
			}
			
			$i++;
		}
		
		// re-order teams by rank
		foreach ( $teams AS $key => $row ) {
			$rank[$key] = $row->rank;
		}
		array_multisort($rank, SORT_ASC, $teams);
		
		return $teams;
	}
	
	
	/**
	 * determine if two teams are tied based on Head to Head Matches. Original code by LaMonte Forthun
	 *
	 * @param array $teams
	 * @param int $i index of first tied team
	 * @param int $j index of second tied team
	 * @return array
	 */
	private function headToHeadTwoTeams( $teams, $i, $j ) {
		$this->is_tie = false;
		$this->is_tiebreak = false;
		
		$team1Wins = $team2Wins = 0;
		
		$team1 = $teams[$i];
		$team2 = $teams[$j];
		
		// only use selected matches for standings selection
		if ( $this->match_query_args['standingstable'] ) {
			/*if ( $mode == "home" )
				$match_args['home_team'] = $team->id;
			// get only away matches
			if ( $mode == "away" )
				$match_args['away_team'] = $team->id;
			// get all matches for given team
			if ( $mode == "all" )
				$match_args['team_id'] = $team->id;
			*/
			
			$matches = $this->getMatches(array('team_id' => $team1->id));
		} else {
			$matches = $this->getMatches( array("team_id" => $team1->id, "match_day" => -1, "limit" => false, "cache" => false, "reset_query_args" => true) );
		}
		
		foreach ( $matches AS $match ) {
			if ( $match->home_team == $team2->id ) {
				if ( $match->home_points > $match->away_points ) {
					$team2Wins++;
				} else {
					$team1Wins++;
				}
			} elseif ( $match->away_team == $team2->id ) {
				if ( $match->home_points > $match->away_points ) {
					$team1Wins++;
				} else {
					$team2Wins++;
				}
			}
		}

		// increase rank with lower win counts
		if( $team1Wins < $team2Wins ) {
			$team1->rank += 1;
			$team1->status = $this->getTeamStatus($team1, $team1->rank);
			$this->is_tiebreak = true;
		} elseif( $team1Wins > $team2Wins ) {
			$team2->rank += 1;
			$team2->status = $this->getTeamStatus($team2, $team2->rank);
			$this->is_tiebreak = true;
		} else {
			$this->is_tie = true;
		}
		
		$teams[$i] = $team1;
		$teams[$j] = $team2;
		
		return $teams;
	}
	
	
	/**
	* ========================
	* Sports customization section. The following functions can be overriden by sports class
	* ========================
	*/
	
	/**
	 * determine if two teams are tied based on
	 *
	 * 1) Primary points
	 * 2) Secondary point difference
	 * 3) Secondary points
	 * 4) Win Percentage
	 *
	 * @param Team $team1
	 * @param Team $team2
	 * @return boolean
	 */
	protected function isTie( $team1, $team2 ) {
		// initialize results array
		$res = array('primary' => false, 'diff' => false, 'secondary' => false, 'winpercent' => false);
		
		if ( $team1->points['plus'] == $team2->points['plus'] )
			$res['primary'] = true;
		
		if ( $team1->diff == $team2->diff )
			$res['diff'] = true;
		
		if ( $team1->points2['plus'] == $team2->points2['plus'] )
			$res['secondary'] = true;
		
		if ( $team1->winPercent == $team2->winPercent )
			$res['winpercent'] = true;
		
		// get unique results
		$res = array_values(array_unique($res));
		
		// more than one results, i.e. not tied
		if ( count($res) > 1 )
			return false;

		return $res[0];
	}
	
	
	/**
	 * default ranking function. Re-defined in sports-specific class
	 * 1) Primary points DESC
	 * 2) Win Percentage DESC
	 * 3) Done Matches ASC
	 *
	 * @param array $teams
	 * @return array
	 */
	protected function rankTeams($teams) {
		$points = $done = $WinPerc = array();
		foreach ( $teams AS $key => $row ) {
			$points[$key] = $row->points['plus'];
			$done[$key] = $row->done_matches;
			$WinPerc[$key] = $row->winPercent;
		}
		array_multisort($points, SORT_DESC, $WinPerc, SORT_DESC, $done, SORT_ASC, $teams);
		return $teams;
	}
	
	
	/**
	 * custom update results method
	 *
	 * @param Match $match
	 * @return Match
	 */
	protected function updateResults($match) {
		return $match;
	}
	
	
	/**
	 * calculate secondary points
	 *
	 * @param Team $team
	 * @param array $matches (optional)
	 * @param array An associative array with fields 'plus' and 'minus'
	 */
	protected function calculateSecondaryPoints($team, $matches = false) {
		$points = array('plus' => 0, 'minus' => 0);
		
		// general secondary points calculated from sum of primary points, e.g. soccer, handball, basketball
		if ( isset($this->fields_team['points2']) ) {
			if (!$matches) $matches =  $this->getMatches( array("team_id" => $team->id, "match_day" => -1, "limit" => false, "cache" => false, "reset_query_args" => true) );
			if ( $matches ) {
				foreach ( $matches AS $match ) {
					$custom = maybe_unserialize($match->custom);
					if ( isset($custom['overtime']) && $custom['overtime']['home'] != '' && $custom['overtime']['away'] != '' ) {
						$home_goals = $custom['overtime']['home'];
						$away_goals = $custom['overtime']['away'];
					} else {
						$home_goals = $match->home_points;
						$away_goals = $match->away_points;
					}

					if ( $match->home_team == $team->id ) {
						$points['plus'] += $home_goals;
						$points['minus'] += $away_goals;
					} else {
						$points['plus'] += $away_goals;
						$points['minus'] += $home_goals;
					}
				}
			}
		}

		return $points;
	}
	
	
	/**
	 * get custom standings data
	 *
	 * @param Team $team
	 * @param array $data
	 * @param array|false $matches
	 * @return array
	 */
	protected function getStandingsData($team, $data, $matches = false) {
		return $data;
	}
	
	
	/**
	 * add custom standings table display options
	 *
	 * @param array $options
	 * @return array
	 */
	public function standingsTableDisplayOptions( $options ) {
		if ( count($this->fields_team) > 0 ) {
			foreach ( $this->fields_team AS $key => $data ) {
				$options[$key] = isset($data['desc']) ? $data['desc'] : $data['label'];
			}
		}
		
		return $options;
	}
	
	
	/**
	 * display hidden fields in team edit form
	 *
	 * @param Team $team
	 */
	public function editTeamForm( $team ) {
		if ( count($this->fields_team) > 0 ) {
			foreach ( $this->fields_team AS $key => $data ) {
				if ( !isset($team->{$key}) ) {
					if ( isset($data['keys']) ) {
						$team->{$key} = array();
						foreach ($data['keys'] AS $k)
							$team->{$key}[$k] = '';
					} else {
						$team->{$key} = '';
					}
				}
				
				if ( $key == "diff" ) {
					$team->{$key} = intval($team->{$key});
				}
				
				if ( is_array($team->{$key}) ) {
					foreach ( $team->{$key} AS $k => $v ) {
						echo '<input type="hidden" name="custom['.$key.']['.$k.']" value="'.$v.'" />';
					}
				} else {
					echo '<input type="hidden" name="custom['.$key.']" value="'.$team->{$key}.'" />';
				}
			}
		}			
	}
	
	
	/**
	 * display custom standings header
	 *
	 */
	public function displayStandingsHeader() {
		if ( count($this->fields_team) > 0 ) {
			foreach ( $this->fields_team AS $key => $data ) {
				if ( show_standings($key) || (is_admin() && get_league_pointrule() == 'manual') ) {
					echo '<th class="manage-column column-'.$key.' column-num">'.$data['label'].'</th>';
				}
			}
		}
	}
	
	/**
	 * display custom standings columns
	 *
	 * @param Team $team
	 * @param string $rule
	 */
	public function displayStandingsColumns($team, $rule) {
		if ( count($this->fields_team) > 0 ) {
			foreach ( $this->fields_team AS $key => $data ) {
				if ( !isset($team->{$key}) ) {
					if ( isset($data['keys']) ) {
						$team->{$key} = array();
						foreach ($data['keys'] AS $k)
							$team->{$key}[$k] = '';
					} else {
						$team->{$key} = '';
					}
				}
				
				if ( (isset($data['type']) && $data['type'] == 'input') && is_admin() && $rule == 'manual' ) {
					echo '<td class="column-'.$key.' column-num" data-colname="'.$data['label'].'">';
					if ( is_array($team->{$key}) ) {
						foreach ( $team->{$key} AS $k => $v ) {
							echo '<input class="points" type="text" size="2" id="home_'.$team->id.'_'.$k.'" name="custom['.$team->id.']['.$key.']['.$k.']" value="'.$team->{$key}[$k].'" />';
						}
					} else {
						echo '<input class="points" type="text" size="2" id="home_'.$team->id.'" name="custom['.$team->id.']['.$key.']" value="'.$team->{$key}.'" />';
					}
					echo '</td>';
				} elseif ( show_standings($key) ) {
					if (is_array($team->{$key})) {
						//$team->{$key} = array_values($team->{$key});
						$team->{$key} = vsprintf($this->point_format2, $team->{$key});
					}
					echo '<td class="num column-'.$key.'" data-colname="'.$data['label'].'">'.$team->{$key}.'</td>';
				}
			}
		}
	}
	
	
	/**
	 * display custom standings header
	 *
	 */
	public function displayMatchesHeader() {
		if ( count($this->fields_match) > 0 ) {
			foreach ( $this->fields_match AS $key => $data ) {
				echo '<th class="manage-column column-'.$key.' column-num">'.$data['label'].'</th>';
			}
		}
	}
	
	/**
	 * display custom standings columns
	 *
	 * @param Match $match
	 */
	public function displayMatchesColumns($match) {
		if ( count($this->fields_match) > 0 ) {
			foreach ( $this->fields_match AS $key => $data ) {
				if (!isset($match->{$key})) {
					$x = array();
					$tmp_key = array_keys($data['keys']);
					if ( isset($data['keys']) && is_array($data['keys'][$tmp_key[0]]) ) {
						foreach ( $data['keys'] AS $k => $v ) {
							$x[$k] = array();
							$x[$k][$v[0]] = '';
							$x[$k][$v[1]] = '';
						}
					} else {
						if ( isset($data['keys']) ) {
							$x[$data['keys'][0]] = '';
							$x[$data['keys'][1]] = '';
						} else {
							$x = '';
						}
					}
					$match->{$key} = $x;
				}
	
				echo '<td class="column-'.$key.' column-input column-num" data-colname="'.$data['label'].'">';
				
				if ( isset($data['url']) ) {
					echo '<a href="'.$data['url'].'&league_id='.$match->league_id.'&season='.$match->season.'&match='.$match->id.'">'.$data['label'].'</a>';
				} elseif ( isset($data['keys']) && is_array($data['keys'][array_keys($data['keys'])[0]]) ) {
					// two-dimensional match keys, e.g. Basketball quarters, volleyball/tennis setSeason
					foreach ( $data['keys'] AS $k => $v ) {
						echo '<p>';
						foreach ( $v AS $f ) {
							echo '<input class="points" type="text" size="2" id="'.$key.'_'.$k.'_'.$f.'_'.$match->id.'" name="custom['.$match->id.']['.$key.']['.$k.']['.$f.']" value="'.$match->{$key}[$k][$f].'" />';
						}
						echo '</p>';
					}
				} else {
					if ( isset($data['keys']) ) {
						foreach ( $data['keys'] AS $f ) {
							echo '<input class="points" type="text" size="2" id="'.$key.'_'.$f.'_'.$match->id.'" name="custom['.$match->id.']['.$key.']['.$f.']" value="'.$match->{$key}[$f].'" />';
						}
					} else {
						echo '<input class="points" type="text" size="2" id="'.$key.'_'.$match->id.'" name="custom['.$match->id.']['.$key.']" value="'.$match->{$key}.'" />';
					}
				}
				echo '</td>';
			}
		}
	}
	
	
	/**
	 * export matches header
	 *
	 * @param string $content
	 * @return string
	 */
	public function exportMatchesHeader( $content ) {
		if ( count($this->fields_match) > 0 ) {
			foreach ( $this->fields_match AS $key => $data ) {
				$content .= "\t".utf8_decode($data['label']);
				
				if ( isset($data['keys']) && is_array($data['keys'][array_keys($data['keys'])[0]]) ) {
					$content .= str_repeat("\t".utf8_decode($data['label']), count($data['keys'])-1);
				}
			}
		}
		return $content;
	}
	
	/**
	 * export matches data
	 *
	 * @param string $content
	 * @param Match $match
	 * @return string
	 */
	public function exportMatchesData( $content, $match ) {
		if ( count($this->fields_match) > 0 ) {
			foreach ( $this->fields_match AS $key => $data ) {
				if (!isset($match->{$key})) {
					if ( isset($data['keys']) && is_array($data['keys'][array_keys($data['keys'])[0]]) ) {
						$x = array();
						foreach ( $data['keys'] AS $k => $v ) {
							$x[$k] = array();
							$x[$k][$v[0]] = '';
							$x[$k][$v[1]] = '';
						}
					} else {
						if ( isset($data['keys']) ) {
							$x = array();
							$x[$data['keys'][0]] = '';
							$x[$data['keys'][1]] = '';
						} else {
							$x = '';
						}
					}
					$match->{$key} = $x;
				}
				
				if ( isset($data['keys']) && is_array($data['keys'][array_keys($data['keys'])[0]]) ) {
						foreach ( $data['keys'] AS $k => $v ) {
							$content .= "\t".vsprintf("%d-%d", $match->{$key}[$k]);
						}
				} else {
					if ( isset($data['keys']) ) {
						$content .= "\t".vsprintf("%d-%d", $match->{$key});
					} else {
						$content .= "\t".$match->{$key};
					}
				}
			}
		}
		
		return $content;
	}
	
	
	/**
	 * import matches
	 *
	 * @param array $custom
	 * @param array $line
	 * @param int $match_id
	 * @param int $col the starting column index
	 * @return array
	 */
	public function importMatches( $custom, $line, $match_id, $col ) {
		if ( count($this->fields_match) > 0 ) {
			foreach ( $this->fields_match AS $key => $data ) {				
				if ( isset($data['keys']) && is_array($data['keys'][array_keys($data['keys'])[0]]) ) {
					foreach ( $data['keys'] AS $k => $v ) {
						$p = (isset($line[$col]) && !empty($line[$col])) ? explode("-", $line[$col]) : array('','');
						
						$x = array();
						$x[$v[0]] = $p[0];
						$x[$v[1]] = $p[1];
						
						$custom[$match_id][$key][$k] = $x;
				
						$col++;
					}
				} else {
					if ( isset($data['keys']) ) {
						$p = (isset($line[$col]) && !empty($line[$col])) ? explode("-", $line[$col]) : array('','');					
						
						$x = array();
						$x[$data['keys'][0]] = $p[0];
						$x[$data['keys'][1]] = $p[1];
					} else {
						$x = (isset($line[$col]) && !empty($line[$col])) ? $line[$col] : '';
					}
					
					$custom[$match_id][$key] = $x;
				
					$col++;
				}
				
			}
		}

		return $custom;
	}
	
	
	/**
	 * export teams header
	 *
	 * @param string $content
	 * @return string
	 */
	public function exportTeamsHeader( $content ) {
		if ( count($this->fields_team) > 0 ) {
			foreach ( $this->fields_team AS $key => $data )
				$content .= "\t".utf8_decode($data['label']);
		}
		
		return $content;
	}
	
	/**
	 * export teams data
	 *
	 * @param string $content
	 * @param object $team
	 * @return the content
	 */
	public function exportTeamsData( $content, $team ) {
		if ( count($this->fields_team) > 0 ) {
			foreach ( $this->fields_team AS $key => $data ) {
				if ( !isset($team->{$key}) ) {
					if ( isset($data['keys']) ) {
						$team->{$key} = array();
						foreach ($data['keys'] AS $k)
							$team->{$key}[$k] = '';
					} else {
						$team->{$key} = '';
					}
				}
				
				if (is_array($team->{$key})) {
					//$team->{$key} = array_values($team->{$key});
					$team->{$key} = vsprintf("%d-%d", $team->{$key});
				}
					
				$content .= "\t".$team->{$key};
			}
		}

		return $content;
	}
	
	
	/**
	 * import teams
	 *
	 * @param array $custom
	 * @param array $line
	 * @param int $col the starting column index
	 * @return array
	 */
	public function importTeams( $custom, $line, $col ) {
		if ( count($this->fields_team) > 0 ) {
			foreach ( $this->fields_team AS $key => $data ) {
				if ( isset($data['keys']) ) {
					$p = (isset($line[$col]) && !empty($line[$col])) ? explode("-", $line[$col]) : array('','');
					$x = array();
					$x[$data['keys'][0]] = $p[0];
					$x[$data['keys'][1]] = $p[1];
					$custom[$key] = $x;
				} else {
					$custom[$key] = isset($line[$col]) ? $line[$col] : 0;
				}
				
				$col++;
			}
		}
		
		return $custom;
	}
	
	
	/**
	 * get image path
	 *
	 * @param string|false $file
	 * @return string
	 */
	public function getFilePath( $file = false ) {
		$base = WP_CONTENT_DIR.'/uploads/leagues/League-'.$this->id;

		if ( $file ) {
			return $base .'/'. basename($file);
		} else {
			return $base;
		}
	}
}


/**
 * get League object
 *
 * @param int|League|null League ID or league object. Defaults to global $league
 * @return League|null
 */
function get_league( $league = null ) {
	if ( empty( $league ) && isset( $GLOBALS['league'] ) )
		$league = $GLOBALS['league'];
	
	if ( $league instanceof League ) {
		$_league = $league;
	} elseif ( is_object( $league ) ) {
		// check if specific sports class exists
		if ( !isset($league->sport) ) $league->sport = '';
		$instance = "League_". ucfirst($league->sport);
		if (class_exists($instance)) {
			$_league = new $instance( $league );
		} else {
			$_league = new League( $league );
		}
	} else {
		$_league = League::get_instance( $league );
	}

	if ( ! $_league )
		return null;

	return $_league;
}
?>