<?php
/**
 * Match API: Match class
 *
 * @package LeagueManager
 * @subpackage Match
 */
 
/**
 * Class to implement the Match object
 *
 */
final class Match {
	/**
	 * match ID
	 *
	 * @var int
	 */
	public $id = 0;
	
	/**
	 * match title
	 *
	 * @var string
	 */
	public $match_title = '';
	
	/**
	 * league ID
	 *
	 * @var int
	 */
	public $league_id = 0;
	
	/**
	 * season
	 *
	 * @var string
	 */
	public $season = '';
	
	/**
	 * group
	 *
	 * @var string
	 */
	public $group = '';
	
	/**
	 * final flag
	 *
	 * @var string
	 */
	public $final_round = '';
	
	/**
	 * home team ID
	 *
	 * @var int
	 */
	public $home_team = 0;
	
	/**
	 * away team ID
	 *
	 * @var int
	 */
	public $away_team = 0;
	
	/**
	 * match day
	 *
	 * @var int
	 */
	public $match_day = 1;
	
	/**
	 * match location
	 *
	 * @var string
	 */
	public $location = '';
	
	/**
	 * match status
	 *
	 * @var string
	 */
	public $status = 'scheduled';
	
	/**
	 * home points
	 *
	 * @var string
	 */
	public $home_points = '';
	
	/**
	 * away points
	 *
	 * @var string
	 */
	public $away_points = '';
	
	/**
	 * winning team ID
	 *
	 * @var int
	 */
	public $winner_id = 0;
	
	/**
	 * losing team ID
	 *
	 * @var int
	 */
	public $loser_id = 0;
	
	/** 
	 * Project category for match statistics
	 *
	 * @var int
	 */
	public $stat_id = 0;
	
	/**
	 * match report post ID
	 *
	 * @var int
	 */
	public $post_id = 0;
	
	/**
	 * match date
	 *
	 * @var string
	 */
	public $date = '';
	
	/**
	 * match date
	 *
	 * @var string
	 */
	public $match_date = '';
	
	/**
	 * match time
	 *
	 * @var string
	 */
	public $start_time = '';
	
	/**
	 * day
	 *
	 * @var int
	 */
	public $day = 0;
	
	/**
	 * month
	 *
	 * @var int
	 */
	public $month = 0;
	
	/**
	 * year
	 *
	 * @var int
	 */
	public $year = 0;
	
	/**
	 * hour
	 *
	 * @var int
	 */
	public $hour = 0;
	
	/**
	 * minutes
	 *
	 * @var int
	 */
	public $minutes = 0;
	
	/**
	 * custom data
	 *
	 * @var array
	 */
	public $custom = array();
	
	/**
	 * home match?
	 *
	 * @var boolean
	 */
	public $is_home = false;
	
	/**
	 * team objects of current match
	 *
	 * @var array
	 */
	public $teams = array('home' => null, 'away' => null);
	
	/**
	 * had overtime?
	 *
	 * @var boolean
	 */
	public $hadOvertime = false;
	
	/**
	 * had penatly shootout?
	 *
	 * @var boolean
	 */
	public $hadPenalty = false;
	
	/**
	 * home score
	 *
	 * @var int
	 */
	public $homeScore = 0;
	
	/**
	 * away score
	 *
	 * @var int
	 */
	public $awayScore = 0;
	
	/**
	 * formatted score
	 *
	 * @var string
	 */
	public $score = '';
	
	/**
	 * link to match report
	 *
	 * @var string
	 */
	public $report = '';
	
	/**
	 * page URL
	 *
	 * @var string
	 */
	public $pageURL = '';
	
	/**
	 * tooltip title
	 *
	 * @var string
	 */
	public $tooltipTitle = '';
	
	/**
	 * check if match is selected
	 *
	 * @var boolean
	 */
	public $is_selected = false;
	
	/**
	 * League Object
	 *
	 * @var League
	 */
	private $league = null;
	
	
	/**
	 * retrieve match instance
	 *
	 * @param int $match_id
	 */
	public static function get_instance($match_id) {
		global $wpdb;

		$match_id = (int) $match_id;
		if ( ! $match_id )
			return false;

		$match = wp_cache_get( $match_id, 'matches' );
		if ( ! $match ) {
			$match = $wpdb->get_row( $wpdb->prepare("SELECT `final` AS final_round, `group`, `home_team`, `away_team`, DATE_FORMAT(`date`, '%%Y-%%m-%%d %%H:%%i') AS date, DATE_FORMAT(`date`, '%%e') AS day, DATE_FORMAT(`date`, '%%c') AS month, DATE_FORMAT(`date`, '%%Y') AS year, DATE_FORMAT(`date`, '%%H') AS `hour`, DATE_FORMAT(`date`, '%%i') AS `minutes`, `match_day`, `location`, `status`, `league_id`, `home_points`, `away_points`, `winner_id`, `loser_id`, `stat_id`, `post_id`, `season`, `id`, `custom` FROM {$wpdb->leaguemanager_matches} WHERE `id` = '%d' LIMIT 1", $match_id) );
			
			if ( !$match ) return false;
			
			$match = new Match( $match );
			
			wp_cache_add( $match->id, $match, 'matches' );
		}
		
		return $match;
	}
	
	
	/**
	 * Constructor
	 *
	 * @param object $match Match object.
	 */
	public function __construct( $match = null ) {
		if ( !is_null($match) ) {
			if ( isset($match->custom) ) {
				$match->custom = stripslashes_deep((array)maybe_unserialize($match->custom));
				$match = (object)array_merge((array)$match, (array)$match->custom);
			}
			
			foreach ( get_object_vars( $match ) as $key => $value )
				$this->$key = $value;
			
			// get League Object
			$this->league = get_league();
			if ( is_null($this->league) || (!is_null($this->league) && $this->league->id != $this->league_id) )
				$this->league = get_league($this->league_id);
			
			$this->location = $this->location != '' ? stripslashes($this->location) : 'N/A';
			$this->report = ( $this->post_id != 0 ) ? '<a href="'.get_permalink($this->post_id).'">'.__('Report', 'leaguemanager').'</a>' : '';
			
			$this->hadOvertime = ( isset($this->overtime) && $this->overtime['home'] != '' && $this->overtime['away'] != '' ) ? true : false;
			$this->hadPenalty = ( isset($this->penalty) && $this->penalty['home'] != '' && $this->penalty['away'] != '' ) ? true : false;
			if ( $this->hadPenalty ) {
				$this->homeScore = $this->penalty['home']+$this->overtime['home'];
				$this->awayScore = $this->penalty['away']+$this->overtime['away'];
				$this->score = sprintf("%d - %d", $this->homeScore, $this->awayScore)." ".__( '(o.P.)', 'leaguemanager' );
			} elseif ( $this->hadOvertime ) {
				$this->homeScore = $this->overtime['home'];
				$this->awayScore = $this->overtime['away'];
				$this->score = sprintf("%d - %d", $this->homeScore, $this->awayScore)." ".__( '(AET)', 'leaguemanager' );
			} elseif ( $this->home_points != "" && $this->away_points != "" ) {
				$this->homeScore = $this->home_points;
				$this->awayScore = $this->away_points;
				$this->score = sprintf("%d - %d", $this->homeScore, $this->awayScore);
			} else {
				$this->homeScore = "-";
				$this->awayScore = "-";
				$this->score = sprintf("%s:%s", $this->homeScore, $this->awayScore);
			}
			
			if (is_admin()) {
				$url = '';
			} else {
				$url = esc_url(get_permalink());
				$url = add_query_arg( 'match_'.$this->league_id, $this->id, $url );
				foreach ( $_GET AS $key => $value ) {
					$url = add_query_arg( $key, htmlspecialchars(strip_tags($value)), $url );
				}
				$url = remove_query_arg( 'team_'.$this->league_id, $url );
			}
			$this->pageURL = esc_url($url);

			$this->setTeams();
			
			$this->setDate();
			$this->setTime();

			$this->is_home = $this->isHome();
			$this->match_title = $this->getTitle();
			
			// set selected marker
			if (isset($_GET['match_'.$this->league_id])) $this->is_selected = true;
		}
	}
	
	
	/**
	 * get Team objects
	 *
	 */
	private function setTeams() {
		if ( is_numeric($this->home_team) && is_numeric($this->away_team) ) {
			$this->teams = array('home' => $this->league->getTeam($this->home_team), 'away' => $this->league->getTeam($this->away_team));
		} else {
			// get championship final rounds teams
			if ( $this->league->championship instanceof League_Championship ) {
				$teams = $this->league->championship->getFinalTeams($this->final_round);
				$this->teams = array( 'home' => $teams[$this->home_team], 'away' => $teams[$this->away_team] );
			}
		}
	}
	
	
	/**
	 * get match title
	 *
	 * @param boolean show_logo
	 * @return string
	 */
	public function getTitle( $show_logo = true ) {
		// set default title
		$title = "N/A";
		
		// Specific teams are set
		if ( is_numeric($this->home_team) && is_numeric($this->away_team) ) {
			$homeTeam = $this->teams['home'];
			$awayTeam = $this->teams['away'];
			
			if ( isset($this->title) && (!$homeTeam || !$awayTeam || $this->home_team == $this->away_team) ) {
				$title = stripslashes($this->title);
			} elseif ( isset($this->home_partner) && !empty($this->home_partner) && isset($this->guest_partner) && !empty($this->guest_partner) ) {
				// tennis double match
				$homePartner = get_team($this->home_partner);
				$awayPartner = get_team($this->guest_partner);
				
				$title = sprintf("%s/%s &mdash; %s/%s", $homeTeam->title, $homePartner->title, $awayTeam->title, $awayPartner->title);
			} else {
				$home_logo_img = ($homeTeam->logo != '' && $show_logo === true) ? "<img class='match-title home-logo logo' src='".$homeTeam->getImageURL($homeTeam->logo, 'thumb')."' alt='' />" : "";
				$away_logo_img = ($awayTeam->logo != '' && $show_logo === true) ? "<img class='match-title away-logo logo' src='".$homeTeam->getImageURL($awayTeam->logo, 'thumb')."' alt='' />" : "";
				
				$home_team_name = $this->is_home ? "<strong>".$homeTeam->title."</strong>" : $homeTeam->title; 
				$away_team_name = $this->is_home ? "<strong>".$awayTeam->title."</strong>" : $awayTeam->title; 
			
				$title = sprintf("<span class='team home'>%s</span> <span class='logos'><span class='logo home'>%s</span> &#8211; <span class='logo away'>%s</span></span> <span class='team away'>%s</span>", $home_team_name, $home_logo_img, $away_logo_img, $away_team_name);
			}
		} elseif ( isset($this->title) ) {
			$title = stripslashes($this->title);
		} else {
			// get general final rounds team name for championship mode
			if ( $this->league->championship instanceof League_Championship ) {
				$teams = $this->league->championship->getFinalTeams($this->final_round);
				$title = sprintf("<span class='team home'>%s</span> &#8211; <span class='team away'>%s</span>", $teams[$this->home_team]->title, $teams[$this->away_team]->title);
			}
		}
		
		return "<span class='match_title'>".$title."</span>";
	}
	
	
	/**
	 * test if it's a match of home team
	 *
	 * @return boolean
	 */
	private function isHome() {
		if ( $this->teams['home'] && $this->teams['home']->home == 1 )
			return true;
		elseif ( $this->teams['away'] && $this->teams['away']->home == 1 )
			return true;
		else
			return false;
	}
	
	/**
	 * set match date
	 *
	 * @param string $date_format
	 */
	public function setDate($date_format = '') {
		if ($date_format == '') $date_format = get_option('date_format');
		$this->match_date = ( substr($this->date, 0, 10) == '0000-00-00' ) ? 'N/A' : mysql2date($date_format, $this->date);
		$this->setTooltipTitle();
	}
	
	
	/**
	 * set match start time
	 *
	 * @param string $time_format
	 */
	public function setTime($time_format = '') {
		if ($time_format == '') $time_format = get_option('time_format');
		//$this->start_time = ( '00:00' == $this->hour.":".$this->minutes ) ? '' : mysql2date($time_format, $this->date);
		$this->start_time = mysql2date($time_format, $this->date);
	}
	
	
	/**
	 * set tooltip title
	 *
	 */
	private function setTooltipTitle() {
		// make tooltip title for last-5 standings
		if ( $this->home_points == "" && $this->away_points == "")
			$tooltipTitle = 'Next Match: '.$this->teams['home']->title.' - '.$this->teams['away']->title.' ['.$this->match_date.']';
		elseif ( isset($this->title) )
			$tooltipTitle = stripslashes($this->title) .' ['.$this->match_date.']'; 
		else
			$tooltipTitle = $this->homeScore.':'.$this->awayScore. ' - '.$this->teams['home']->title.' - '.$this->teams['away']->title.' ['.$this->match_date.']';
		
		$this->tooltipTitle = $tooltipTitle;
	}
	
	
	/** 
	 * update match results
	 *
	 * @param string $home_points
	 * @param string $away_points
	 * @param string $status
	 * @param array $custom
	 */
	public function updateResults($home_points, $away_points, $status, $custom) {
		// determine match score
		if ( isset($custom['penalty']) && $custom['penalty']['home'] != "" && $custom['penalty']['home'] != "" )
			$score = array( 'home' => $custom['penalty']['home'], 'away' => $custom['penalty']['away'] );
		elseif ( isset($custom['overtime']) && $custom['overtime']['home'] != "" && $custom['overtime']['away'] != "" )
			$score = array( 'home' => $custom['overtime']['home'], 'away' => $custom['overtime']['away'] );
		else
			$score = array( 'home' => $home_points, 'away' => $away_points );
		
		// update status. Automatically set to 'done' if scores are present and no specific status is set
		if ( $status == 'scheduled' && $home_points != "" && $away_points != "" )
			$status = 'done';
		
		$this->status = $status;
		
		// determine match result
		$this->getResult( $score['home'], $score['away'], 'winner' );
		$this->getResult( $score['home'], $score['away'], 'loser' );
		
		// save original score points
		$this->home_points = $home_points;
		$this->away_points = $away_points;
		
		// push new custom data into object
		$this->custom = array_merge( (array)$this->custom, (array)$custom );
		foreach ( $this->custom AS $key => $value )
			$this->{$key} = $value;
	}
	
	
	/**
	 * determine match result for updating results
	 *
	 * @param int $home_points
	 * @param int $away_points
	 * @param string $index
	 * @return int
	 */
	public function getResult( $home_points, $away_points, $index ) {
		$match = array();
		if ( $home_points > $away_points ) {
			$match['winner'] = $this->home_team;
			$match['loser'] = $this->away_team;
		} elseif ( $home_points < $away_points ) {
			$match['winner'] = $this->away_team;
			$match['loser'] = $this->home_team;
		} elseif ( 'NULL' === $home_points && 'NULL' === $away_points ) {
			$match['winner'] = 0;
			$match['loser'] = 0;
		} elseif ( '' == $home_points && '' == $away_points ) {
			$match['winner'] = 0;
			$match['loser'] = 0;
		} else {
			$match['winner'] = -1;
			$match['loser'] = -1;
		}

		$val = intval($match[$index]);
		
		if ( $index == "winner" )
			$this->winner_id = $val;
		if ( $index == "loser" )
			$this->loser_id = $val;
		
		return $val;
	}
	
	
	/**
	 * get possible status options
	 *
	 * @param string $key optional status key to retrieve formatted value
	 * @return array
	 */
	public function getStatusOptions( $key = '' ) {
		$status = array(
			'scheduled' => __('Scheduled', 'leaguemanager'),
			'forfeited' => __('Forfeited', 'leaguemanager'),
			'ppd' => __('Postponed', 'leaguemanager'),
			'done' => __('Done', 'leaguemanager'),
			'cancelled' => __('Cancelled', 'leaguemanager')
		);
		
		if ( $key == "" )
			return $status;
		else
			return $status[$key];
	}
}

/**
 * get Match object
 *
 * @param int|Match|null Match ID or match object. Defaults to global $match
 * @return Match|null
 */
function get_match( $match = null ) {
	if ( empty( $match ) && isset( $GLOBALS['match'] ) )
		$match = $GLOBALS['match'];

	if ( $match instanceof Match ) {
		$_match = $match;
	} elseif ( is_object( $match ) ) {
		$_match = new Match( $match );
	} else {
		$_match = Match::get_instance( $match );
	}

	if ( ! $_match )
		return null;

	return $_match;
}
?>