<?php
/**
 * ProjectManager driven extensions
 *
 */
 
add_action( 'league_settings_general', 'leaguemanager_projectmanager_settings' );

/**
 * add settings
 *
 * @param League $league
 * @return void
 */
function leaguemanager_projectmanager_settings( $league ) {
	global $leaguemanager, $projectmanager;
	
	$fields = array( 'teamroster' => 'Team Roster', 'teamprofiles' => 'Team Profiles', 'matchstats' => 'Match Statistics' );
	
	if ( $leaguemanager->hasProjectManager() ) {
		echo '<h3>'.__( 'ProjectManager Extensions', 'leaguemanager' ).'</h3>';
		
		echo '<table class="form-table">';
		foreach ( $fields AS $key => $label ) {
			echo '<tr>';
			echo '<th scope="row"><label for="'.$key.'">'.__( $label, 'leaguemanager' ).'</label></th>';
			echo '<td>';
			echo '<select size="1" name="settings['.$key.']" id="'.$key.'">';
			echo '<option value="">'.__('None','leaguemanager').'</option>';
			$projectmanager->displayProjectIndex($args = array('child_of' => -1, 'class' => '', 'level' => 0, 'selected' => $league->{$key}), $output = 'select');
			echo '</select>';
			echo ' <p class="tagline-description description">'.sprintf( __( "Check the <a href='%s'>Documentation</a> for details", 'leaguemanager'), admin_url() . 'admin.php?page=leaguemanager-doc' ).'</p>';
			echo '</td>';
			echo '</tr>';
		}
		echo "</table>";
	}
}


/**
 * ProjectManager driven extensions
 *
 * @package	LeagueManager
*/
class LeagueManagerProjectManager extends LeagueManager {
	
	/**
	 * constructor
	 *
	 */
	function __construct() {
		// add LeagueManager template directory to projectmanager search path
		add_filter( 'projectmanager_template_paths', array(&$this, 'addTemplatePath') );
		add_filter( 'projectmanager_formfields', array(&$this, 'addFormfields') );
		
		add_filter( 'dataset_meta_leaguemanager_roster', array(&$this, "TeamRosterOutput"));
		
		
	}
	
	
	/**
	 * add ProjectManager Formfields
	 *
	 * 1. Team Roster: creates a dataset dropdown menu in dataset form
	 *
	 * @param array $formfields
	 * @return array
	 */
	public function addFormfields($formfields) {
		$formfields['leaguemanager_roster'] = array( 'name' => "Team Roster", 'input_callback' => array(&$this, 'TeamRosterInput'), 'input_args' => array(), 'options_type' => 'project');

		return $formfields;
	}
	
	
	/**
	 * Projectmanager FormFields callback for Team Roster input
	 *
	 * @param int $dataset_id Dataset ID
	 * @param object $form_field Form field object
	 * @param string $data
	 * @param string $field_name
	 * @param string $field_options
	 */
	public function TeamRosterInput($dataset_id, $form_field, $data, $field_name, $field_options ) {
		$project = get_project(intval($field_options));
		$project->printDatasetDropdown($field_name, $data);
	}
	
	
	/**
	 * ProjectManager Team Roster display
	 *
	 * @param string $meta_value
	 * @return string
	 */
	public function TeamRosterOutput( $meta_value ) {
		//$meta_value = isset($data['meta_value']) ? intval($data['meta_value']) : 0;
		$dataset = get_dataset($meta_value);
		
		if ( $dataset )
			return $dataset->name;
		
		return '';
	}
	
	
	/**
	 * add Leaguemanager template path to ProjectManager search path
	 *
	 * @param array $paths
	 * @return array
	 */
	public function addTemplatePath( $paths ) {
		$paths[] = LEAGUEMANAGER_PATH . "/templates";
		return $paths;
	}
	
	
	/**
	 * display Table Header for Match Statistics Administration
	 *
	 * @param none
	 * @return void
	 */
	public function displayMatchesHeader() {
		echo '<th>'.__('Stats', 'leaguemanager').'</th>';
	}


	/**
	 * display Table columns for Match Statistics Administration
	 *
	 * @param Match $match
	 * @return void
	 */
	public function displayMatchesColumns( $match ) {
		echo '<td></td>';
	}
}
?>