<?php
/**
 * LeagueManagerShortcodes API: LeagueManagerShortcodes class
 *
 * @package LeagueManager
 * @subpackage LeagueManagerShortcodes
 */
 
/**
 * Class to implement shortcode functions
 *
 */
class LeagueManagerShortcodes extends LeagueManager {
	/**
	 * initialize shortcodes
	 *
	 */
	public function __construct() {
		add_shortcode( 'standings', array(&$this, 'showStandings') );
		add_shortcode( 'matches', array(&$this, 'showMatches') );
		add_shortcode( 'match', array(&$this, 'showMatch') );
		add_shortcode( 'championship', array(&$this, 'showChampionship') );
		add_shortcode( 'crosstable', array(&$this, 'showCrosstable') );
		add_shortcode( 'teams', array(&$this, 'showTeams') );
		add_shortcode( 'team', array(&$this, 'showTeam') );
		add_shortcode( 'leaguearchive', array(&$this, 'showArchive') );
		add_shortcode( 'league', array(&$this, 'showLeague') );
	}


	/**
	 * Display League Standings
	 *
	 *	[standings league_id="1" template="name"]
	 *
	 * - league_id is the ID of league
	 * - season: display specific season (optional). default is current season
	 * - template is the template used for displaying. Replace name appropriately. Templates must be named "standings-template.php" (optional)
	 * - group: optional group
	 *
	 * @param array $atts shortcode attributes
	 * @return string
	 */
	public function showStandings( $atts ) {
		global $wpdb, $league;
		
		extract(shortcode_atts(array(
			'league_id' => 0,
			'template' => '',
			'season' => '',
			'group' => '',
			'widget_options' => '',
			'widget_id' => 0
		), $atts ));

		$league = $this->getLeague($league_id);
		
		$league->setTemplate('standingstable', $template);
		$league->setSeason($season);
		$league->setGroup($group);
		
		// set standings display options for widget
		if ( intval($widget_id) > 0 ) {
			$widget = true;
			$options = get_option($widget_options)[intval($widget_id)]['standings'];
			$league->standings = array();
			foreach ( $this->getStandingsDisplayOptions() AS $key => $label ) {
				if ( isset($options[$key]) )
					$league->standings[$key] = $options[$key];
			}
		} else {
			$widget = false;
		}
		
		$team_args = array("orderby" => array('rank' => 'ASC'));
		if ( $group ) $team_args["group"] = $group;
		$teams = $league->getTeams( $team_args );

		if (empty($template))
			$filename = 'standings';
		else
			$filename = ( !$widget && $this->checkTemplate('standings-'.$league->sport) ) ? 'standings-'.$league->sport : 'standings-'.$template;
		$out = $this->loadTemplate( $filename, array('league' => $league, 'teams' => $teams, 'widget' => $widget) );

		return $out;
	}


	/**
	 * Display League Matches
	 *
	 *	[matches league_id="1" mode="all|home|racing" template="name" roster=ID]
	 *
	 * - league_id is the ID of league
	 * - league_name: get league by name and not ID (optional)
	 * - mode can be either "all" or "home". For racing it must be "racing". If it is not specified the matches are displayed on a weekly basis
	 * - season: display specific season (optional)
	 * - template is the template used for displaying. Replace name appropriately. Templates must be named "matches-template.php" (optional)
	 * - template_type: this is only applicable to template='by_matchday', can be either empty or "accordion" or "tabs" to activitate jQuery accordion/tabs functionality
	 * - roster is the ID of individual team member (currently only works with racing)
	 * - match_day: specific match day (integer)
	 *
	 * @param array $atts shorcode attributes
	 * @return string
	 */
	public function showMatches( $atts ) {
		global $league;

		extract(shortcode_atts(array(
			'league_id' => 0,
			'team' => 0,
			'template' => '',
			'template_type' => '',
			'mode' => '',
			'season' => '',
			'limit' => true,
			'roster' => false,
			'match_day' => false,
			'home_only' => 'false',
			'match_date' => false,
			'group' => '',
			'time' => '',
			'dateformat' => '',
			'timeformat' => '',
			'show_team_selection' => 'true',
			'show_match_day_selection' => 'true'
		), $atts ));
		
		$league = $this->getLeague($league_id);
		
		$league->setTemplate('matches', $template);
		
		// Always disable match day in template to show matches by matchday or if $time is not empty
		if ( in_array($template, array("by_matchday")) || !empty($time) )
			$match_day = -1;
		
		$league->setMatchesSelection($show_match_day_selection, $match_day, $show_team_selection, $team);
		
		$league->setSeason($season);
		$league->setGroup($group);
		$league->setMatchDay($match_day);
		$league->matches_template_type = $template_type;
		
		//$match_date = $match_date . " 00:00:00";

		$matches = false;
		if ( !isset($_GET['match_'.$league->id]) ) {
			$match_args = array("final" => '');
 
			// get matches of specific team
			if ( !empty($team) || (isset($_GET['team_id_'.$league->id]) && !empty($_GET['team_id_'.$league->id])) )
				$team_id = !empty($team) ? $team : (int)$_GET['team_id_'.$league->id];

			if ( !empty($team_id) ) {
				$match_args['team_id'] = $team_id;
			} elseif ( !empty($group) ) {
				$match_args['group'] = $group;
			}
	
			if ( $limit === 'false' || in_array($template, array('by_matchday', 'by_matchday-tabs', 'by_matchday-accordion')) )  {
				$match_args['limit'] = false;
			} elseif ( $limit && is_numeric($limit) ) {
				$match_args['limit'] = intval($limit);
			}
			
			$match_args['time'] = $time;
			//$match_args['match_date'] = $match_date;
			$match_args['home_only'] = ( $home_only == 1 || $home_only == 'true' ) ? true : false;

			// get matches
			$matches = $league->getMatches( $match_args );
			//$league->setNumMatches();
		
			/*
			 * re-sort matches
			 */
            /*if ($time=='prev1'){ 
				 foreach ( $matches AS $key => $row ) {
					$matchdate[$key] = $row->date;
				}
				array_multisort( $matchdate, SORT_ASC, $matches );
			}*/

			foreach ( $matches AS $i => $match ) {
				$match->setTime($timeformat);
				$match->setDate($dateformat);
				
				$matches[$i] = $match;
			}
			$league->matches = $matches;
		}

		if ( empty($template) && $this->checkTemplate('matches-'.$league->sport) )
			$filename = 'matches-'.$league->sport;
		elseif ($this->checkTemplate('matches-'.$template.'-'.$league->sport) )
			$filename = 'matches-'.$template.'-'.$league->sport;
		else
			$filename = ( !empty($template) ) ? 'matches-'.$template : 'matches';

		$out = $this->loadTemplate( $filename, array('league' => $league, 'matches' => $matches) );

		return $out;
	}


	/**
	 * Display single match
	 *
	 * [match id="1" template="name"]
	 *
	 * - id is the ID of the match to display
	 * - template is the template used for displaying. Replace name appropriately. Templates must be named "match-template.php" (optional)
	 *
	 * @param array $atts shorcode attributes
	 * @return string
	 */
	public function showMatch( $atts ) {
		global $match;
		extract(shortcode_atts(array(
			'id' => 0,
			'template' => ''
		), $atts ));

		$league = get_league();
		$match = get_match(intval($id));
		
		// make sure that within a league only matches belonging to this league are displayed
		if (!is_null($league) && $match->league_id != $league->id)
			$match = false;
		
		if (!is_null($league) && empty($template) && $this->checkTemplate('match-'.$league->sport) )
			$filename = 'team-'.$league->sport;
		else
			$filename = ( !empty($template) ) ? 'match-'.$template : 'match';
		
		$out = $this->loadTemplate( $filename, array('match' => $match) );

		return $out;
	}


	/**
	 * Display Championship
	 *
	 *	[championship league_id="1" template="name"]
	 *
	 * - league_id is the ID of league
	 * - season: display specific season (optional)
	 * - template is the template used for displaying. Replace name appropriately. Templates must be named "matches-template.php" (optional)
	 *
	 * @param array $atts shorcode attributes
	 * @return string
	 */
	public function showChampionship( $atts ) {
		extract(shortcode_atts(array(
			'league_id' => 0,
			'template' => '',
			'season' => '',
		), $atts ));

		$league = $this->getLeague($league_id);
		
		$league->setSeason($season);
		$league->setTemplate('championship', $template);

		$finals = array();
		foreach ( $league->championship->getFinals() AS $key => $data ) {
			$matches_raw = $league->getMatches( array("final" => $data['key'], "orderby" => array("id" => "ASC")) );
		
			$matches = array();
			for ( $i = 1; $i <= $data['num_matches']; $i++ ) {
				$match = $matches_raw[$i-1];
				$data['field_id'] = '';
				$data['is_finished'] = false;
				
				if ( $match ) {
					if ($data['is_final']) {
						// tournament is over
						if ( $match->home_points != "" && $match->away_points != "" ) $data['is_finished'] = true;
						
						$data['field_id'] = ( $match->winner_id == $match->home_team ) ? "final_home" : "final_away";
					}
					
					$matches[$i] = $match;
				}
			}

			$data['matches'] = $matches;
			$finals[] = (object)$data;
		}

		$league->finals = $finals;
		
		if ( empty($template) && $this->checkTemplate('championship-'.$league->sport) )
			$filename = 'championship-'.$league->sport;
		else
			$filename = ( !empty($template) ) ? 'championship-'.$template : 'championship';

		$out = $this->loadTemplate( $filename, array('league' => $league) );

		return $out;
	}


	/**
	 * Display Team list
	 *
	 *	[teams league_id=ID template=X season=x]
	 *
	 * - league_id is the ID of league
	 * - season: use specific season (optional)
	 * - template is the template used for displaying. Replace name appropriately. Templates must be named "teams-template.php" (optional)
	 * - group: show teams only from specific group
	 *
	 * @param array $atts shorcode attributes
	 * @return string
	 */
	public function showTeams( $atts ) {
		global $league;
		extract(shortcode_atts(array(
			'league_id' => 0,
			'template' => '',
			'season' => '',
			'group' => ''
		), $atts ));

		$league = $this->getLeague($league_id);
		
		$league->setTemplate('teams', $template);
		$league->setSeason($season);
		$league->setGroup($group);

		$league->getTeams( array("limit" => true, "orderby" => array("title" => "ASC")) );

		$filename = ( empty($template) && $this->checkTemplate('teams-'.$league->sport) ) ? 'teams-'.$league->sport : ( !empty($template) ) ? 'teams-'.$template : 'teams-list';
		$out = $this->loadTemplate( $filename, array('league' => $league) );

		return $out;
	}


	/**
	 * Display Team Info Page
	 *
	 *	[team id=ID template=X]
	 *
	 * - id: the team ID
	 * - template is the template used for displaying. Replace name appropriately. Templates must be named "team-template.php" (optional)
	 *
	 * @param array $atts shorcode attributes
	 * @return string
	 */
	public function showTeam( $atts ) {
		global $league, $team;
		extract(shortcode_atts(array(
			'id' => 0,
			'template' => ''
		), $atts ));

		$league = get_league();
		if ( !is_null($league) )
			$team = $league->getTeam(intval($id));
		else
			$team = get_team(intval($id));
		
		// make sure that within a league only teams belonging to this league are displayed
		if (!is_null($league) && $team->league_id != $league->id)
			$team = false;
		
		if (!is_null($league) && empty($template) && $this->checkTemplate('team-'.$league->sport) )
			$filename = 'team-'.$league->sport;
		else
			$filename = ( !empty($template) ) ? 'team-'.$template : 'team';
		
		$out = $this->loadTemplate( $filename, array('league' => $league, 'team' => $team) );

		return $out;
	}


	/**
	 * Display Crosstable
	 *
	 * [crosstable league_id="1" mode="popup" template="name"]
	 *
	 * - league_id is the ID of league to display
	 * - mode set to "popup" makes the crosstable be displayed in a thickbox popup window.
	 * - template is the template used for displaying. Replace name appropriately. Templates must be named "crosstable-template.php" (optional)
	 * - season: display crosstable of given season (optional)
	 * - logo: 'true' or 'false'
	 * - group: show crosstable for specific group
	 *
	 * @param array $atts shorcode attributes
	 * @return string
	 */
	public function showCrosstable( $atts ) {
		global $league, $team;
		extract(shortcode_atts(array(
			'league_id' => 0,
			'season' => '',
			'group' => '',
			'template' => '',
			'mode' => '',
			'logo' => 'true'
		), $atts ));
		
		$league = $this->getLeague($league_id);
		
		$league->setTemplate('crosstable', $template);
		$league->setSeason($season);
		$league->setGroup($group);
		
		$league->show_logo = ($logo == 'true');
		$league->crosstable_mode = $mode;
		
		//$league->getTeams(array('orderby' => array('rank' => 'ASC'), 'cache' => false));
		$league->getTeams(array('orderby' => array('rank' => 'ASC')));
		if ( empty($template) && $this->checkTemplate('crosstable-'.$league->sport) )
			$filename = 'crosstable-'.$league->sport;
		else
			$filename = ( !empty($template) ) ? 'crosstable-'.$template : 'crosstable';

		$out = $this->loadTemplate( $filename, array('league' => $league) );

		return $out;
	}

	
	/**
	 * Display League
	 *
	 * [league id=ID season=X template=X]
	 *
	 * - id: ID of league
	 * - season: season to show
	 * - template: teamplate to use
	 * - standingstable: template for standings table
	 * - crosstable: template for crosstable
	 * - matches: template for matches
	 * - teams: template for teams
	 * - matches_template_type: type of match template
	 *
	 * @param array $atts shorcode attributes
	 * @return string
	 */
	public function showLeague( $atts ) {
		global $league;
		extract(shortcode_atts(array(
			'id' => 0,
			'season' => false,
			'template' => '', 
			'standingstable' => '',
			'crosstable' => '',
			'matches' => '',
			'teams' => 'list',
			'matches_template_type' => 'accordion'
		), $atts ));
		
		$league = get_league(intval($id));
		$league->setSeason($season);
		$league->setTab();
		$league->setTemplates(array( 'league' => $template, 'standingstable' => $standingstable, 'crosstable' => $crosstable, 'matches' => $matches, 'teams' => $teams ));
		$league->matches_template_type = $matches_template_type;
		
		if ( empty($template) && $this->checkTemplate('league-'.$league->sport) )
			$filename = 'league-'.$league->sport;
		else
			$filename = ( !empty($template) ) ? 'league-'.$template : 'league';

		$out = $this->loadTemplate( $filename, array('league' => $league) );
		return $out;
	}
	
	 
	/**
	 * Display Archive
	 *
	 *	[leaguearchive template=X]
	 *
	 * - template: teamplate to use
	 * - standingstable: template for standings table
	 * - crosstable: template for crosstable
	 * - matches: template for matches
	 * - teams: template for teams
	 * - matches_template_type: type of match template
	 *
	 * @param array $atts shorcode attributes
	 * @return string
	 */
	public function showArchive( $atts ) {
		global $league;
		extract(shortcode_atts(array(
			'template' => '',
			'standingstable' => '',
			'crosstable' => '',
			'matches' => '',
			'teams' => 'list',
			'matches_template_type' => 'accordion'
		), $atts ));

		// get all leagues, needed for dropdown
		$leagues = $this->getLeagues();

		// select current league either from selection or first league
		$league_id = ( isset($_GET['league_id']) && !empty($_GET['league_id']) ) ? intval($_GET['league_id']) : $leagues[0]->id;
		
		// select first league
		$league = get_league($league_id);
		$league->setTab(true);		
		$league->setTemplates(array( 'standingstable' => $standingstable, 'crosstable' => $crosstable, 'matches' => $matches, 'teams' => $teams ));
		$league->matches_template_type = $matches_template_type;
		
		if ( empty($template) && $this->checkTemplate('archive-'.$league->sport) )
			$filename = 'archive-'.$league->sport;
		else
			$filename = ( !empty($template) ) ? 'archive-'.$template : 'archive';

		$out = $this->loadTemplate( $filename, array('leagues' => $leagues, 'league' => $league) );
		return $out;
	}


	/**
	 * Load template for user display. First the current theme directory is checked for a template
	 * before defaulting to the plugin
	 *
	 * @param string $template Name of the template file (without extension)
	 * @param array $vars Array of variables name=>value available to display code (optional)
	 * @return string
	 */
	private function loadTemplate( $template, $vars = array() ) {
		global $league, $team, $match;
		extract($vars);

		ob_start();

		if ( file_exists( get_stylesheet_directory() . "/leaguemanager/$template.php")) {
			include(get_stylesheet_directory() . "/leaguemanager/$template.php");
		} elseif ( file_exists( get_template_directory() . "/leaguemanager/$template.php")) {
			include(get_template_directory() . "/leaguemanager/$template.php");
		} elseif ( file_exists(LEAGUEMANAGER_PATH . "/templates/".$template.".php") ) {
			include(LEAGUEMANAGER_PATH . "/templates/".$template.".php");
		} else {
			$this->setMessage( sprintf(__('Could not load template %s.php', 'leaguemanager'), $template), true );
			$this->printMessage();
		}
		$output = ob_get_contents();
		ob_end_clean();
		return $output;
	}


	/**
	 * check if template exists
	 *
	 * @param string $template
	 * @return boolean
	 */
	private function checkTemplate( $template ) {
		if ( file_exists( get_stylesheet_directory() . "/leaguemanager/$template.php")) {
			return true;
		} elseif  ( file_exists( get_template_directory() . "/leaguemanager/$template.php")) {
			return true;
		} elseif ( file_exists(LEAGUEMANAGER_PATH . "/templates/".$template.".php") ) {
			return true;
		}

		return false;
	}
	
	
	/**
	 * get league
	 *
	 * @param int $league_id
	 * @return null|League
	 */
	private function getLeague( $league_id ) {
		$league = null;
		if ($league_id == 0) $league = get_league();
		if (is_null($league)) $league = get_league(intval($league_id));
		
		return $league;
	}
}
?>