<?php
/**
 * Team API: Team class
 *
 * @package LeagueManager
 * @subpackage Team
 */
 
/**
 * Class to implement the Team object
 *
 */
final class Team {
	/**
	 * team ID
	 *
	 * @var int
	 */
	public $id = 0;
	
	/**
	 * team name
	 *
	 * @param string
	 */
	public $title = '';
	
	/**
	 * team website`
	 *
	 * @param string
	 */
	public $website = '';
	
	/**
	 * team coach
	 *
	 * @var string
	 */
	public $coach = '';
	
	/**
	 * team stadium
	 *
	 * @var string
	 */
	public $stadium = '';
	
	/**
	 * team logo
	 *
	 * @var string
	 */
	public $logo = '';
	
	/**
	 * home team selector
	 *
	 * @param int
	 */
	public $home = 0;
	
	/**
	 * team group
	 *
	 * @var string
	 */
	public $group = '';
	
	/**
	 * team roster
	 *
	 * @var int
	 */
	public $roster = 0;
	
	/**
	 * team profile
	 *
	 * @var int
	 */
	public $profile = 0;
	
	/**
	 * primary points
	 *
	 * @var array
	 */
	public $points = array();
	
	/**
	 * primary points plus
	 *
	 * @var int
	 */
	public $points_plus = 0;
	
	/**
	 * primary points minus
	 *
	 * @var int
	 */
	public $points_minus = 0;
	
	/**
	 * secondary points
	 *
	 * @var array
	 */
	public $points2 = array();
	
	/**
	 * secondary points plus
	 *
	 * @var int
	 */
	public $points2_plus = 0;
	
	/**
	 * secondary points minus
	 *
	 * @var int
	 */
	public $points2_minus = 0;
	
	/**
	 * add points
	 *
	 * @var int
	 */
	public $add_points = 0;
	
	/**
	 * number of done matches
	 *
	 * @var int
	 */
	public $done_matches = 0;
	
	/**
	 * number of won matches
	 *
	 * @var int
	 */
	public $won_matches = 0;
	
	/**
	 * number of draw matches
	 *
	 * @var int
	 */
	public $draw_matches = 0;
	
	/**
	 * number of lost matches
	 *
	 * @var int
	 */
	public $lost_matches = 0;
	
	/**
	 * percentage of won matches
	 *
	 * @var float
	 */
	public $winPercent = 0;
	
	/**
	 * point difference
	 *
	 * @var int
	 */
	public $diff = 0;
	
	/**
	 * league ID
	 *
	 * @var int
	 */
	public $league_id = 0;
	
	/**
	 * league season`
	 *
	 * @var string
	 */
	public $season = '';
	
	/**
	 * rank
	 *
	 * @var int
	 */
	public $rank = 0;
	
	/**
	 * rank status
	 *
	 * @var string
	 */
	public $status = '';
	
	/**
	 * custom data
	 *
	 * @var array
	 */
	public $custom = array();
	
	/**
	 * team roster datasets
	 *
	 * @var array of Dataset objects
	 */
	public $rosterData = array();
	
	/**
	 * team profile data
	 *
	 * @var Datast
	 */
	public $profileData = null;
	
	/**
	 * next match object
	 *
	 * @var Match
	 */
	public $next_match = null;
	
	/**
	 * previous match object
	 *
	 * @var Match
	 */
	public $prev_match = null;
	
	/**
	 * css class
	 *
	 * @param array
	 */
	public $cssClass = array();
	
	/**
	 * team page URL
	 *
	 * @var string
	 */
	public $pageURL = '';
	
	/**
	 * formatted points
	 *
	 * @var array
	 */
	public $pointsFormatted = array('primary' => '', 'secondary' => '');
	
	/**
	 * set project tabs for team roster and profile
	 *
	 * @var boolean
	 */
	public $projects_tabs = false;
	
	/**
	 * check if team is selected
	 *
	 * @var boolean
	 */
	public $is_selected = false;
	
	/**
	 * win percent multiplication factor
	 *
	 * @var int
	 */
	private $pct_mult = 1;
	
	
	/**
	 * retrieve team instance
	 *
	 * @param int $team_id
	 */
	public static function get_instance($team_id) {
		global $wpdb;

		$team_id = (int) $team_id;
		if ( ! $team_id )
			return false;

		$team = wp_cache_get( $team_id, 'teams' );
		
		if ( ! $team ) {
			$team = $wpdb->get_row( $wpdb->prepare( "SELECT `title`, `website`, `coach`, `stadium`, `logo`, `home`, `group`, `roster`, `profile`, `points_plus`, `points_minus`, `points2_plus`, `points2_minus`, `add_points`, `done_matches`, `won_matches`, `draw_matches`, `lost_matches`, `diff`, `league_id`, `id`, `season`, `rank`, `status`, `custom` FROM {$wpdb->leaguemanager_teams} WHERE `id` = '%d' LIMIT 1", $team_id ) );
			
			if ( !$team ) return false;
		
			$team = new Team( $team );
			
			wp_cache_add( $team->id, $team, 'teams' );
		}
		
		return $team;
	}
	
	
	/**
	 * Constructor
	 *
	 * @param object $team Team object.
	 */
	public function __construct( $team = null ) {
		if ( !is_null($team) ) {
			if ( isset($team->custom) ) {
				$team->custom = stripslashes_deep((array)maybe_unserialize($team->custom));
				$team = (object)array_merge((array)$team, (array)$team->custom);
			}
			
			foreach ( get_object_vars( $team ) as $key => $value )
				$this->$key = $value;
			
			$this->title = htmlspecialchars(stripslashes($this->title), ENT_QUOTES);
			$this->coach = stripslashes($this->coach);
			$this->website = esc_url($this->website);
			$this->stadium = stripslashes($this->stadium);
			
			$this->points_plus += $this->add_points; // add or substract extra points
			$this->points = array( 'plus' => $this->points_plus, 'minus' => $this->points_minus );
			$this->points2 = array( 'plus' => $this->points2_plus, 'minus' => $this->points2_minus );
			$this->diff = ( $this->diff > 0 ) ? '+'.$this->diff : $this->diff;
			$this->winPercent();

			$this->roster = intval($this->roster);
			$this->profile = intval($this->profile);
			if ( $this->roster > 0 && $this->profile > 0 ) $this->projects_tabs = true;
			
			// set selected marker
			if (isset($_GET['team_'.$this->league_id])) $this->is_selected = true;
			
			if (!is_admin()) {
				$url = get_permalink();
				$url = add_query_arg( 'team_'.$this->league_id, $this->id, $url );
				foreach ( $_GET AS $key => $value ) {
					$url = add_query_arg( $key, htmlspecialchars(strip_tags($value)), $url );
				}
				$this->pageURL = $url;
			}
		}
	}
	
	
	/**
	 * compute win percentage
	 */
	public function winPercent() {
		$this->winPercent = ($this->done_matches > 0) ? round(($this->won_matches + $this->draw_matches/2)/$this->done_matches,3) * $this->pct_mult : 0;
	}
	
	
	/**
	 * check home team flag
	 *
	 * @return boolean
	 */
	public function isHome() {
		return $this->home == 1;
	}
	
	
	/**
	 * check if logo is set
	 *
	 * @return boolean
	 */
	public function hasLogo() {
		return $this->logo != "";
	}
	
	
	/**
	 * get image path
	 *
	 * @param string|false $file
	 * @param string $size
	 * @return string
	 */
	public function getImagePath( $file = false, $size = 'full' ) {
		$base = WP_CONTENT_DIR.'/uploads/leagues/League-'.$this->league_id;

		if ( $file ) {
			if ( $size == 'full' || $size == '' )
				$file = basename($file);
			else
				$file = $size . "_" . basename($file);
			
			return $base .'/'. $file;
		} else {
			return $base;
		}
	}
	
	
	/**
	 * get url of image
	 *
	 * @param string|false $file image file
	 * @param string $size
	 * @return string
	 */
	public function getImageURL( $file = false, $size = 'full' ) {
		$base = WP_CONTENT_URL.'/uploads/leagues/League-'.$this->league_id;

		if ( $file ) {
			if ( $size == 'full' || $size == '' )
				$file = basename($file);
			else
				$file = $size . "_" . basename($file);
			
			if (file_exists($this->getImagePath($file, '')))
				return esc_url($base .'/'. $file);
			else
				return false;
		} else {
			return esc_url($base);
		}
	}
	
	
	/**
	 * get next match
	 *
	 * @return Match next Match object
	 */
	public function getNextMatch() {
		$league = get_league($this->league_id);
		$this->next_match = $league->getMatches( array("team_id" => $this->id, "time" => "next", "limit" => 1, "reset_limit" => true) );
		
		return $this->next_match;
	}
	
	
	/**
	 * get previous match
	 *
	 * @return Match previous Match object
	 */
	public function getPrevMatch() {
		$league = get_league($this->league_id);
		$this->prev_match = $league->getMatches( array("team_id" => $this->id, "time" => "prev", "limit" => 1, "reset_limit" => true) );
		if ($this->prev_match && $this->prev_match->score == "-:-") $this->prev_match->score = "N/A";
		
		return $this->prev_match;
	}
	
	
	/**
	 * get last 5 icons for standings table. original code by LaMonte Forthun
	 *
	 * @return string
	 */
	public function last5() {
		$league = get_league($this->league_id);
		
		$last5 = '<span>';
		// get next scheduled match
		$next_result = $league->getMatches( array("time" => "next", "team_id" => $this->id, "match_day" => -1, "limit" => 1, "reset_query_args" => true) );
		if ($next_result)
			$last5 .= '<a href="?match_'.$this->league_id.'='.$next_result->id.'"  class="N last5-bg" title="'.$next_result->tooltipTitle.'">&nbsp;</a>';
		else
			$last5 .= '<a class="N last5-bg" title="'.__('Next Match: No Game Scheduled', 'leaguemanager').'">&nbsp;</a>';
		
		// get last 5 match results
		$last_results = $league->getMatches( array("time" => "prev", "team_id" => $this->id, "match_day" => -1, "limit" => 5, "reset_query_args" => true) );
		foreach ($last_results as $key => $match) {
			$class = array();
			if ($this->id == $match->winner_id)
				$class[] = "W";
			elseif ($this->id == $match->loser_id)
				$class[] = "L";
			elseif ($match->winner_id == -1 && $match->loser_id == -1)
				$class[] = "D";
			else
				$class[] = "N";
			
			if ( $key == 2 ) $class[] = "clear";
			$last5 .= '<a href="?match_'.$this->league_id.'='.$match->id.'"  class="'.implode(' ', $class).' last5-bg" title="'.$match->tooltipTitle.'">&nbsp;</a>';
		}
		
		$last5 .= '</span>';
		
		return $last5;
	}
	
	
	/**
	 * get number of finished matches from database
	 *
	 * @return int
	 */
	public function getNumDoneMatches() {
		global $wpdb;
		$league = get_league();
		if (is_null($league)) $league = get_league($this->league_id);

		//$num_matches = $wpdb->get_var( $wpdb->prepare("SELECT COUNT(ID) FROM {$wpdb->leaguemanager_matches} WHERE `final` = '' AND (`home_team` = '%1$d' OR `away_team` = '%1$d') AND `home_points` != '' AND `away_points` != ''", $this->id) );
		$num_matches = $league->getMatches(array('count' => true, 'team_id' => $this->id, 'limit' => false, 'home_points' => 'not_empty', 'away_points' => 'not_empty', 'match_day' => -1, 'cache' => false, 'reset_query_args' => true));
		$num_matches = apply_filters('team_done_matches', $num_matches, $this->id);
		
		$this->done_matches = $num_matches;
		
		// re-compute win percentage
		$this->winPercent();
		
		return $num_matches;
	}


	/**
	 * get number of won matches from database
	 *
	 * @return int
	 */
	public function getNumWonMatches() {
		global $wpdb;
		$league = get_league();
		if (is_null($league)) $league = get_league($this->league_id);

		//$num_won = $wpdb->get_var( $wpdb->prepare("SELECT COUNT(ID) FROM {$wpdb->leaguemanager_matches} WHERE `final` = '' AND `winner_id` = '%d'", $this->id) );
		$num_won = $league->getMatches(array('count' => true, 'limit' => false, 'cache' => false, 'winner_id' => $this->id, 'match_day' => -1, 'reset_query_args' => true));
		$num_won = apply_filters('team_won_matches', $num_won, $this->id);
		
		$this->won_matches = $num_won;
		
		// re-compute win percentage
		$this->winPercent();
		
		return $num_won;
	}


	/**
	 * get number of draw matches from database
	 *
	 * @return int
	 */
	public function getNumDrawMatches() {
		global $wpdb;
		$league = get_league();
		if (is_null($league)) $league = get_league($this->league_id);

		//$num_draw = $wpdb->get_var( $wpdb->prepare("SELECT COUNT(ID) FROM {$wpdb->leaguemanager_matches} WHERE `final` = '' AND `winner_id` = -1 AND `loser_id` = -1 AND (`home_team` = '%1$d' OR `away_team` = '%1$d')", $this->id) );
		$num_draw = $league->getMatches(array('count' => true, 'limit' => false, 'cache' => false, 'team_id' => $this->id, "winner_id" => -1, "loser_id" => -1, 'match_day' => -1, 'reset_query_args' => true));
		$num_draw = apply_filters('team_draw_matches', $num_draw, $this->id);
		
		$this->draw_matches = $num_draw;
		
		// re-compute win percentage
		$this->winPercent();
		
		return $num_draw;
	}


	/**
	 * get number of lost matches from database
	 *
	 * @return int
	 */
	public function getNumLostMatches() {
		global $wpdb;
		$league = get_league();
		if (is_null($league)) $league = get_league($this->league_id);

		//$num_lost = $wpdb->get_var( $wpdb->prepare("SELECT COUNT(ID) FROM {$wpdb->leaguemanager_matches} WHERE `final` = '' AND `loser_id` = '%d'", $this->id) );
		$num_lost = $league->getMatches(array('count' => true, 'limit' => false, 'cache' => false, 'loser_id' => $this->id, 'match_day' => -1, 'reset_query_args' => true));
		$num_lost = apply_filters('team_lost_matches', $num_lost, $this->id);
		
		$this->lost_matches = $num_lost;
		
		// re-compute win percentage
		$this->winPercent();
		
		return $num_lost;
	}
	
	
	/**
	 * check if team roster is set
	 *
	 * @return boolean
	 */
	public function hasRoster() {
		if ( intval($this->roster) > 0 )
			return true;
	
		return false;
	}
	
	
	/**
	 * get team roster data, by default without pagination limit
	 *
	 * @param array $args An associative array of query args for $project->getDatasets()
	 * @return array
	 * @see Project::getDatasets()
	 */
	public function getRoster( $args = array() ) {
		global $leaguemanager;
		
		$datasets = false;
		// make sure that ProjectManager and team roster are present
		if ( $leaguemanager->hasProjectManager() && $this->hasRoster() ) {
			$league = get_league($this->league_id);
			$project = get_project($league->teamroster);
		
			if ( $league->teamroster > 0 ) {
				$args = array_merge($args, array('cat_id' => $this->roster));
				if (!isset($args['limit'])) $args['limit'] = false;
				
				$datasets = $project->getDatasets( $args );
				$this->rosterData = $datasets;
			}
		}
		
		return $datasets;
	}
	
	
	/**
	 * check if team profile is set
	 *
	 * @return boolean
	 */
	public function hasProfile() {
		if ( intval($this->profile) > 0 )
			return true;
		
		return false;
	}
	
	
	/**
	 * get team profile data
	 *
	 * @return Dataset|false
	 */
	public function getProfile() {
		global $leaguemanager;
		
		// make sure that ProjectManager and team profile are present
		if ( $leaguemanager->hasProjectManager() && $this->hasProfile() ) {
			$dataset = get_dataset($this->profile);
			$this->profileData = $dataset;
			return $dataset;
		}
		
		return false;
	}
}

/**
 * get Team object
 *
 * @param int|Team|null Team ID or team object. Defaults to global $team
 * @return Team|null
 */
function get_team( $team = null ) {
	if ( empty( $team ) && isset( $GLOBALS['team'] ) )
		$team = $GLOBALS['team'];

	if ( $team instanceof Team ) {
		$_team = $team;
	} elseif ( is_object( $team ) ) {
		$_team = new Team( $team );
	} else {
		$_team = Team::get_instance( $team );
	}

	if ( ! $_team )
		return null;

	return $_team;
}
?>