<?php
/**
 * LeagueManagerWidget API: LeagueManagerWidget class
 *
 * @package LeagueManager
 * @subpackage LeagueManagerWidget
 */
 
/**
 * Class to implement the widget
 *
 * @package LeagueManager
 * @subpackage LeagueManagerWidget
 */
class LeagueManagerWidget extends WP_Widget {
	/**
	 * index for matches in widget
	 *
	 * @var array
	 */
	private $match_index = array( 'next' => 0, 'prev' => 0 );


	/**
	 * initialize
	 *
	 * @param boolean $template set to true if widget is display in template
	 */
	public function __construct( $template = false ) {
		if ( !$template ) {
			$widget_ops = array('classname' => 'leaguemanager_widget', 'description' => __('League results and upcoming matches at a glance.', 'leaguemanager') );
			parent::__construct('leaguemanager-widget', __( 'League Manager', 'leaguemanager' ), $widget_ops);
		}
	}
	
	
	/**
	 * get index for current match
	 *
	 * @param string $type next|prev
	 * @return string
	 */
	private function getMatchIndex( $type ) {
		return $this->match_index[$type];
	}
	
	
	/**
	 * set index for current match
	 *
	 * @param int $index
	 * @param string $type
	 */
	public function setMatchIndex( $index, $type ) {
		$this->match_index[$type] = $index;
	}
	
		
	/**
	 * display widget
	 *
	 * @param array $args
	 * @param array $instance
	 */
	public function widget( $args, $instance ) {
		$defaults = array(
			'before_widget' => '<li id="'.sanitize_title(get_class($this)).'" class="widget '.get_class($this).'_'.__FUNCTION__.'">',
			'after_widget' => '</li>',
			'before_title' => '<h2 class="widgettitle">',
			'after_title' => '</h2>',
			'number' => $this->number,
		);
		$args = array_merge( $defaults, $args );
		extract( $args , EXTR_SKIP );
	
		$league = get_league( $instance['league'] );
		if (empty($instance['season']) ) {
            $season = $league->current_season['name'];
		} else {
		    $season = $instance['season'];
		}
		
		if ( empty($instance['group']) ) {
			echo $before_widget . $before_title .  $league->title . ' - ' . __('Season', 'leaguemanager') . " " . $season . $after_title;
		} else {
			echo $before_widget . $before_title . $league->title . ' - ' . __('Season', 'leaguemanager') . " " . $season . " - " . __('Group', 'leaguemanager') . " " . $instance['group'] . $after_title;
		}
				
		echo "<div class='leaguemanager_widget_content'>";
		if ( $instance['match_display'] != 'none' ) {
			$show_prev_matches = $show_next_matches = false;
			if ( $instance['match_display'] == 'prev' )
				$show_prev_matches = true;
			elseif ( $instance['match_display'] == 'next' )
				$show_next_matches = true;
			elseif ( $instance['match_display'] == 'all' )
				$show_prev_matches = $show_next_matches = true;
			
			if ( $show_next_matches ) {
				echo "<div id='next_matches_".$number."'>";
				$this->showNextMatchBox($number, $instance);
				echo "</div>";
			}

			if ( $show_prev_matches ) {
				echo "<div id='prev_matches_".$number."'>";
				$this->showPrevMatchBox($number, $instance);
				echo "</div>";
			}
	
		}
		
		if ( $instance['standingstable'] == 1 ) {		
			echo "<h4 class='standings'>". __( 'Table', 'leaguemanager' ). "</h4>";
			echo do_shortcode("[standings league_id='".$instance['league']."' group='".$instance['group']."' season='".$instance['season']."' widget_options='".$this->option_name."' widget_id='".$number."' ]");
		}

		echo "</div>";
		echo $after_widget;
	}


	/**
	 * show next match box
	 *
	 * @param int $number
	 * @param array $instance
	 * @param boolean $echo (optional)
	 * @return string|void
	 */
	public function showNextMatchBox($number, $instance, $echo = true) {
		$league = get_league($instance['league']);
		$match_limit = ( is_numeric($instance['match_limit']) && intval($instance['match_limit']) > 0 ) ? intval($instance['match_limit']) : false;			
		$match_args = array("final" => '', "season" => $instance['season'], "time" => "next");
		if ( !empty($instance['group']) ) {
			$match_args['group'] = $instance['group'];
		}

		if ( isset($instance['home_only']) && $instance['home_only'] == 1 )
			$match_args['home_only'] = true;
		
		$match_args['limit'] = $match_limit;
			
		$matches = $league->getMatches( $match_args );
		if ( $matches ) {
			$curr = $this->getMatchIndex('next');
			if (isset($matches[$curr]))
				$match = $matches[$curr];
			else
				die("Error: Match with index ".$curr." does not exist");
			
			$match_limit_js = ( $match_limit ) ? $match_limit : 'false';
			$home_only = ( isset($instance['home_only']) ) ? $home_only = $instance['home_only'] : $home_only = 0;

			$next_link = $prev_link = '';
			if ( $curr < count($matches) - 1 ) {
				$next_link = "<a class='next' href='#null' onclick='Leaguemanager.setMatchBox(\"".admin_url( 'admin-ajax.php' )."\", ".$curr.", \"next\", \"next\", ".$instance['league'].", \"".$match_limit_js."\", ".$number.", \"".$instance['season']."\", \"".$instance['group']."\", ".intval($home_only).", \"".$instance['date_format']."\"); return false'><img src='".LEAGUEMANAGER_URL."/images/arrow_right.png' alt='&raquo;' /></a>";
			}
			if ( $curr > 0 ) {
				$prev_link = "<a class='prev' href='#null' onclick='Leaguemanager.setMatchBox(\"".admin_url( 'admin-ajax.php' )."\", ".$curr.", \"prev\", \"next\", ".$instance['league'].", \"".$match_limit_js."\", ".$number.", \"".$instance['season']."\", \"".$instance['group']."\", ".intval($home_only).", \"".$instance['date_format']."\"); return false'><img src='".LEAGUEMANAGER_URL."/images/arrow_left.png' alt='&laquo;' /></a>";
			}
	
			$out = "<div id='next_match_box_".$number."' class='match_box'>";
			$out .= "<h4>$prev_link".__( 'Next Match', 'leaguemanager' )."$next_link</h4>";
						
			$out .= "<div class='match' id='match-".$match->id."'>";
			
			$homeTeam = $match->teams['home'];
			$awayTeam = $match->teams['away'];
			
			if ( $homeTeam->website != '' )
				$home_team = "<a href='http://".$homeTeam->website."' target='_blank'>".$homeTeam->title."</a>";
			if ( $awayTeam->website != '' )
				$away_team = "<a href='http://".$awayTeam->website."' target='_blank'>".$awayTeam->title."</a>";
								
			$out .= "<p class='match_title'><strong>".$match->getTitle(false)."</strong></p>";
			$out .= "<p class='logos'><img class='home_logo' src='".$homeTeam->getImageURL($homeTeam->logo, 'thumb')."' alt='' /><img class='away_logo' src='".$awayTeam->getImageURL($awayTeam->logo, 'thumb')."' alt='' /></p>";

			if ( !empty($match->match_day) )
			$out .= "<p class='match_day'>".sprintf(__("<strong>%d.</strong> Match Day", 'leaguemanager'), $match->match_day)."</p>";
			
			$match->setDate($instance['date_format']);
			$out .= "<p class='date'>".$match->match_date.", <span class='time'>".$match->start_time."</span></p>";
			$out .= "<p class='location'>".$match->location."</p>";
			
			$out .= "</div></div>";
		
	
			if ( $echo )
				echo $out;
				
			return $out;
		}
	}
	
	
	/**
	 * show previous match box
	 *
	 * @param int $number
	 * @param array $instance
	 * @param boolean $echo (optional)
	 * @return string|void
	 */
	public function showPrevMatchBox($number, $instance, $echo = true) {
		$league = get_league($instance['league']);
		$match_limit = intval($instance['match_limit']);
		$match_args = array("final" => '', "season" => $instance['season'], "time" => "prev");
		if ( !empty($instance['group']) ) {
			$match_args['group'] = $instance['group'];
		}

		if ( isset($instance['home_only']) && $instance['home_only'] == 1 )
			$match_args['home_only'] = true;
		
		$match_args['limit'] = $match_limit;
		$match_args['orderby'] = array("date" => "DESC", "id" => "DESC");
		
		$matches = $league->getMatches( $match_args );
		if ( $matches ) {		
			$curr = $this->getMatchIndex('prev');
			if (isset($matches[$curr]))
				$match = $matches[$curr];
			else
				die("Error: Match with index ".$curr." does not exist");
		
			
			$match_limit_js = ( $match_limit > 0 ) ? $match_limit : 'false';
			$home_only = ( isset($instance['home_only']) ) ? $instance['home_only'] : 0;
			
			$next_link = $prev_link = '';
			if ( $curr < count($matches) - 1 ) {
				$next_link = "<a class='next' href='#null' onclick='Leaguemanager.setMatchBox(\"".admin_url( 'admin-ajax.php' )."\", ".$curr.", \"next\", \"prev\", ".$instance['league'].", \"".$match_limit_js."\", ".$number.", \"".$instance['season']."\", \"".$instance['group']."\", ".intval($home_only).", \"".$instance['date_format']."\"); return false'><img src='".LEAGUEMANAGER_URL."/images/arrow_right.png' alt='&raquo;' /></a>";
			}
			if ( $curr > 0 ) {
				$prev_link = "<a class='prev' href='#null' onclick='Leaguemanager.setMatchBox(\"".admin_url( 'admin-ajax.php' )."\", ".$curr.", \"prev\", \"prev\", ".$instance['league'].", \"".$match_limit_js."\", ".$number.", \"".$instance['season']."\", \"".$instance['group']."\", ".intval($home_only).", \"".$instance['date_format']."\"); return false'><img src='".LEAGUEMANAGER_URL."/images/arrow_left.png' alt='&laquo;' /></a>";
			}
			
			$out = "<div id='prev_match_box_".$number."' class='match_box'>";
			$out .= "<h4>$prev_link".__( 'Last Match', 'leaguemanager' )."$next_link</h4>";			
			
			$out .= "<div class='match' id='match-".$match->id."'>";
			
			$homeTeam = $match->teams['home'];
			$awayTeam = $match->teams['away'];

			if ( $homeTeam->website != '' )
				$homeTeam->title = "<a href='".esc_url($homeTeam->website)."' target='_blank'>".$homeTeam->title."</a>";
			if ( $awayTeam->website != '' )
				$awayTeam->title = "<a href='".esc_url($awayTeam->website)."' target='_blank'>".$awayTeam->title."</a>";

			$out .= "<p class='match_title'><strong>".$match->getTitle(false)."</strong></p>";
			$out .= "<p class='logos'><img class='home_logo' src='".$homeTeam->getImageURL($homeTeam->logo)."' alt='' /><span class='result'>".$match->score."</span><img class='away_logo' src='".$awayTeam->getImageURL($awayTeam->logo)."' alt='' /></p>";

			if ( !empty($match->match_day) )
			$out .= "<p class='match_day'>".sprintf(__("<strong>%d.</strong> Match Day", 'leaguemanager'), $match->match_day)."</p>";
			
			$match->setDate($instance['date_format']);
			$out .= "<p class='date'>".$match->match_date.", <span class='time'>".$match->start_time."</span></p>";
			$out .= "<p class='location'>".$match->location."</p>";
			
			if ( $match->report != "" && ( isset($instance['report']) && ($instance['report'] == 1)) )
				$out .=  "<p class='report'>".$match->report."</p>";
					
			$out .= "</div></div>";
		
			if ( $echo )
				echo $out;
			
			return $out;
		}
	}
	
	
	/**
	 * save settings
	 *
	 * @param array $new_instance
	 * @param array $old_instance
	 * @return array
	 */
	public function update( $new_instance, $old_instance ) {
		global $leaguemanager;
		
		$new_instance['home_only'] = isset($new_instance['home_only']) ? 1 : 0;
		$new_instance['report'] = isset($new_instance['report']) ? 1 : 0;
		$new_instance['match_limit'] = intval($new_instance['match_limit']);
		$new_instance['standingstable'] = isset($new_instance['standingstable']) ? 1 : 0;
		
		foreach ( $leaguemanager->getStandingsDisplayOptions() AS $key => $label ) {
			$new_instance['standings'][$key] = isset($new_instance['standings'][$key]) ? 1 : 0;
		}
		
		return $new_instance;
	}


	/**
	 * widget control panel
	 *
	 * @param array $instance
	 */
	public function form( $instance ) {
		global $leaguemanager;
		
		$group = ( isset($instance['group']) ) ? $instance['group'] : '';

		$season = isset($instance['season']) ? $instance['season'] : '';
		$match_limit = isset($instance['match_limit']) ? $instance['match_limit'] : '';
		$date_format = isset($instance['date_format']) ? $instance['date_format'] : '';
		
		echo '<div class="leaguemanager_widget_control" id="leaguemanager_widget_control_'.$this->number.'">';
		echo '<p><label for="'.$this->get_field_id('league').'">'.__('League','leaguemanager').': </label>';
		echo '<select size="1" name="'.$this->get_field_name('league').'" id="'.$this->get_field_id('league').'">';
		foreach ( $leaguemanager->getLeagues() AS $league ) {
			$selected = ( $instance['league'] == $league->id ) ? ' selected="selected"' : '';
			echo '<option value="'.$league->id.'"'.$selected.'>'.$league->title.'</option>';
		}
		echo '</select>';
		echo '<p><label for="'.$this->get_field_id('season').'">'.__('Season','leaguemanager').': </label><input type="text" name="'.$this->get_field_name('season').'" id="'.$this->get_field_id('season').'" size="8" value="'.$season.'" /></p>';
		echo '<p><label for="'.$this->get_field_id('group').'">'.__('Group','leaguemanager').': </label><input type="text" name="'.$this->get_field_name('group').'" id="'.$this->get_field_id('group').'" size="8" value="'.$group.'" /></p>';
		echo '<p><label for="'.$this->get_field_id('match_display').'">'.__('Matches','leaguemanager').': </label>';
		$match_display = array( 'none' => __('Do not show','leaguemanager'), 'prev' => __('Last Matches','leaguemanager'), 'next' => __('Next Matches','leaguemanager'), 'all' => __('Next & Last Matches','leaguemanager') );
		echo '<select size="1" name="'.$this->get_field_name('match_display').'" id="'.$this->get_field_id('match_display').'">';
		foreach ( $match_display AS $key => $text ) {
			$selected = ( $key == $instance['match_display'] ) ? ' selected="selected"' : '';
			echo '<option value="'.$key.'"'.$selected.'>'.$text.'</option>';
		}
		echo '</select></p>';
		$home_checked = ( isset($instance['home_only']) && $instance['home_only'] == 1 ) ? ' checked="checked"' : '';
		echo '<p><input type="checkbox" name="'.$this->get_field_name('home_only').'" id="'.$this->get_field_id('home_only').'" value="1"'.$home_checked.' /><label for="'.$this->get_field_id('home_only').'" class="right">'.__('Only own matches','leaguemanager').'</label></p>';
		echo '<p><label for="'.$this->get_field_id('match_limit').'">'.__('Limit','leaguemanager').': </label><input type="text" name="'.$this->get_field_name('match_limit').'" id="'.$this->get_field_id('match_limit').'" value="'.$match_limit.'" size="5" /></p>';
		
		$table_checked = ( isset($instance['standingstable']) && $instance['standingstable'] == 1 ) ? ' checked="checked"' : '';
		echo '<p><input type="checkbox" name="'.$this->get_field_name('standingstable').'" id="'.$this->get_field_id('standingstable').'" value="1"'.$table_checked.' /><label for="'.$this->get_field_id('standingstable').'" class="right">'.__('Show Standings','leaguemanager').'</label></p>';
		echo '<h4>'.__( 'Standings Table Display', 'leaguemanager' ).'</h4>';
		if ( $instance['league'] > 0 ) $league = get_league($instance['league']);
		foreach ( $leaguemanager->getStandingsDisplayOptions() AS $key => $label ) {
			$checked = isset($instance['standings'][$key]) && $instance['standings'][$key] == 1 ? ' checked="checked"' : '';
			echo '<p><input type="checkbox" name="'.$this->get_field_name('standings['.$key.']').'" id="'.$this->get_field_id('standings['.$key.']').'" value="1" '.$checked.' /><label for="'.$this->get_field_id('standings['.$key.']').'">'.$label.'</label></p>';
		}
		echo '<hr />';
		
		$report_checked = ( isset($instance['report']) && $instance['report'] == 1 ) ? ' checked="checked"' : '';
		echo '<p><input type="checkbox" name="'.$this->get_field_name('report').'" id="'.$this->get_field_id('report').'" value="1"'.$report_checked.' /><label for="'.$this->get_field_id('report').'" class="right">'.__('Link to report','leaguemanager').'</label></p>';
		echo '<p><label for="'.$this->get_field_id('date_format').'">'.__('Date Format').': </label><input type="text" id="'.$this->get_field_id('date_format').'" name="'.$this->get_field_name('date_format').'" value="'.$date_format.'" size="10" /></p>';
		echo '</div>';
		
		return;
	}
}
?>