<?php
/**
 * League_Baseball API: League_Baseball class
 *
 * @package LeagueManager
 * @subpackage League_Baseball
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_baseball' );
/**
 * add baseball to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_baseball( $sports ) {
	$sports['baseball'] = __( 'Baseball', 'leaguemanager' );
	return $sports;
}

/**
 * Class to implement the League_Baseball object
 *
 */
final class League_Baseball extends League {
	/**
	 * sports key
	 *
	 * @var string
	 */
	public $sport = 'baseball';

	/**
	 * define field keys
	 *
	 * @var array
	 */
	private $keys = array( 'runs' => array('for','against'), 'shutouts' => array('home', 'away') );
	
	
	/**
	 * load specific settings
	 *
	 * @param object $league League object
	 */
	public function __construct($league) {
		parent::__construct($league);
		
		// save field keys and translated labels
		$fields = array( 'runs' => __( 'Runs', 'leaguemanager' ), 'gb' => __( 'GB', 'leaguemanager' ), 'shutouts' => __( 'SO', 'leaguemanager' ) );
		foreach ( $fields AS $key => $label ) {
			$this->fields_team[$key] = array('label' => $label, 'type' => 'input');
		}
			
		$fields = array( 'runs' => __( 'Runs', 'leaguemanager' ), 'shutouts' => __( 'SO', 'leaguemanager' ) );
		foreach ( $fields AS $key => $label ) {
			$this->fields_match[$key] = array('label' => $label, 'keys' => $this->keys[$key]);
		}
	}

	
	/**
	 * rank Teams
	 * 1. Won matches DESC
	 * 2. Lost matches ASC
	 *
	 * @param array $teams
	 * @return array of teams
	 */
	protected function rankTeams( $teams ) {
		$won = $lost = array();
		foreach ( $teams AS $key => $row ) {
			$won[$key] = $row->won_matches;
			$lost[$key] = $row->lost_matches;
		}

		array_multisort( $won, SORT_DESC, $lost, SORT_ASC, $teams );
		return $teams;
	}


	/**
	 * get custom standings data
	 *
	 * @param Team $team
	 * @param array $data
	 * @param array|false $matches
	 * @return array
	 */
	protected function getStandingsData($team, $data, $matches = false) {
		$data['runs'] = $this->getRuns($team, $matches);
		$data['gb'] = $this->getGamesBehind($team, $matches);
		$data['shutouts'] = $this->getShutouts($team, $matches);
		
		return $data;
	}
	
	
	/**
	 * get number of runs for team
	 *
	 * @param Team $team
	 * @param array|false $matches
	 * @return array number of runs for and against as associative array
	 */
	private function getRuns( $team, $matches = false ) {
		$runs = array( 'for' => 0, 'against' => 0 );

		if ( !$matches ) $matches =  $this->getMatches( array("team_id" => $team->id, "match_day" => -1, "limit" => false, "cache" => false, "reset_query_args" => true) );
		foreach ( $matches AS $match ) {
			if ( $match->home_team == $team->id ) {
				$runs['for'] += $match->runs['for'];
				$runs['against'] += $match->runs['against'];
			}
			if ( $match->away_team == $team->id ) {
				$runs['for'] += $match->runs['against'];
				$runs['against'] += $match->runs['for'];
			}
		}

		return $runs;
	}


	/**
	 * get number of games behind
	 *
	 * @param Team $team
	 * @param array|false $matches
	 * @return int games behind
	 */
	private function getGamesBehind( $team, $matches ) {
		global $wpdb;

		if ( $team->rank == 1 ) {
			return 0;
		} else {
			$first = $wpdb->get_row( $wpdb->prepare("SELECT `rank`, `won_matches`, `lost_matches` FROM {$wpdb->leaguemanager_teams} WHERE `rank` = 1 AND `league_id` = '%d' AND `season` = '%s' LIMIT 1", $team->league_id, $team->season) );
			
			if ( $first )
				$gb = ( $first->won_matches - $team->won_matches + $team->lost_matches - $first->lost_matches ) / 2;
			else
				$gb = 0;
			
			return round($gb, 3);
		}
	}


	/**
	 * get number of shutouts
	 *
	 * @param Team $team
	 * @param array|false $matches
	 * @return int number of shutouts
	 */
	private function getShutouts( $team, $matches = false ) {	
		$shutouts = 0;

		if( !$matches ) $matches = $this->getMatches( array("team_id" => $team->id, "match_day" => -1, "limit" => false, "cache" => false, "reset_query_args" => true) );
		foreach ( $matches AS $match ) {
			if ( $match->home_team == $team->id ) {
				$shutouts += $match->shutouts['home'];
			} else {
				$shutouts += $match->shutouts['away'];
			}
		}
		
		return $shutouts;
	}
}
?>