<?php
/**
 * League_Basketball API: League_Basketball class
 *
 * @package LeagueManager
 * @subpackage League_Basketball
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_basketball' );
/**
 * add basketball to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_basketball( $sports ) {
	$sports['basketball'] = __( 'Basketball', 'leaguemanager' );
	return $sports;
}

/**
 * Class to implement the League_Basketball object
 *
 */
final class League_Basketball extends League {
	/**
	 * sports keys
	 *
	 * @var string
	 */
	public $sport = 'basketball';

	
	/**
	 * load specific settings
	 *
	 * @param League $league
	 */
	public function __construct($league) {
		parent::__construct($league);
		
		$this->fields_match['quarters'] = array( 'label' => __( 'Quarters', 'leaguemanager' ), 'keys' => array() );
		for ( $i = 1; $i <= 4; $i++ ) {
			$this->fields_match['quarters']['keys'][$i] = array('plus','minus');
		}
		$this->fields_match['overtime'] = array( 'label' => __( 'Overtime', 'leaguemanager' ), 'keys' => array('home', 'away') );
		
		$this->fields_team['points2'] = array('label' => __( 'Baskets', 'leaguemanager' ), 'keys' => array('plus', 'minus'), 'type' => 'input');
		$this->fields_team['diff'] = array('label' => __( 'Diff', 'leaguemanager' ), 'desc' => __('Basekets Difference', 'leaguemanager'));
	}

	
	
	/**
	 * rank Teams
	 * 1. Primary points DESC
	 * 2. Win Percentage DESC
	 * 3. Games Advance DESC
	 *
	 * @param array $teams
	 * @return array
	 */
	protected function rankTeams( $teams ) {
		$points = $GA = $WinPerc = array();
		foreach ( $teams AS $key => $row ) {
			$points[$key] = $row->points['plus'];
			$GA[$key] = round((($row->points2_plus) > 0 ? (($row->points2_minus) > 0 ? ((($row->points2_plus)/($row->points2_minus))*1000) : 1000) : 0),0);
            $WinPerc[$key] =  $row->winPercent;
		}
		
		array_multisort( $points, SORT_DESC, SORT_NUMERIC, $WinPerc, SORT_DESC, SORT_NUMERIC, $GA, SORT_DESC, SORT_NUMERIC, $teams );
		return $teams;
	}


	/**
	 * update match results and automatically calculate apparatus points and score
	 *
	 * @param Match $match
	 * @return Match
	 */
	protected function updateResults( $match ) {
		$force = true;
		if (is_admin() && current_user_can('update_results')) {
			if ( $force || $match->home_points == "" && $match->away_points == "" ) {
				$score = array( 'home' => 0, 'guest' => 0 );
				foreach ( $match->quarters AS $quarter ) {
					if ($quarter['plus'] != '' && $quarter['minus'] != '') {
						$score['home'] += intval($quarter['plus']);
						$score['guest'] += intval($quarter['minus']);
					}
				}
				if ($match->overtime['home'] != '' && $match->overtime['away'] != '') {
					$score['home'] += intval($match->overtime['home']);
					$score['guest'] += intval($match->overtime['away']);			
				}
				
				if ( $score['home'] == 0 && $score['guest'] == 0 )
						$score['home'] = $score['guest'] = '';
					
				$match->home_points = $score['home'];
				$match->away_points = $score['guest'];
			}
		}
		
		return $match;
	}
}
?>