<?php
/**
 * League_TwoScores API: League_TwoScores class
 *
 * @package LeagueManager
 * @subpackage League_TwoScores
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_two_scores' );
/**
 * add two scores to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_two_scores( $sports ) {
	$sports['twoScores'] = __( 'Sports with two Scores', 'leaguemanager' );
	return $sports;
}

 
/**
 * Class to implement the League_TwoScores object
 *
 */
final class League_TwoScores extends League {
	/**
	 * sports key
	 *
	 * @var string
	 */
	public $sport = 'twoScores';


	/**
	 * load custom setings
	 *
	 * @param object $league League object
	 */
	public function __construct($league) {
		parent::__construct($league);
		
		$this->fields_team['points2'] = array('label' => __( 'Scores', 'leaguemanager' ), 'keys' => array('plus', 'minus'), 'type' => 'input');
		$this->fields_team['diff'] = array('label' => __( 'Diff', 'leaguemanager' ), 'desc' => __('Scores Difference', 'leaguemanager'));
	}

	
	/**
	 * rank Teams
	 * 1. Primary points DESC
	 * 2. Secondary Point Difference DESC
	 * 3. Done Matches ASC
	 *
	 * @param array $teams
	 * @return array of teams
	 */
	protected function rankTeams( $teams ) {
		$points = $done = $diff = array();
		foreach ( $teams AS $key => $row ) {
			$points[$key] = $row->points['plus'];
			$done[$key] = $row->done_matches;
			$diff[$key] = $row->diff;
		}

		array_multisort( $points, SORT_DESC, $diff, SORT_DESC, $done, SORT_ASC, $teams );
		return $teams;
	}
}
?>