<?php
/**
 * League_Pool API: League_Pool class
 *
 * @package LeagueManager
 * @subpackage League_Pool
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_pool' );
/**
 * add pool to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_pool( $sports ) {
	$sports['pool'] = __( 'Pool', 'leaguemanager' );
	return $sports;
}

/**
 * Class to implement the League_Pool object
 *
 */
final class League_Pool extends League {
	/**
	 * sports key
	 *
	 * @var string
	 */
	public $sport = 'pool';


	/**
	 * load specific settings
	 *
	 * @param object $league League object
	 */
	public function __construct($league) {
		parent::__construct($league);
		
		$fields = array('forScore' => __('For', 'leaguemanager'), 'againstScore' => __('Against', 'leaguemanager'));
		foreach ( $fields AS $key => $label ) {
			$this->fields_team[$key] = array('label' => $label, 'type' => 'input');
			$this->fields_match[$key] = array('label' => $label);
		}
	}

	
	/**
	 * rank Teams
	 * 1. Primary points
	 * 2. Secondary points difference
	 * 3. Won games
	 * 4. Draw games
	 * 5. Done games
	 *
	 * @param array $teams
	 * @return array of teams
	 */
	protected function rankTeams( $teams ) {
		$points = $done = $diff = $won = $draw = array();
		foreach ( $teams AS $key => $row ) {
			$points[$key] = $row->points['plus'];
			$diff[$key] = $row->forScore - $row->againstScore;
			$won[$key] = $row->won_matches;
			$draw[$key] = $row->draw_matches;
			$done[$key] = $row->done_matches;
		}

		array_multisort( $points, SORT_DESC, $diff, SORT_DESC, $won, SORT_DESC, $draw, SORT_DESC, $done, SORT_ASC, $teams );
		return $teams;
	}


	/**
	 * get standings data for given team
	 *
	 * @param Team $team
	 * @param array $data
	 * @param array|false $matches
	 * @return array
	 */
	protected function getStandingsData( $team, $data, $matches = false ) {
		if ( !$matches ) $matches = $this->getMatches( array("team_id" => $team->id, "limit" => false, "cache" => false, "match_day" => -1, "reset_query_args" => true) );
		
		$data['forScore'] = $this->getPoolScore($team, 'for', $matches);
		$data['againstScore'] = $this->getPoolScore($team, 'against', $matches);
		
		return $data;
	}
	

	/**
	 * get score for left balls
	 *
	 * @param int $team_id
	 * @param string $index
	 * @param array $matches
	 * @return array number of runs for and against as assoziative array
	 */
	private function getPoolScore( $team_id, $index, $matches ) {
		$score = array( 'for' => 0, 'against' => 0 );
		foreach ( $matches AS $match ) {
			if ( $team_id == $match->home_team ) {
				$score['for'] += $match->forScore;
				$score['against'] += $match->againstScore;
			} else {
				$score['for'] += $match->againstScore;
				$score['against'] += $match->forScore;
			}
		}

		return $score[$index];
	}
}
?>