<?php
/**
 * League_Racing API: League_Racing class
 *
 * @package LeagueManager
 * @subpackage League_Racing
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_racing' );
/**
 * add racing to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_racing( $sports ) {
	$sports['racing'] = __( 'Racing', 'leaguemanager' );
	return $sports;
}

/**
 * Class to implement the League_Racing object
 *
 */
final class League_Racing extends League {
	/**
	 * sports keys
	 *
	 * @var string
	 */
	public $sport = 'racing';


	/**
	 * load specific settings
	 *
	 * @param object $league League object
	 */
	public function __construct($league) {
		parent::__construct($league);
		
		$this->fields_match['racetype'] = array('label' => __( 'Race Type', 'leaguemanager' ));
		$this->fields_match['raceresults'] = array('label' => __( 'Results', 'leaguemanager' ), 'url' => 'admin.php?page=leaguemanager&subpage='.$this->sport);
		
		// race results need to be inserted in matches tables
		add_filter( 'league_menu_'.$this->sport, array(&$this, 'menu'), 10, 3 );
		
		add_action('leaguemanager_edit_match_'.$this->sport, array(&$this, 'matchForm'), 10, 5);
	}


	/**
	 * add menu page
	 *
	 * @param array $menu
	 * @param int $league_id
	 * @param mixed $season
	 * @return array
	 */
	public function menu( $menu, $league_id, $season ) {
		$menu[$this->sport] = array( 'title' => __( 'Race Results', 'leaguemanager' ), 'file' => false, 'show' => false, 'callback' => array(&$this, 'page'), 'cap' => 'update_results', 'display' => false );
		return $menu;
	}


	/**
	 * custom match editing form
	 *
	 * @param object $teams
	 * @param int $max_matches
	 * @param array $matches
	 * @param string $submit_title
	 * @param string $mode
	 */
	public function matchForm( $teams, $max_matches, $matches, $submit_title, $mode )  {
		global $leaguemanager;
		$league = get_league();
		$edit = ($mode == 'edit') ? true : false;
		$class = '';
	?>
		<form action="admin.php?page=leaguemanager&amp;subpage=show-league&amp;league_id=<?php echo $league->id?>&amp;season=<?php echo $season['name'] ?>" method="post">
			<?php wp_nonce_field( 'leaguemanager_manage-matches' ) ?>
			
			<table class="widefat wp-list-table leagues">
				<thead>
					<tr>
						<th scope="col" class="manage-column column-date column-primary"><?php _e( 'Date', 'leaguemanager' ) ?></th>
						<?php if ( !isset($edit) || (isset($edit) && !$edit) ) : ?>
						<th scope="col" class="manage-column column-add"><?php _e( 'Add', 'leaguemanager' ) ?></th>
						<?php endif; ?>
						<th scope="col" class="manage-column column-match_day"><?php _e( 'Match Day', 'leaguemanager' ) ?></th>
						<th scope="col" class="manage-column column-event"><?php _e( 'Event', 'leaguemanager' ) ?></th>
						<th scope="col" class="manage-column column-location"><?php _e( 'Location','leaguemanager' ) ?></th>
						<th scope="col" class="manage-column column-race_type"><?php _e( 'Race Type', 'leaguemanager' ) ?></th>
						<th scope="col" class="manage-column column-begin"><?php _e( 'Begin','leaguemanager' ) ?></th>
						<?php if ( count($league->groups) > 0 && !$is_finals ) : ?>
						<th scope="col" class="manage-column column-group"><?php _e( 'Group', 'leaguemanager' ) ?></th>
						<?php endif; ?>
						<th scope="col" class="manage-column column-status"><?php _e( 'Status','leaguemanager' ) ?></th>
						<th scope="col"><?php _e( 'Description', 'leaguemanger' ) ?></th>
					</tr>
				</thead>
				<tbody class="the-list">
				<?php for ( $i = 0; $i < $max_matches; $i++ ) : $class = ( 'alternate' == $class ) ? '' : 'alternate'; if (!isset($matches[$i]->racetype)) $matches[$i]->racetype = ''; if (!isset($matches[$i]->description)) $matches[$i]->description = ''; if (!isset($matches[$i]->title)) $matches[$i]->title = '';?>
				<tr class="<?php echo $class; ?>">
					<td class="column-date column-primary" data-colname="<?php _e( 'Date', 'leaguemanager' ) ?>">
						<input type="text" name="mydatepicker[<?php echo $i ?>]" id="mydatepicker[<?php echo $i ?>]" class="mydatepicker" value="<?php echo substr($matches[$i]->date, 0, 10); ?>" onChange="Leaguemanager.setMatchDate('<?php bloginfo( 'wpurl' ); ?>/wp-admin/admin-ajax.php', this.value, <?php echo $i ?>, <?php echo $max_matches ?>, '<?php echo $mode ?>');" /><button class="toggle-row" type="button"></button>
						<?php //echo $leaguemanager->getDateSelection( $matches[$i]->day, $matches[$i]->month, $matches[$i]->year, $i) ?>
					</td>
					<?php if ( !isset($edit) || (isset($edit) && !$edit) ) : ?>
					<td class="column-add" data-colname="<?php _e( 'Add', 'leaguemanager' ) ?>">
						<input type="checkbox" name="add_match[<?php echo $i ?>]" id="add_match_<?php echo $i ?>" />
					</td>
					<?php endif; ?>
					<td class="column-match_day" data-colname="<?php _e( 'Match Day', 'leaguemanager' ) ?>">
					<select size="1" name="match_day[<?php echo $i ?>]">
						<?php for ($d = 1; $d <= $league->current_season['num_match_days']; $d++) : ?>
							<option value="<?php echo $d ?>" <?php selected($d, $league->match_day); ?>><?php echo $d ?></option>
						<?php endfor; ?>
					</select>
					</td>
					<td class="column-event" data-colname="<?php _e( 'Event', 'leaguemanager' ) ?>">
						<input type="text" size="15" name="custom[<?php echo $i ?>][title]" id="title_<?php echo $i ?>" value="<?php echo $matches[$i]->title ?>" />
						
						<?php if ( count($league->groups) == 0 || $is_finals ) : ?>
						<input type="hidden" name="group[<?php echo $i ?>]" id="group_<?php echo $i ?>" value="" />
						<?php endif; ?>
					</td>
					<td class="column-location" data-colname="<?php _e( 'Location','leaguemanager' ) ?>"><input type="text" name="location[<?php echo $i ?>]" id="location[<?php echo $i ?>]" size="20" value="<?php echo $matches[$i]->location ?>" size="30" /></td>
					<td class="column-race_type" data-colname="<?php _e( 'Race Type', 'leaguemanager' ) ?>"><input type="text" size="15" name="custom[<?php echo $i ?>][racetype]" id="racetype_<?php echo $i ?>" value="<?php echo $matches[$i]->racetype ?>" /></td>
					<td class="column-begin" data-colname="<?php _e( 'Begin','leaguemanager' ) ?>">
						<select size="1" name="begin_hour[<?php echo $i ?>]">
						<?php for ( $hour = 0; $hour <= 23; $hour++ ) : ?>
							<option value="<?php echo str_pad($hour, 2, 0, STR_PAD_LEFT) ?>"<?php selected( $hour, $matches[$i]->hour ) ?>><?php echo str_pad($hour, 2, 0, STR_PAD_LEFT) ?></option>
						<?php endfor; ?>
						</select>
						<select size="1" name="begin_minutes[<?php echo $i ?>]">
						<?php for ( $minute = 0; $minute <= 60; $minute++ ) : ?>
							<?php if ( 0 == $minute % 5 && 60 != $minute ) : ?>
							<option value="<?php  echo str_pad($minute, 2, 0, STR_PAD_LEFT) ?>"<?php selected( $minute, $matches[$i]->minutes ) ?>><?php echo str_pad($minute, 2, 0, STR_PAD_LEFT) ?></option>
							<?php endif; ?>
						<?php endfor; ?>
						</select>
					</td>
					<?php if ( count($league->groups) > 0 && !$is_finals ) : ?>
					<td class="column-group" data-colname="<?php _e( 'Group', 'leaguemanager' ) ?>">
						<select size="1" name="group[<?php echo $i ?>]" id="group_<?php echo $i ?>">
							<?php foreach ( $league->groups AS $group ) : ?>
							<option value="<?php echo $group ?>" <?php selected( $group, $matches[$i]->group ) ?>><?php echo $group ?></option>
							<?php endforeach; ?>
						</select>
					</td>
					<?php endif; ?>
					<td class="column-status" data-colname="<?php _e( 'Status', 'leaguemanager' ) ?>">
						<select size="1" name="status[<?php echo $i ?>]">
							<?php foreach ( $matches[$i]->getStatusOptions() AS $key => $value ) : ?>
								<option value="<?php echo $key ?>" <?php selected( $key, $matches[$i]->status ); ?>><?php echo $value ?></option>
							<?php endforeach; ?>
						</select>
					</td>
					<td><textarea name="custom[<?php echo $i ?>][description]" id="description_<?php echo $i ?>" cols="20" rows="5"><?php echo $matches[$i]->description ?></textarea></td>
					<input type="hidden" name="match[<?php echo $i ?>]" value="<?php echo $matches[$i]->id ?>" />
				</tr>
				<?php endfor; ?>
				</tbody>
			</table>
			
			<input type="hidden" name="mode" value="<?php echo $mode ?>" />
			<input type="hidden" name="league_id" value="<?php echo $league->id ?>" />
			<input type="hidden" name="season" value="<?php echo $season['name'] ?>" />
			<input type="hidden" name="final" value="" />
			<input type="hidden" name="updateLeague" value="match" />
			
			<p class="submit"><input type="submit" value="<?php echo $submit_title ?> &raquo;" class="button" /></p>
		</form>
	<?php
	}


	/**
	 * display reslts managing page
	 *
	 */
	public function page() {
		global $leaguemanager;
		
		$league = get_league();
		
		$match_id = intval($_GET['match']);
		$season = htmlspecialchars($_GET['season']);

		if ( isset($_POST['save_results']) ) {
			$this->saveResults( $_POST['racer'], $_POST['racer_name'], $_POST['category'], $_POST['points'], $_POST['time'], $_POST['info'], $match_id );
			$leaguemanager->printMessage();
		}
		
		$match = get_match($match_id);
		$teams = $this->getTeams(array('cache' => false));
		$team_list = $teams;
		
		$team_id = isset($_GET['team']) ? (int)$_GET['team'] : false;
		if ($team_id) $teams = array(get_team($team_id));
		$class = '';
	?>
	<div class="wrap">
		<p class="leaguemanager_breadcrumb"><a href="admin.php?page=leaguemanager"><?php _e( 'LeagueManager', 'leaguemanager' ) ?></a> &raquo; <a href="admin.php?page=leaguemanager&amp;subpage=show-league&amp;league_id=<?php echo $this->id ?>"><?php echo $this->title ?></a> &raquo; <?php _e( 'Race Results', 'leaguemanager' ) ?></p>
		<h2><?php printf(__( 'Racing Results - %s', 'leaguemanager' ), $match->match_title) ?></h2>
		
		<!--
		<div class="tablenav alignright">
			<form action="admin.php" method="get">
				<input type="hidden" name="page" value="leaguemanager" />
				<input type="hidden" name="subpage" value="racing" />
				<input type="hidden" name="league_id" value="<?php echo $league->id ?>" />
				<input type="hidden" name="season" value="<?php echo $season ?>" />
				<input type="hidden" name="match" value="<?php echo $match_id ?>" />
				
				<label for="team"><?php _e( 'Choose Team', 'leaguemanager' ) ?></label>
				<select size="1" name="team" id="team">
					<option value="" <?php selected('', $team_id) ?>><?php _e('Show all Teams', 'leaguemanager') ?>
				<?php foreach ( $team_list AS $t ) : ?>
					<option value="<?php echo $t->id ?>"<?php selected($t->id, $team_id) ?>><?php echo $t->title ?></option>
				<?php endforeach; ?>
				</select>
				<input type="submit" value="<?php _e('Filter', 'leaguemanager') ?>" class="button-secondary" />
			</form>
		</div>
		-->
		
		<?php if ( $leaguemanager->hasProjectManager() && $league->hasTeamRoster() ) : ?>
		<form action="" method="post">
		
		<div class="raceresults-blocks jquery-ui-accordion" id="">
			<?php foreach ($teams AS $team) : ?>
			<div id='raceresults_team_<?php echo $team->id ?>' class='raceresult-container'>
				<h2><?php echo $team->title ?></h2>
				
				<div class="raceresult-block content">
					<?php if ( $team->hasRoster() ) : ?>
					<table class="widefat wp-list-table leagues">
					<thead>
					<tr>
						<th scope="col" class="manage-column column-name column-primary"><?php _e( 'Name', 'leaguemanager' ) ?></th>
						<th scope="col" class="manage-column column-points"><?php _e( 'Points', 'leaguemanager' ) ?></th>
						<th scope="col" class="manage-column column-time"><?php _e( 'Time', 'leaguemanager' ) ?></th>
						<th scope="col" class="manage-column column-category"><?php _e( 'Category', 'leaguemanager' ) ?></th>
						<th scope="col" class="manage-column column-info"><?php _e( 'Other Info', 'leaguemanager' ) ?></th>
					</tr>
					</thead>
					<tbody id="the-list" class="form-table">
					<?php foreach ( $team->getRoster() AS $roster ) : $class = ( 'alternate' == $class ) ? '' : 'alternate'; ?>
					<?php if (!isset($match->raceresult[$roster->id])) $match->raceresult[$roster->id] = array('category' => '', 'points' => '', 'time' => '', 'info' => ''); ?>
					<?php if (isset($match->raceresult[$roster->id]['result'])) $match->raceresult[$roster->id]['time'] = $match->raceresult[$roster->id]['result']; ?>
					<tr class="<?php echo $class ?>">
						<td class="column-name column-primary" data-colname="<?php _e( 'Name', 'leaguemanager' ) ?>"><input type="hidden" name="racer[<?php echo $roster->id ?>]" value="<?php echo $roster->id ?>" /><input type="hidden" name="racer_name[<?php echo $roster->id ?>]" value="<?php echo $roster->name ?>" /><?php echo $roster->name ?></td>
						<td class="column-points" data-colname="<?php _e( 'Points', 'leaguemanager' ) ?>"><input type="text" name="points[<?php echo $roster->id ?>]" id="points_<?php echo $roster->id ?>" value="<?php echo $match->raceresult[$roster->id]['points'] ?>" /></td>
						<td class="column-time" data-colname="<?php _e( 'Time', 'leaguemanager' ) ?>"><input type="text" name="time[<?php echo $roster->id ?>]" id="time_<?php echo $roster->id ?>" value="<?php echo $match->raceresult[$roster->id]['time'] ?>" /></td>
						<td class="column-category" data-colname="<?php _e( 'Category', 'leaguemanager' ) ?>"><input type="text" name="category[<?php echo $roster->id ?>]" id="category_<?php echo $roster->id ?>" value="<?php echo $match->raceresult[$roster->id]['category'] ?>" /></td>
						<td class="column-info" data-colname="<?php _e( 'Other Info', 'leaguemanager' ) ?>"><input type="text" name="info[<?php echo $roster->id ?>]" id="info_<?php echo $roster->id ?>" value="<?php echo $match->raceresult[$roster->id]['info'] ?>" /></td>
					</tr>
					<?php endforeach; ?>
					</tbody>
					</table>
					<?php else : ?>
						<p class="error"><?php _e( 'No Team Roster found.', 'leaguemanager' ) ?></p>
					<?php endif; ?>
				</div>
			</div>
			<?php endforeach; ?>
		</div>
		
		<input type="hidden" name="match_id" value="<?php echo $match->id ?>" />
		
		<p class="submit"><input type="submit" name="save_results" value="<?php _e( 'Save Team Results', 'leaguemanager' ) ?>" class="button button-primary" /></p>
		
		</form>
		<?php endif; ?>
	</div>
	<?php
	}


	/**
	 * save race results for one team
	 *
	 * @param array $racer
	 * @param array $racer_name
	 * @param array $category
	 * @param array $points
	 * @param array $time
	 * @param array $info
	 * @param int $match_id
	 * @return true
	 */
	private function saveResults($racer, $racer_name, $category, $points, $time, $info, $match_id) {
		global $wpdb, $leaguemanager;

		$match = get_match( $match_id );
		$custom = $match->custom;

		$data = isset($custom['raceresult']) ? $custom['raceresult'] : array();
		while ( list($id) = each($racer) ) {
			$data[$id]['name'] = htmlspecialchars($racer_name[$id]);
			$data[$id]['category'] = $category[$id];
			$data[$id]['time'] = $time[$id];
			$data[$id]['points'] = intval($points[$id]);
			$data[$id]['info'] = htmlspecialchars($info[$id]);
		}
		
		$custom['raceresult'] = $data;

		$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->leaguemanager_matches} SET `custom` = '%s' WHERE `id` = '%d'", maybe_serialize($custom), $match_id ) );
		$leaguemanager->setMessage( __( 'Race Results Saved', 'leaguemanager' ) );
		return true;
	}
	
	
	/**
	 * get racing results for one racer
	 *
	 */
	/*function getRacerResults($teams)
	{
		global $leaguemanager; global $projectmanager;
		
		$team_roster = array();
		foreach ($teams AS $team) {
			if (isset($team->teamRoster) && is_array($team->teamRoster)) {
				foreach ($team->teamRoster AS $racer) {
					$team_roster[$racer->id] = array('project_id' => $team->roster['id'], 'racer_name' => $racer->name, 'team_id' => $team->id);
				}
			}
		}

		$url = get_permalink();
		$r = array();
		$keys = array();
		$points = array();
		$matches = $leaguemanager->getMatches( array("league_id" => $teams[0]->league_id, "season" => $teams[0]->season, "limit" => false) );
		foreach ($matches AS $match) {
			if (isset($match->raceresult)) {
				foreach ($match->raceresult AS $id => $result) {
					if (isset($team_roster[$id])) {
						if (!isset($points[$id])) $points[$id] = 0;
						$url = add_query_arg('show_'.$team_roster[$id]['project_id'], $id, $url);
						
						$r[$id]['points'] = $points[$id] + $result['points'];
						$r[$id]['name'] = $result['name'];
						$r[$id]['name_url'] = '<a href="'.$url.'">'.$result['name'].'</a>';
						$r[$id]['time'][] = $result['time'];
						$r[$id]['team_id'] = $team_roster[$id]['team_id'];
						$r[$id]['id'] = $id;
					
						$points[$id] = $points[$id] + $result['points'];
					}
				}
				arsort($points);
			}
		}
		
		$keys = array();
		foreach ($points AS $id => $pts) {
			$keys[] = $id;
		}

		$rank = 1; $class = ""; $i = 0;
		$racerlist = array();
		foreach ($points AS $id => $pts) {
			$racer = $r[$id];
			$racer['rank'] = $rank;
			$racer['class'] = ($class == 'alternate') ? '' : 'alternate';
			
			$team = $leaguemanager->getTeam($racer['team_id']);
			if ( 1 == $team->home ) $team->title = '<strong>'.$team->title.'</strong>';
			$racer['team_logo'] = $team->logo;
			$racer['team_logo_url'] = $leaguemanager->getThumbnailUrl($team->logo);
			
			if ( $team->website != '' )
				$racer['team_name'] = '<a href="http://'.$team->website.'" target="_blank">'.$team->title.'</a>';
            else
    			$racer['team_name'] = $team->title;
            
			$racerlist[$id] = $racer;
		
			if ($i > 0) {
				$key = $keys[$i-1];
			}
			$curr = $pts;
			
			$rank++;
			$i++;
		}
		
		return $racerlist;
	}*/
}
?>