<?php
/**
 * League_Soccer API: League_Soccer class
 *
 * @package LeagueManager
 * @subpackage League_Soccer
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_soccer' );
/**
 * add soccer to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_soccer( $sports ) {
	$sports['soccer'] = __( 'Soccer', 'leaguemanager' );
	return $sports;
}

 
/**
 * Class to implement the League_Soccer object
 *
 */
final class League_Soccer extends League {
	/**
	 * sports key
	 *
	 * @var string
	 */
	public $sport = 'soccer';


	/**
	 * load custom setings
	 *
	 * @param object $league League object
	 */
	public function __construct($league) {
		parent::__construct($league);
		
		$fields = array( 'halftime' => __( 'Halftime', 'leaguemanager' ), 'overtime' => __( 'Overtime', 'leaguemanager' ), 'penalty' => __( 'Penalty', 'leaguemanager' ) );
		foreach ( $fields AS $key => $label ) {			
			if ( $key == "halftime" )
				$keys = array('plus', 'minus');
			else
				$keys = array('home', 'away');
			
			$this->fields_match[$key] = array('label' => $label, 'keys' => $keys);
		}
		
		$this->fields_team['points2'] = array('label' => __( 'Goals', 'leaguemanager' ), 'keys' => array('plus', 'minus'), 'type' => 'input');
		$this->fields_team['diff'] = array('label' => __( 'Diff', 'leaguemanager' ), 'desc' => __('Goals Difference', 'leaguemanager'));
	}

	
	/**
	 * rank Teams
	 * 1. Primary points DESC
	 * 2. Secondary Point Difference DESC
	 * 3. Goals
	 * 4. Done Matches ASC
	 *
	 * @param array $teams
	 * @return array of teams
	 */
	protected function rankTeams( $teams ) {
		$points = $done = $diff = $goals = array();
		foreach ( $teams AS $key => $row ) {
			$points[$key] = $row->points['plus'];
			$diff[$key] = $row->diff;
			$goals[$key] = $row->points2['plus'];
			$done[$key] = $row->done_matches;
		}
		array_multisort( $points, SORT_DESC, $diff, SORT_DESC, $goals, SORT_DESC, $done, SORT_ASC, $teams );
		return $teams;
	}
}
?>