<?php
/**
 * League_Tabletennis API: League_Tabletennis class
 *
 * @package LeagueManager
 * @subpackage League_Tabletennis
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_tabletennis' );
/**
 * add tabletennis to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_tabletennis( $sports ) {
	$sports['tabletennis'] = __( 'Tabletennis', 'leaguemanager' );
	return $sports;
}

/**
 * Class to implement the League_Tabletennis object
 *
 */
final class League_Tabletennis extends League {
	/**
	 * sports key
	 *
	 * @var string
	 */
	public $sport = 'tabletennis';

	/**
	 * number of sets
	 *
	 * @var int
	 */
	public $num_sets = 5;
	
	
	/**
	 * load custom setings
	 *
	 * @param object $league League object
	 */
	public function __construct( $league ) {
		parent::__construct($league);
		
		if ( $this->num_sets > 1 ) {
			$this->fields_match['sets'] = array('label' => __( 'Sets', 'leaguemanager' ), 'keys' => array());
			for ( $i = 1; $i <= $this->num_sets; $i++ ) {
				$this->fields_match['sets']['keys'][$i] = array('home', 'away');
			}
		}
		
		add_filter( 'team_points_'.$this->sport, array(&$this, 'calculatePoints'), 10, 4 );
		add_filter( 'leaguemanager_point_rules_list', array(&$this, 'getPointRuleList') );
		add_filter( 'leaguemanager_point_rules',  array(&$this, 'getPointRules') );
		
		add_action( 'league_settings_'.$this->sport, array(&$this, 'leagueSettings') );
	}

	
	/**
	 * get Point Rule list
	 *
	 * @param array $rules
	 * @return array
	 */
	public function getPointRuleList( $rules ) {
		$rules[$this->sport] = __('Tabletennis', 'leaguemanager');

		return $rules;
	}


	/**
	 * get Point rules
	 *
	 * @param array $rules
	 * @return array
	 */
	public function getPointRules( $rules ) {
		$rules[$this->sport] = array( 'forwin' => 1, 'fordraw' => 0, 'forloss' => 0 );

		return $rules;
	}


	/**
	 * add league settings
	 *
	 * @param object $league
	 */
	public function leagueSettings( $league ) {
		echo "<tr valign='top'>";
		echo "<th scope='row'><label for='num_sets'>".__('Number of Sets', 'leaguemanager')."</label></th>";
		echo "<td><input type='text' name='settings[num_sets]' id='num_sets' value='".$league->num_sets."' size='3' /></td>";
		echo "</tr>";
	}
	
	
	/**
	 * calculate Points: add match score
	 *
	 * @param array $points
	 * @param int $team_id
	 * @param array $rule
	 * @param array $matches
	 */
	public function calculatePoints( $points, $team_id, $rule, $matches ) {
		$points = array( 'plus' => 0, 'minus' => 0 );
	
		foreach ( $matches AS $match ) {
			if ( $team_id == $match->home_team ) {
				$points['plus'] += $match->home_points;
				$points['minus'] += $match->away_points;
			} else {
				$points['plus'] += $match->away_points;
				$points['minus'] += $match->home_points;
			}
		}

		return $points;
	}
	
	
	/**
	 * update match results and automatically calculate apparatus points and score
	 *
	 * @param Match $match
	 * @return Match
	 */
	protected function updateResults( $match ) {
		$force = true;
		if (is_admin() && current_user_can('update_results')) {
			// update primary points
			if ( $force || $match->home_points == "" && $match->away_points == "" ) {
				$score = array( 'home' => 0, 'guest' => 0 );
				foreach ( $match->sets AS $set ) {
					if ( $set['home'] != '' && $set['away'] != '' ) {
						if ( $set['home'] > $set['away'] ) {
							$score['home'] += 1;
						} else {
							$score['guest'] += 1;
						}
					}
				}
				
				$match->home_points = $score['home'];
				$match->away_points = $score['guest'];
			}
		}
		
		return $match;
	}
}
?>