<?php
/**
 * League_USFootball API: League_USFootball class
 *
 * @package LeagueManager
 * @subpackage League_USFootball
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_usfootball' );
/**
 * add us-football to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_usfootball( $sports ) {
	$sports['USFootball'] = __( 'US Football', 'leaguemanager' );
	return $sports;
}

 
/**
 * Class to implement the League_USFootball object
 *
 */
final class League_USFootball extends League {
	/**
	 * sports key
	 *
	 * @var string
	 */
	public $sport = 'USFootball';

	/**
	 * us football types
	 *
	 * @var array
	 */
	private $keys = array();
	
	
	/**
	 * load custom setings
	 *
	 * @param object $league League object
	 */
	public function __construct($league) {
		parent::__construct($league);
		
		$this->keys = array( 'nfl' => __( 'US Football (NFL)', 'leaguemanager' ), 'ncaa' => __( 'US Collegiate Football (NCAA)', 'leaguemanager' ), 'hs' => __( 'US High School Football (HS)', 'leaguemanager' ), 'fb' => __( 'US Football (fb)', 'leaguemanager' ) );
		
		$fields = array( 'halftime' => __( 'Halftime', 'leaguemanager' ), 'overtime' => __( 'Overtime', 'leaguemanager' ), 'penalty' => __( 'Penalty', 'leaguemanager' ) );
		foreach ( $fields AS $key => $label ) {			
			if ( $key == "halftime" )
				$keys = array('plus', 'minus');
			else
				$keys = array('home', 'away');
			
			$this->fields_match[$key] = array('label' => $label, 'keys' => $keys);
		}
		
		$this->fields_team['points2'] = array('label' => __( 'Scores', 'leaguemanager' ), 'keys' => array('plus', 'minus'), 'type' => 'input');
		$this->fields_team['diff'] = array('label' => __( 'Diff', 'leaguemanager' ), 'desc' => __('Scores Difference', 'leaguemanager'));
		
		add_filter( 'leaguemanager_point_rules_list', array(&$this, 'getPointRuleList') );
		add_filter( 'leaguemanager_point_rules',  array(&$this, 'getPointRules') );
		
		add_filter( 'team_points_'.$this->sport, array(&$this, 'calculatePoints'), 10, 4 );
	}
	
	
	/**
	 * get Point Rule list
	 *
	 * @param array $rules
	 * @return array
	 */
	public function getPointRuleList( $rules ) {
		foreach ( $this->keys AS $key => $name )
			$rules[$key] = $name;

		return $rules;
	}


	/**
	 * get Point rules
	 *
	 * @param array $rules
	 * @return array
	 */
	public function getPointRules( $rules ) {
		// nfl rule
		$rules['nfl'] = array( 'forwin' => 1, 'fordraw' => 0, 'forloss' => 0, 'forwin_overtime' => 1, 'forloss_overtime' => 0, 'forscoring' => 0 );
		// ncaa rule
		$rules['ncaa'] = array( 'forwin' => 1, 'fordraw' => 0, 'forloss' => 0, 'forwin_overtime' => 1, 'forloss_overtime' => 0, 'forscoring' => 0 );
		// nfl rule
		$rules['hs'] = array( 'forwin' => 1, 'fordraw' => 0, 'forloss' => 0, 'forwin_overtime' => 1, 'forloss_overtime' => 0, 'forscoring' => 0 );
		// ncaa rule
		$rules['fb'] = array( 'forwin' => 1, 'fordraw' => 0, 'forloss' => 0, 'forwin_overtime' => 1, 'forloss_overtime' => 0, 'forscoring' => 0 );
		
		return $rules;
	}
	
	/**
	 * rank Teams
	 * 1. Win Percentage
	 * 2. Secondary points differenc
	 * 3. Primary points
	 *
	 * @param array $teams
	 * @return array of teams
	 */
	protected function rankTeams( $teams ) {
		$points = $diff = $WinPerc = array();
		foreach ( $teams AS $key => $row ) {
			$points[$key] = $row->points['plus'];
			$diff[$key] = $row->diff;
            $WinPerc[$key] = $row->winPercent;
		}

		array_multisort( $WinPerc, SORT_DESC, SORT_NUMERIC, $diff, SORT_DESC, $points, SORT_DESC, $teams );
		return $teams;
	}


	/**
	 * Re-calculate primary points
	 *
	 * @param array $points
	 * @param int $team_id
	 * @param array $rule
	 * @param array $matches
	 * @return array
	 */
	public function calculatePoints( $points, $team_id, $rule, $matches ) {
		extract($rule);
		
		$num_won_overtime = $num_lost_overtime = 0;
		foreach ( $matches AS $match ) {
			// match with overtime
			if ( isset($match->overtime) && $match->overtime['home'] != '' && $match->overtime['away'] != '') {
				if ( $team_id == $match->winner_id )
					$num_won_overtime += 1;
				
				if ( $team_id == $match->loser_id )
					$num_lost_overtime += 1;
			}
		}
		
		$points['plus'] = $points['plus'] - $num_won_overtime * $forwin + $num_won_overtime * $forwin_overtime + $num_lost_overtime * $forloss_overtime;
		$points['minus'] = $points['minus'] - $num_lost_overtime * $forwin + $num_won_overtime * $forloss_overtime + $num_lost_overtime * $forwin_overtime;
	
		return $points;
	}
}
?>