<?php
/**
 * Template tags
 */


/**
 * get league ID
 *
 * @return int
 * @category template-tags
 */
function get_league_id() {
	global $league;
	return $league->id;
}
/**
 * print league ID
 *
 * @category template-tags
 */
function the_league_id() {
	echo get_league_id();
}

/**
 * get league title
 *
 * @return string
 * @category template-tags
 */
function get_league_title() {
	global $league;
	return $league->title;
}
/**
 * print league title
 *
 * @category template-tags
 */
function the_league_title() {
	echo get_league_title();
}

/**
 * get current season
 *
 * @return string
 * @category template-tags
 */
function get_current_season() {
	global $league;
	return $league->current_season['name'];
}
/**
 * print current season
 *
 * @category template-tags
 */
function the_current_season() {
	echo get_current_season();
}

/**
 * get current archive tab
 *
 * @return string
 * @category template-tags
 */
function the_league_archive_tab() {
	global $league;
	echo $league->archive_tab;
}

/**
 * check if we are in a league archive
 *
 * @return boolean
 * @category template-tags
 */
function is_league_archive() {
	global $league;
	return $league->is_archive;
}

/**
 * get specific template
 *
 * @param string $template
 * @return string
 * @category template-tags
 */
function get_league_template($template = "" ) {
	global $league;
	
	if (!empty($template) && isset($league->templates[$template]))
		return $league->templates[$template];
	
	return "";
}

/**
 * get current match day
 *
 * @return int
 * @category template-tags
 */
function get_current_match_day() {
	global $league;
	return $league->match_day;
}
/**
 * print current match day
 *
 * @category template-tags
 */
function the_current_match_day() {
	echo get_current_match_day();
}

/**
 * get number of match days
 *
 * @return int
 * @category template-tags
 */
function get_num_match_days() {
	global $league;
	return $league->num_match_days;
}

/**
 * get current Team ID
 *
 * @return int
 * @category template-tags
 */
function get_selected_team_id() {
	global $league;
	return $league->current_team;
}

/**
 * get current Match ID
 *
 * @return int
 * @category template-tags
 */
function get_selected_match_id() {
	global $league;
	return $league->current_match;
}

/**
 * check if a specific standings columns is activated for display
 *
 * @param string $key
 * @return boolean
 * @category template-tags
 */
function show_standings($key) {
	global $league;
	
	if (isset($league->standings[$key]) && $league->standings[$key] == 1)
		return true;
	
	return false;
}


/**
 * get League sport type
 *
 * @return string
 * @category template-tags
 */
function get_league_sport() {
	global $league;
	return $league->sport;
}

/**
 * get League point rule
 *
 * @return string
 * @category template-tags
 */
function get_league_pointrule() {
	global $league;
	return $league->point_rule;
}


/**
 * check if crosstable is popup
 *
 * @return boolean
 * @category template-tags
 */
function is_crosstable_popup() {
	global $league;
	return $league->crosstable_mode == 'popup';
}


/**
 * check if logos are shown
 *
 * @return boolean
 * @category template-tags
 */
function league_show_logo() {
	global $league;
	return $league->show_logo == 1;
}


/**
 * get total number of teams
 *
 * @return int
 * @category template-tags
 */
function get_num_teams_total() {
	global $league;
	return $league->num_teams_total;
}

/**
 * display standings header
 *
 * @category template-tags
 */
function the_standings_header() {
	global $league;
	$league->displayStandingsHeader();
}
/**
 * display standings columns
 *
 * @category template-tags
 */
function the_standings_columns() {
	global $league, $team;
	$league->displayStandingsColumns($team, get_league_pointrule());
}


/**
 * test whether league has teams or we are in the loop
 *
 * @return boolean
 */
function have_teams() {
	global $league;

	if ( $league->current_team + 1 < count($league->teams) ) {
		return true;
	} elseif ( $league->current_team == count($league->teams)-1 && count($league->teams) > 0 ) {
		// End of Loop
		$league->current_team = -1;	
	}
	
	$league->in_the_team_loop = false;
	return false;
}
/**
 * loop through teams
 *
 */
function the_team() {
	global $league, $team;
	
	$league->in_the_team_loop = true;
	
	// Loop start
	if ( $league->current_team == -1 ) {
		
	}
	
	// Increment team count
	$league->current_team++;
	$team = $league->teams[$league->current_team];
}

/**
 * check if team is selected
 *
 * @return boolean
 * @category template-tags
 */
function is_team() {
	global $league;
	return $league->is_selected_team;
}

/**
 * display single team
 *
 * @param string $template
 * @return void
 */
function the_single_team( $template = "" ) {
	global $league;
	echo do_shortcode("[team id='".$league->current_team."' template='".$template."']");
}

/**
 * get team ID
 *
 * @return int
 * @category template-tags
 */
function get_team_id() {
	global $team;
	return $team->id;
}
/**
 * print team ID
 *
 * @category template-tags
 */
function the_team_id() {
	echo get_team_id();
}

/**
 * get team name
 *
 * @return string
 * @category template-tags
 */
function get_team_name() {
	global $team;
	return $team->title;
}
/**
 * print team name
 *
 * @category template-tags
 */
function the_team_name() {
	echo get_team_name();
}

/**
 * print team name URL
 *
 * @param string $url
 * @category template-tags
 */
function the_team_name_url($url = true) {
	global $team;
	
	if ( $url )
		echo '<a href="'.get_team_url().'">'.get_team_name().'</a>';
	else
		the_team_name();
}

/**
 * print team coach
 *
 * @category template-tags
 */
function the_team_coach() {
	global $team;
	echo $team->coach;
}

/**
 * get team website
 *
 * @category template-tags
 */
function the_team_website() {
	global $team;
	echo esc_url($team->website);
}

/**
 * get team logo URL
 *
 * @param string $size
 * @return string
 * @category template-tags
 */
function get_team_logo_url($size = 'full') {
	global $team;
	
	return $team->getImageURL($team->logo, $size);
}
/**
 * print team logo URL
 *
 * @param string $size
 * @category template-tags
 */
function the_team_logo_url($size = 'full') {
	echo get_team_logo_url($size);
}
/**
 * print team logo image
 *
 * @param string $size
 * @category template-tags
 */
function the_team_logo($size = 'full') {
	global $team;
	
	if (!empty($team->logo))
		echo '<img src="'.get_team_logo_url($size).'" alt="'.__( 'Logo', 'leaguemanager' ).'" />';
}

/**
 * print team CSS class
 *
 * @category template-tags
 */
function the_team_class() {
	global $team;
	echo $team->class;
}

/**
 * get team URL
 *
 * @return string
 * @category template-tags
 */
function get_team_url() {
	global $team;
	return $team->pageURL;
}
/**
 * print team URL
 *
 * @category template-tags
 */
function the_team_url() {
	echo get_team_url();
}

/**
 * check if team has specific field
 *
 * @param string $key
 * @return boolean
 * @category template-tags
 */
function team_has( $key ) {
	global $team;
	
	if (!empty($team->$key))
		return true;
	
	return false;
}

/**
 * get team rank
 *
 * @return int
 * @category template-tags
 */
function get_team_rank() {
	global $team;
	return $team->rank;
}
/**
 * print team rank
 *
 * @category template-tags
 */
function the_team_rank() {
	echo get_team_rank();
}

/**
 * print team status
 *
 * @category template-tags
 */
function the_team_status() {
	global $team;
	echo $team->status;
}

/**
 * print formatted team points
 *
 * @param string $ind
 * @category template-tags
 */
function the_team_points($ind = "primary") {
	global $team;
	echo $team->pointsFormatted[$ind];
}

/**
 * print number of done matches of team
 *
 * @category template-tags
 */
function num_done_matches() {
	global $team;
	echo $team->done_matches;
}
/**
 * print number of won matches of team
 *
 * @category template-tags
 */
function num_won_matches() {
	global $team;
	echo $team->won_matches;
}
/**
 * print number of lost matches of team
 *
 * @category template-tags
 */
function num_lost_matches() {
	global $team;
	echo $team->lost_matches;
}
/**
 * print number of draw matches of team
 *
 * @category template-tags
 */
function num_draw_matches() {
	global $team;
	echo $team->draw_matches;
}
/**
 * print win percentage
 *
 * @category template-tags
 */
function win_percentage() {
	global $team;
	echo $team->winPercent;
}

/**
 * check if team has a next match
 *
 * @return boolean
 * @category template-tags
 */
function has_next_match() {
	global $team, $match;
	
	$match = $team->getNextMatch();
	
	if ($match)
		return true;
	
	return false;
}
/**
 * check if team has a previous match
 *
 * @return boolean
 * @category template-tags
 */
function has_prev_match() {
	global $team, $match;
	
	$match = $team->getPrevMatch();
	
	if ($match)
		return true;
	
	return false;
}

/**
 * print last5 matches column for team
 *
 * @category template-tags
 */
function the_last5_matches() {
	global $team;
	echo $team->last5();
}

/**
 * check if team roster is present
 *
 * @return boolean
 * @category template-tags
 */
function team_roster_exists() {
	global $leaguemanager;
	return $leaguemanager->hasProjectManager();
}

/**
 * check if team has tabs, i.e. roster & profile
 *
 * @return boolean
 * @category template-tags
 */
function has_roster_tabs() {
	global $team;
	return $team->projects_tabs;
}

/**
 * check if team has roster
 *
 * @return boolean
 * @category template-tags
 */
function has_roster() {
	global $team;
	return $team->hasRoster();
}

/**
 * check if team has profile
 *
 * @return boolean
 * @category template-tags
 */
function has_profile() {
	global $team;
	return $team->hasProfile();
}

/**
 * print team roster
 *
 * @param string $template
 * @category template-tags
 */
function the_team_roster($template = 'gallery') {
	global $league, $team;
	
	project( $league->teamroster, array('cat_id' => $team->roster, 'template' => $template, 'show_title' => false, 'selections' => false, 'searchform' => false) );
}
/**
 * print team profile
 *
 * @param string $template
 * @category template-tags
 */
function the_team_profile($template = 'leaguemanager-teamprofile') {
	global $league, $team, $project;
	$project = get_project($league->teamprofiles);
	dataset( $team->profile, array('template' => $template) );
}

/**
 * print team profile data to incorporate into team profile list
 *
 * @param array $args
 * @category template-tags
 */
function the_team_profile_data( $args = array() ) {
	global $team;
	$defaults = array( 'output' => 'dl', 'show_all' => true, 'show_dataset_image' => false, 'show_header_image' => false, 'show_dataset_name' => false );
	
	$dataset = get_dataset( $team->profile );
	if ( $dataset )
		$dataset->printData( array_merge( $defaults, $args ) );
}

/**
 * print teams pagination
 *
 * @param string $start_el
 * @param string $end_el
 * @category template-tags
 */
function the_teams_pagination($start_el = "<p class='leaguemanager-pagination page-numbers'>", $end_el = "</p>") {
	global $league;
	
	if ( !empty($league->pagination_teams) )
		echo $start_el . $league->pagination_teams . $end_el;
}

 
/**
 * test whether league has matches or we are in the loop
 *
 * @return boolean
 */
function have_matches() {
	global $league;

	if ( $league->current_match + 1 < count($league->matches) ) {
		return true;
	} elseif ( $league->current_match == count($league->matches)-1 && count($league->matches) > 0 ) {
		// End of Loop
		$league->current_match = -1;	
	}
	
	$league->in_the_match_loop = false;
	return false;
}
/**
 * loop through matches
 *
 */
function the_match() {
	global $league, $match;
	
	$league->in_the_match_loop = true;
	
	// Loop start
	if ( $league->current_match == -1 ) {
		
	}
	
	// Increment dataset count
	$league->current_match++;
	$match = $league->matches[$league->current_match];
}

/**
 * check if match is selected
 *
 * @return boolean
 * @category template-tags
 */
function is_single_match() {
	global $league;
	return $league->is_selected_match;
}

/**
 * display single match
 *
 * @param string $template
 */
function the_single_match( $template = "" ) {
	global $league;
	echo do_shortcode("[match id='".$league->current_match."' template='".$template."']");
}

/**
 * print matches pagination
 *
 * @param string $start_el
 * @param string $end_el
 * @category template-tags
 */
function the_matches_pagination($start_el = "<p class='leaguemanager-pagination page-numbers'>", $end_el = "</p>") {
	global $league;
	
	if ( !empty($league->pagination_matches) )
		echo $start_el . $league->pagination_matches . $end_el;
}

/**
 * get Match ID
 *
 * @return int
 * @category template-tags
 */
function get_match_id() {
	global $match;
	return $match->id;
}
/**
 * print Match ID
 *
 * @category template-tags
 */
function the_match_id() {
	echo get_match_id();
}

/**
 * print Match CSS class
 *
 * @category template-tags
 */
function the_match_class() {
	global $match;
	echo $match->class;
}

/**
 * print Match title
 *
 * @param boolean $show_logo
 * @category template-tags
 */
function the_match_title($show_logo = true) {
	global $match;
	
	echo $match->getTitle($show_logo);
}

/**
 * get Match day
 *
 * @return int
 * @category template-tags
 */
function get_match_day() {
	global $match;
	return $match->match_day;
}
/**
 * print Match day
 *
 * @category template-tags
 */
function the_match_day() {
	echo get_match_day();
}

/**
 * print Match date
 *
 * @param string $format
 * @category template-tags
 */
function the_match_date($format = '') {
	global $match;
	
	if ($format == '') 
		echo $match->match_date;
	else
		echo mysql2date($format, $match->date);
}

/**
 * print Match time
 *
 * @category template-tags
 */
function the_match_time() {
	global $match;
	echo $match->start_time;
}

/**
 * print Match location
 *
 * @category template-tags
 */
function the_match_location() {
	global $match;
	echo $match->location;
}

/**
 * get Match score
 *
 * @return string
 * @category template-tags
 */
function get_match_score() {
	global $match;
	return $match->score;
}
/**
 * print Match score
 *
 * @category template-tags
 */
function the_match_score() {
	echo get_match_score();
}

/**
 * get Match status
 *
 * @return string
 * @category template-tags
 */
function get_match_status() {
	global $match;
	return $match->status;
}
/**
 * print Match status
 *
 * @param null|Match $match
 * @category template-tags
 */
function the_match_status($match = null) {
	if (is_null($match)) global $match;
	// only print match status if it's not scheduled or done
	if ( !in_array($match->status, array('scheduled', 'done')) ) {
		echo "<p class='match-status ".$match->status."'>".$match->getStatusOptions($match->status)."</p>";
	}
}

/**
 * check if match is finished
 *
 * @return boolean
 * @category template-tags
 */
function match_is_finished() {
	if (get_match_score() == "-:-")
		return false;
	
	return true;
}

/**
 * print Match URL
 *
 * @category template-tags
 */
function the_match_url() {
	global $match;
	echo $match->pageURL;
}

/**
 * check if match has report
 *
 * @return boolean
 * @category template-tags
 */
function match_has_report() {
	global $match;
	
	if ($match->post_id != 0)
		return true;
	
	return false;
}
/**
 * print Match report link
 *
 * @category template-tags
 */
function the_match_report() {
	global $match;
	echo $match->report;
}

/**
 * get match logo URL
 *
 * @param string $index 'home' or 'away'
 * @param string $size
 * @return string
 * @category template-tags
 */
function get_match_logo_url($index = "home", $size="thumb") {
	global $match;
	
	if (isset($match->teams[$index]))
		return $match->teams[$index]->getImageURL($match->teams[$index]->logo, $size);
	else
		return '';
}
/**
 * get match logo image
 *
 * @param string $index 'home' or 'away'
 * @param string $size
 * @category template-tags
 */
function the_match_logo($index = "home", $size = "thumb") {
	global $match;
	
	if (isset($match->teams[$index]) && !empty($match->teams[$index]->logo))
		echo '<img src="'.get_match_logo_url($index, $size).'" alt="" />';
}

/**
 * get match template type
 *
 * @return string
 * @category template-tags
 */
function get_match_template_type() {
	global $league;
	return $league->matches_template_type;
}

/**
 * print match CSS class for list
 *
 * @category template-tags
 */
function the_matchlist_class() {
	if (get_match_template_type() == 'accordion')
		echo 'jquery-ui-accordion';
	elseif (get_match_template_type() == 'tabs')
		echo 'jquery-ui-tabs';
}
/**
 * print match container CSS class for jQuery UI Tabs
 *
 * @category template-tags
 */
function the_matchbox_class() {
	if (get_match_template_type() == 'tabs')
		echo 'jquery-ui-tab';
}
/**
 * print match header CSS class for jQuery UI Tabs & Accordion
 *
 * @return string
 * @category template-tags
 */
function the_matchbox_header_class() {
	if (in_array(get_match_template_type(), array('tabs', 'accordion')))
		echo 'header';
}
/**
 * print match content CSS class for jQuery UI Tabs & Accordion
 *
 * @return string
 * @category template-tags
 */
function the_matchbox_content_class() {
	if (in_array(get_match_template_type(), array('tabs', 'accordion')))
		echo 'match-content';
}

/**
 * print crosstable field
 *
 * @param int $i
 * @category template-tags
 */
function the_crosstable_field($i) {
	global $league, $team;
	
	echo $league->getCrosstableField($team->id, $league->teams[$i-1]->id, $team->home);
}


/*
 * wrapper tags
 */
 
/**
 * display widget 
 *
 * @param int $number a unique number
 * @param array $instance widget instance parameters
 * @category template-tags
 */
function leaguemanager_display_widget( $number, $instance ) {
	echo "<ul id='leaguemanger-widget-".$instance['league']."' class='leaguemanager_widget'>";
	$widget = new LeagueManagerWidget(true);
	$widget->widget( array('number' => $number), $instance );
	echo "</ul>";
}


/**
 * display next match box
 *
 * @param int $number
 * @param array $instance widget instance parameters
 * @category template-tags
 */
function leaguemanager_display_next_match_box( $number, $instance ) {
	$widget = new LeagueManagerWidget(true);
	$widget->showNextMatchBox( $number, $instance );
}


/**
 * display previous match box
 *
 * @param int $number
 * @param array $instance widget instance parameters
 * @category template-tags
 */
function leaguemanager_display_prev_match_box( $number, $instance ) {
	$widget = new LeagueManagerWidget(true);
	$widget->showPrevMatchBox( $number, $instance );
}


/**
 * display standings table
 *
 * @param int $league_id League ID
 * @param array $args associative array of parameters, see default values (optional)
 * @category template-tags
 */
function leaguemanager_standings( $league_id = 0, $args = array() ) {
	global $league;

	$defaults = array( 'season' => false, 'template' => '', 'logo' => 'true', 'group' => false, 'home' => 0 );
	$args = array_merge($defaults, $args);
	$args['league_id'] = $league_id;
	
	$shortcode = "[standings";
	foreach ($args AS $key => $value)
		$shortcode .= " ".$key."='".$value."'";
	$shortcode .= "]";
	echo do_shortcode($shortcode);
}


/**
 * display crosstable table
 *
 * @param int $league_id
 * @param array $args associative array of parameters, see default values (optional)
 * @category template-tags
 */
function leaguemanager_crosstable( $league_id = 0, $args = array() ) {
	global $league;
	
	$defaults = array('season' => false, 'logo' => 'true', 'group' => '', 'template' => '', 'mode' => '');
	$args = array_merge($defaults, $args);
	$args['league_id'] = $league_id;
	
	$shortcode = "[crosstable";
	foreach ($args AS $key => $value)
		$shortcode .= " ".$key."='".$value."'";
	$shortcode .= "]";
	echo do_shortcode($shortcode);
}


/**
 * display matches table
 *
 * @param int $league_id
 * @param array $args associative array of parameters, see default values (optional)
 * @category template-tags
 */
function leaguemanager_matches( $league_id = 0, $args = array() ) {
	global $league;
	
	$defaults = array('season' => '', 'template' => '', 'mode' => '', 'limit' => 'true', 'match_day' => -1, 'group' => false, 'roster' => false, 'order' => false, 'show_match_day_selection' => '', 'show_team_selection' => '', 'time' => '', 'team' => 0, 'home_only' => 'false', 'match_date' => false, 'dateformat' => '', 'timeformat' => '');
	$args = array_merge($defaults, $args);
	$args['league_id'] = $league_id;
	
	$shortcode = "[matches";
	foreach ($args AS $key => $value)
		$shortcode .= " ".$key."='".$value."'";
	$shortcode .= "]";
	echo do_shortcode($shortcode);
}


/**
 * display one match
 *
 * @param int $match_id
 * @param array $args additional arguments as associative array (optional)
 * @category template-tags
 */
function leaguemanager_match( $match_id, $args = array() ) {
	$defaults = array('template' => '');
	$args = array_merge($defaults, $args);
	$args['id'] = $match_id;
	
	$shortcode = "[match";
	foreach ($args AS $key => $value)
		$shortcode .= " ".$key."='".$value."'";
	$shortcode .= "]";
	echo do_shortcode($shortcode);
}


/**
 * display team list
 *
 * @param int|string $league_id
 * @param array $args additional arguments as associative array (optional)
 * @category template-tags
 */
function leaguemanager_teams( $league_id = 0, $args = array() ) {
	global $league;
	
	$defaults = array('season' => false, 'template' => '', 'group' => false);
	$args = array_merge($defaults, $args);
	$args['league_id'] = intval($league_id);
	
	$shortcode = "[teams";
	foreach ($args AS $key => $value)
		$shortcode .= " ".$key."='".$value."'";
	$shortcode .= "]";
	echo do_shortcode($shortcode);
}


/**
 * display one team
 *
 * @param int $team_id
 * @param array $args additional arguments as associative array (optional)
 * @category template-tags
 */
function leaguemanager_team( $team_id, $args = array() ) {
	$defaults = array('template' => '');
	$args = array_merge($defaults, $args);
	$args['id'] = $team_id;
	
	$shortcode = "[team";
	foreach ($args AS $key => $value)
		$shortcode .= " ".$key."='".$value."'";
	$shortcode .= "]";
	echo do_shortcode($shortcode);
}


/**
 * display championship
 *
 * @param int $league_id
 * @param array $args additional arguments as associative array (optional)
 * @category template-tags
 */
function leaguemanager_championship( $league_id = 0, $args = array() ) {
	global $league;
	
	$defaults = array('template' => '', 'season' => false);
	$args = array_merge($defaults, $args);
	$args['league_id'] = $league_id;
	
	$shortcode = "[championship";
	foreach ($args AS $key => $value)
		$shortcode .= " ".$key."='".$value."'";
	$shortcode .= "]";
	echo do_shortcode($shortcode);
}


/**
 * display league archive
 *
 * @param int $league_id
 * @param array $args additional arguments as associative array (optional)
 * @category template-tags
 */
function leaguemanager_archive( $league_id, $args = array() ) {
	$defaults = array('template' => '');
	$args = array_merge($defaults, $args);
	$args['league_id'] = $league_id;
	
	$shortcode = "[leaguearchive";
	foreach ($args AS $key => $value)
		$shortcode .= " ".$key."='".$value."'";
	$shortcode .= "]";
	echo do_shortcode($shortcode);
}


/**
 * display league
 *
 * @param int $league_id
 * @param array $args additional arguments as associative array (optional)
 * @category template-tags
 */
function leaguemanager_league( $league_id, $args = array() ) {
	$defaults = array('season' => false, 'template' => '');
	$args = array_merge($defaults, $args);
	$args['league_id'] = $league_id;
	
	$shortcode = "[league";
	foreach ($args AS $key => $value)
		$shortcode .= " ".$key."='".$value."'";
	$shortcode .= "]";
	echo do_shortcode($shortcode);
}
?>